package gov.va.med.ccht.model.car;

import gov.va.med.ccht.model.ReportConstants;
import gov.va.med.fw.ui.model.TermType;
import gov.va.med.fw.util.DateUtils;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Map;

public class CensusActivityRecord implements Serializable 
{
	
	private static final long serialVersionUID = -7932072567700379138L;
	private static final String NO_REPORT = "(No Report)";
	private static final String IN_PROGRESS = "(In Progress)";
	

	
	private static final String AA = "AA";
	private static final String AR = "AR";
	private static final String IP = "IP";
	
	
	private List<TermType> reportWeek = null;
	private String modalityName = null;
	private Integer total = null;
	private Integer nationalTotal = null;
	private List<VendorCensusReportStatus> vendorCensusReportStatus = null;
	private Map<String, VendorCensusReportStatus> vendorCensusReportStatusAsMap = null;
	
	


	private boolean hasATIReported = false;
	private boolean hasAuthentidateReported = false;
	private boolean hasCardiocomReported = false;
	private boolean hasHealthHeroReported = false;
	private boolean hasVitelNetReported = false;
	private boolean hasViterionReported = false;
	
	private Date reportStartDate = null; 
	private Date reportEndDate = null; 
	private String lastDayOfReportWeek = null;
	
	public String getLastDayOfReportWeek() 
	{
		return lastDayOfReportWeek;
	}

	public void setLastDayOfReportWeek(String lastDayOfReportWeek) 
	{
		this.lastDayOfReportWeek = lastDayOfReportWeek;
	}
	
	public void setLastDayOfReportWeek() 
	{		
		GregorianCalendar cal = new GregorianCalendar();  
		cal.setTime(reportEndDate);
		cal.add(Calendar.DATE, 1);
		
		SimpleDateFormat format = new SimpleDateFormat(DateUtils.MMDDYYYY);			
		lastDayOfReportWeek = format.format(cal.getTime());
	}


	
	public List<TermType> getReportWeek() 
	{
		if(reportWeek != null)
		{
			return reportWeek;
		}
		else
		{
			return new ArrayList<TermType>();
		}
	}

	public void setReportWeek(List<TermType> reportWeek) {
		this.reportWeek = reportWeek;
	}
	
	public String getModalityName() 
	{
		if(modalityName != null)
		{
			return modalityName;
		}
		else
		{
			return "No Modality Assigned";
		}
	}
	
	public void setModalityName(String modalityName) 
	{
		this.modalityName = modalityName;
	}
	
	public Integer getTotal() 
	{
		if(total != null)
		{
			return total;
		}
		else
		{
			return 0;
		}
	}
	
	public void setTotal(Integer total) 
	{
		this.total = total;
	}
	
	public Integer getNationalTotal() 
	{
		if(nationalTotal != null)
		{
			return nationalTotal;
		}
		else
		{
			return 0;
		}
	}
	
	public void setNationalTotal(Integer nationalTotal) 
	{
		this.nationalTotal = nationalTotal;
	}
	
	public List<VendorCensusReportStatus> getVendorCensusReportStatus() 
	{
		VendorCensusReportStatus ati = vendorCensusReportStatusAsMap.get(ReportConstants.ATI);
		VendorCensusReportStatus authentidate =  vendorCensusReportStatusAsMap.get(ReportConstants.AUTHENTIDATE);
		VendorCensusReportStatus cardiocom = vendorCensusReportStatusAsMap.get(ReportConstants.CARDIOCOM);
		VendorCensusReportStatus healthHero = vendorCensusReportStatusAsMap.get(ReportConstants.HEALTH_HERO);
		VendorCensusReportStatus vitelNet = vendorCensusReportStatusAsMap.get(ReportConstants.VITEL_NET);
		VendorCensusReportStatus viterion = vendorCensusReportStatusAsMap.get(ReportConstants.VITERION);
		vendorCensusReportStatus = new ArrayList<VendorCensusReportStatus>();
		
		if(ati != null)
		{
		    //if(!ati.getAckType().equalsIgnoreCase(AA))
			vendorCensusReportStatus.add(ati);
		}
//		else
//		{
//		    vendorCensusReportStatus.add(buildCensusStatusReport(ReportConstants.ATI));
//		}
		
		if(authentidate != null)
		{
		    //if(!authentidate.getAckType().equalsIgnoreCase(AA))
			vendorCensusReportStatus.add(authentidate);
		}
//		else
//		{
//		    vendorCensusReportStatus.add(buildCensusStatusReport(ReportConstants.AUTHENTIDATE));
//		}
		
		if(cardiocom != null)
		{
		    //if(!cardiocom.getAckType().equalsIgnoreCase(AA))
			vendorCensusReportStatus.add(cardiocom);
		}
//		else
//		{
//		    vendorCensusReportStatus.add(buildCensusStatusReport(ReportConstants.CARDIOCOM));
//		}

		if(healthHero != null)
		{
		    //if(!healthHero.getAckType().equalsIgnoreCase(AA))
			vendorCensusReportStatus.add(healthHero);
		}
//		else
//		{
//		    vendorCensusReportStatus.add(buildCensusStatusReport(ReportConstants.HEALTH_HERO));
//		}

		if(vitelNet != null)
		{
		    //if(!vitelNet.getAckType().equalsIgnoreCase(AA))
			vendorCensusReportStatus.add(vitelNet);
		}
//		else
//		{
//		    vendorCensusReportStatus.add(buildCensusStatusReport(ReportConstants.VITEL_NET));
//		}

		if(viterion != null)
		{
		    //if(!viterion.getAckType().equalsIgnoreCase(AA))
			vendorCensusReportStatus.add(viterion);
		}
//		else
//		{
//		    vendorCensusReportStatus.add(buildCensusStatusReport(ReportConstants.VITERION));
//		}

		return vendorCensusReportStatus;
	}
	
	private VendorCensusReportStatus buildCensusStatusReport(String vendor)
	{
	    VendorCensusReportStatus retVal =  new VendorCensusReportStatus();
	    
	    retVal.setVendorName(vendor + " (No Report)"); 
	    retVal.setEndDate(reportEndDate);
	    retVal.setStartDate(reportStartDate);
	    retVal.setAckType(AR);
	    
	    return retVal;
	}
	

	public void setVendorCensusReportStatus(List<VendorCensusReportStatus> vendorCensusReportStatus) 
	{
		this.vendorCensusReportStatus = vendorCensusReportStatus;
	}
		
	public Map<String, VendorCensusReportStatus> getVendorCensusReportStatusAsMap() 
	{
		return vendorCensusReportStatusAsMap;
	}

	public void setVendorCensusReportStatusAsMap(Map<String, VendorCensusReportStatus> vendorCensusReportStatusAsMap) 
	{
		this.vendorCensusReportStatusAsMap = vendorCensusReportStatusAsMap;
	}

	public Date getReportStartDate() 
	{
		return reportStartDate;
	}


	public void setReportStartDate(Date reportStartDate) 
	{
		this.reportStartDate = reportStartDate;
	}

	
	public Date getReportEndDate() 
	{
		return reportEndDate;
	}

	
	public void setReportEndDate(Date reportEndDate) 
	{
		this.reportEndDate = reportEndDate;		
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (hasATIReported ? 1231 : 1237);
		result = prime * result + (hasAuthentidateReported ? 1231 : 1237);
		result = prime * result + (hasCardiocomReported ? 1231 : 1237);
		result = prime * result + (hasHealthHeroReported ? 1231 : 1237);
		result = prime * result + (hasVitelNetReported ? 1231 : 1237);
		result = prime * result + (hasViterionReported ? 1231 : 1237);
		result = prime * result
				+ ((modalityName == null) ? 0 : modalityName.hashCode());
		result = prime * result
				+ ((nationalTotal == null) ? 0 : nationalTotal.hashCode());
		result = prime * result
				+ ((reportEndDate == null) ? 0 : reportEndDate.hashCode());
		result = prime * result
				+ ((reportStartDate == null) ? 0 : reportStartDate.hashCode());
		result = prime * result
				+ ((reportWeek == null) ? 0 : reportWeek.hashCode());
		result = prime * result + ((total == null) ? 0 : total.hashCode());
		result = prime
				* result
				+ ((vendorCensusReportStatus == null) ? 0
						: vendorCensusReportStatus.hashCode());
		result = prime
				* result
				+ ((vendorCensusReportStatusAsMap == null) ? 0
						: vendorCensusReportStatusAsMap.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CensusActivityRecord other = (CensusActivityRecord) obj;
		if (hasATIReported != other.hasATIReported)
			return false;
		if (hasAuthentidateReported != other.hasAuthentidateReported)
			return false;
		if (hasCardiocomReported != other.hasCardiocomReported)
			return false;
		if (hasHealthHeroReported != other.hasHealthHeroReported)
			return false;
		if (hasVitelNetReported != other.hasVitelNetReported)
			return false;
		if (hasViterionReported != other.hasViterionReported)
			return false;
		if (modalityName == null) {
			if (other.modalityName != null)
				return false;
		} else if (!modalityName.equals(other.modalityName))
			return false;
		if (nationalTotal == null) {
			if (other.nationalTotal != null)
				return false;
		} else if (!nationalTotal.equals(other.nationalTotal))
			return false;
		if (reportEndDate == null) {
			if (other.reportEndDate != null)
				return false;
		} else if (!reportEndDate.equals(other.reportEndDate))
			return false;
		if (reportStartDate == null) {
			if (other.reportStartDate != null)
				return false;
		} else if (!reportStartDate.equals(other.reportStartDate))
			return false;
		if (reportWeek == null) {
			if (other.reportWeek != null)
				return false;
		} else if (!reportWeek.equals(other.reportWeek))
			return false;
		if (total == null) {
			if (other.total != null)
				return false;
		} else if (!total.equals(other.total))
			return false;
		if (vendorCensusReportStatus == null) {
			if (other.vendorCensusReportStatus != null)
				return false;
		} else if (!vendorCensusReportStatus
				.equals(other.vendorCensusReportStatus))
			return false;
		if (vendorCensusReportStatusAsMap == null) {
			if (other.vendorCensusReportStatusAsMap != null)
				return false;
		} else if (!vendorCensusReportStatusAsMap
				.equals(other.vendorCensusReportStatusAsMap))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "CensusActivityRecord [hasATIReported=" + hasATIReported
				+ ", hasAuthentidateReported=" + hasAuthentidateReported
				+ ", hasCardiocomReported=" + hasCardiocomReported
				+ ", hasHealthHeroReported=" + hasHealthHeroReported
				+ ", hasVitelNetReported=" + hasVitelNetReported
				+ ", hasViterionReported=" + hasViterionReported
				+ ", modalityName=" + modalityName + ", nationalTotal="
				+ nationalTotal + ", reportEndDate=" + reportEndDate
				+ ", reportStartDate=" + reportStartDate + ", reportWeek="
				+ reportWeek + ", total=" + total
				+ ", vendorCensusReportStatus=" + vendorCensusReportStatus
				+ ", vendorCensusReportStatusAsMap="
				+ vendorCensusReportStatusAsMap + "]";
	}
	
	
	

	

}
