package gov.va.med.ccht.model.coc;

import gov.va.med.ccht.model.ReportConstants;

import java.io.Serializable;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

public class CategoryOfCareResult implements Serializable
{
    private static final int NIC = 1;
    private static final int CHRONIC_CARE = 2;
    private static final int ACUTE_CARE = 3;
    private static final int HEALTH_PROMO_DISEASE_PREVENTION = 4;

    private static final long serialVersionUID = -5730561371681587809L;
    private List<CategoryOfCareRecord> records = null;
    private Map<String, List<CategoryOfCareRecord>> recordsKeyedByVendor = null;
    private List<CategoryOfCareRow> categoryOfCareRows = null;

    public List<CategoryOfCareRow> getCategoryOfCareRows()
    {
	return categoryOfCareRows;
    }

    public void setCategoryOfCareRows(List<CategoryOfCareRow> categoryOfCareRows)
    {
	this.categoryOfCareRows = categoryOfCareRows;
    }

    public void buildCategoryOfCareRows()
    {
	categoryOfCareRows = new ArrayList<CategoryOfCareRow>();

	CategoryOfCareRow row = getCategoryOfCareRow(ReportConstants.ATI);
	if(row != null)
	{
	    categoryOfCareRows.add(row);
	}
	
	row = getCategoryOfCareRow(ReportConstants.AUTHENTIDATE);
	if(row != null)
	{
	    categoryOfCareRows.add(row);
	}
	
	row = getCategoryOfCareRow(ReportConstants.CARDIOCOM);
	if(row != null)
	{
	    categoryOfCareRows.add(row);
	}
	
	row = getCategoryOfCareRow(ReportConstants.HEALTH_HERO);
	if(row != null)
	{
	    categoryOfCareRows.add(row);
	}
	
	row = getCategoryOfCareRow(ReportConstants.VITEL_NET);
	if(row != null)
	{
	    categoryOfCareRows.add(row);
	}
	
	row = getCategoryOfCareRow(ReportConstants.VITERION);
	if(row != null)
	{
	    categoryOfCareRows.add(row);
	}

    }
    
    public void buildCategorOfCareTotalsRow()
    {
	if(categoryOfCareRows == null || categoryOfCareRows.size() == 0)
	{
	    return;
	}
	    
	CategoryOfCareRow row = new CategoryOfCareRow();
	row.setVendorName("Totals by Category");
	for(CategoryOfCareRow currentRow : categoryOfCareRows)
	{
	    int nicTotal = currentRow.getNicTotal();
	    int chronicCareTotal = currentRow.getChronicCareTotal();
	    int acuteCareTotal = currentRow.getAcuteCareTotal();
	    int healthPromoTotal = currentRow.getHealthPromoTotal();
	    int lowAcuityTotal = currentRow.getLowAcuityTotal();
	    
	    
	    
	    nicTotal = row.getNicTotal() + nicTotal;
	    chronicCareTotal = row.getChronicCareTotal() + chronicCareTotal;
	    acuteCareTotal = row.getAcuteCareTotal() + acuteCareTotal;
	    healthPromoTotal = row.getHealthPromoTotal() + healthPromoTotal;
	    lowAcuityTotal = row.getLowAcuityTotal() + lowAcuityTotal;
	    
	    
	    row.setNicTotal(nicTotal);
	    row.setChronicCareTotal(chronicCareTotal);
	    row.setAcuteCareTotal(acuteCareTotal);
	    row.setHealthPromoTotal(healthPromoTotal);
	    row.setLowAcuityTotal(lowAcuityTotal);
	    
	}
	row.setTotalsByVendor();
	
	
	
	int overAllVendorTotal = row.getTotalByVendor();
	CategoryOfCareRow percentageRow = new CategoryOfCareRow();
	percentageRow.setVendorName("Percent of Total");
	int nicTotal = row.getNicTotal();
	int chronicCareTotal = row.getChronicCareTotal();
	int acuteCareTotal = row.getAcuteCareTotal();
	int healthPromoTotal = row.getHealthPromoTotal();
	int lowAcuityTotal = row.getLowAcuityTotal();

	percentageRow.setColumnOne(getPercentOfTotal(nicTotal, overAllVendorTotal));
	percentageRow.setColumnTwo(getPercentOfTotal(chronicCareTotal, overAllVendorTotal));
	percentageRow.setColumnThree(getPercentOfTotal(acuteCareTotal, overAllVendorTotal));
	percentageRow.setColumnFour(getPercentOfTotal(healthPromoTotal, overAllVendorTotal));
	percentageRow.setColumnFive(getPercentOfTotal(lowAcuityTotal, overAllVendorTotal));
	percentageRow.setColumnSix(row.getColumnSix());

	row.setColumnSix("TOTAL PTS CATEGORIZED");
	categoryOfCareRows.add(row);
	categoryOfCareRows.add(percentageRow);
	
    }
    
    private String getPercentOfTotal(int total, int overAllTotal)
    {
	String retVal = "";
	Double dbl = (((double)total)/overAllTotal)*100;
	DecimalFormat df = new DecimalFormat("####0.0");
	retVal = df.format(dbl) + "%";
	
	return retVal;
    }
    
    
    public CategoryOfCareRow getCategoryOfCareRow(String vendorName)
    {
	List<CategoryOfCareRecord> currentRow = recordsKeyedByVendor.get(vendorName);
	CategoryOfCareRow row = null;
	
	if(currentRow != null)
	{
	    row = new CategoryOfCareRow();
		row.setVendorName(vendorName);
		for (CategoryOfCareRecord record : currentRow)
		{
		    int recordId = record.getId();

		    switch (recordId)
		    {
		    case NIC:
			row.setNicTotal(record.getNumResults());
			break;
		    case CHRONIC_CARE:
			row.setChronicCareTotal(record.getNumResults());
			break;
		    case ACUTE_CARE:
			row.setAcuteCareTotal(record.getNumResults());
			break;
		    case HEALTH_PROMO_DISEASE_PREVENTION:
			row.setHealthPromoTotal(record.getNumResults());
			break;
		    }
		}
		row.setTotalsByVendor();
	}

	
	return row;
    }

    public void setRecordsKeyedByVendor()
    {
	if (records != null)
	{
	    recordsKeyedByVendor = new TreeMap<String, List<CategoryOfCareRecord>>();

	    for (CategoryOfCareRecord currentRecord : records)
	    {
		String vendor = currentRecord.getVendorName();
		List<CategoryOfCareRecord> temp = recordsKeyedByVendor.get(vendor);

		if (temp == null)
		{
		    temp = new ArrayList<CategoryOfCareRecord>();
		}

		temp.add(currentRecord);
		recordsKeyedByVendor.put(vendor, temp);
	    }
	}
    }

    public List<CategoryOfCareRecord> getRecords()
    {
	return records;
    }

    public void setRecords(List<CategoryOfCareRecord> records)
    {
	this.records = records;
    }

    @Override
    public int hashCode()
    {
	final int prime = 31;
	int result = 1;
	result = prime * result + ((records == null) ? 0 : records.hashCode());
	return result;
    }

    @Override
    public boolean equals(Object obj)
    {
	if (this == obj) return true;
	if (obj == null) return false;
	if (getClass() != obj.getClass()) return false;
	CategoryOfCareResult other = (CategoryOfCareResult) obj;
	if (records == null)
	{
	    if (other.records != null) return false;
	}
	else if (!records.equals(other.records)) return false;
	return true;
    }

    @Override
    public String toString()
    {
	return "CategoryOfCareResult [records=" + records + "]";
    }
}
