package gov.va.med.ccht.model.responserate;

import gov.va.med.ccht.model.ReportConstants;
import gov.va.med.ccht.model.coc.CategoryOfCareRecord;
import gov.va.med.ccht.model.coc.CategoryOfCareRow;

import java.io.Serializable;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

public class ResponseRateResults implements Serializable
{
    private static final long serialVersionUID = 5329731773474249792L;
    
    private static final int RESPONDERS = 1;
    private static final int PARTIAL_RESPONDERS = 2;
    private static final int NON_RESPONDERS = 3;
    private static final int UNKOWN = 4;
    private static final int DATE_COLS = 5;
    
    private List<ResponseRateRow> responseRateRows = null;
    List<ResponseRateRecord> responseRateRecords = null;
    Map<String, List<ResponseRateRecord>> recordsKeyedByVendor = null;

    public List<ResponseRateRecord> getResponseRateRecords()
    {
	return responseRateRecords;
    }

    public void setResponseRateRecords(List<ResponseRateRecord> responseRateRecords)
    {
	this.responseRateRecords = responseRateRecords;
    }

    public List<ResponseRateRow> getResponseRateRows()
    {
	return responseRateRows;
    }

    public void setResponseRateRows(List<ResponseRateRow> responseRateRows)
    {
	this.responseRateRows = responseRateRows;
    }

    public void setTotalsRow()
    {
	if (responseRateRows == null || responseRateRows.size() == 0)
	{
	    return;
	}

	ResponseRateRow row = new ResponseRateRow();
	row.setVendorName("Totals by RESPONSE RATE");
	for (ResponseRateRow currentRow : responseRateRows)
	{
	    int numResponders = currentRow.getNumResponders();
	    int partialResponders = currentRow.getPartialResponders();
	    int nonResponders = currentRow.getNonResponders();
	    int unknown = currentRow.getUnknown();
	    int total = currentRow.getTotal();
	    int thirtyDaysNoResponse = currentRow.getThirtyDaysNonResponse();
	    int greatestDaysNoResponse = currentRow.getGreatestDaysNonResponse();

	    numResponders = row.getNumResponders() + numResponders;
	    partialResponders = row.getPartialResponders() + partialResponders;
	    nonResponders = row.getNonResponders() + nonResponders;
	    unknown = row.getUnknown() + unknown;
	    total = row.getTotal() + total;
	    thirtyDaysNoResponse = row.getThirtyDaysNonResponse() + thirtyDaysNoResponse;
	    greatestDaysNoResponse = row.getGreatestDaysNonResponse() + greatestDaysNoResponse;

	    row.setNumResponders(numResponders);
	    row.setPartialResponders(partialResponders);
	    row.setNonResponders(nonResponders);
	    row.setUnknown(unknown);
	    row.setTotal(total);
	    row.setColumnFive("TOTAL PTS REPORTED");
	    row.setColumnSix("");
	    row.setColumnSeven("");

	}
	
	
	int overAllVendorTotal = row.getTotal();
	ResponseRateRow percentageRow = new ResponseRateRow();
	percentageRow.setVendorName("Percent of Total");
	int numRespondersTotal = row.getNumResponders();
	int partialRespondersTotal = row.getPartialResponders();
	int nonRespondersTotal = row.getNonResponders();
	int unknownTotal = row.getUnknown();
	
	percentageRow.setColumnOne(getPercentOfTotal(numRespondersTotal, overAllVendorTotal));
	percentageRow.setColumnTwo(getPercentOfTotal(partialRespondersTotal, overAllVendorTotal));
	percentageRow.setColumnThree(getPercentOfTotal(nonRespondersTotal, overAllVendorTotal));
	percentageRow.setColumnFour(getPercentOfTotal(unknownTotal, overAllVendorTotal));
	
	percentageRow.setTotal(overAllVendorTotal);
	percentageRow.setColumnSix("");
	percentageRow.setColumnSeven("");
	
	responseRateRows.add(row);
	responseRateRows.add(percentageRow);
	
    }
    
    private String getPercentOfTotal(int total, int overAllTotal)
    {
	String retVal = "";
	Double dbl = (((double)total)/overAllTotal)*100;
	DecimalFormat df = new DecimalFormat("####0.0");
	retVal = df.format(dbl) + "%";
	
	return retVal;
    }

    public void setRecordsKeyedByVendor()
    {
	if (responseRateRecords != null)
	{
	    recordsKeyedByVendor = new TreeMap<String, List<ResponseRateRecord>>();

	    for (ResponseRateRecord currentRecord : responseRateRecords)
	    {
		String vendor = currentRecord.getVendorName();
		List<ResponseRateRecord> temp = recordsKeyedByVendor.get(vendor);

		if (temp == null)
		{
		    temp = new ArrayList<ResponseRateRecord>();
		}

		temp.add(currentRecord);
		recordsKeyedByVendor.put(vendor, temp);
	    }
	}
    }

    public void buildCategoryOfCareRows()
    {
	responseRateRows = new ArrayList<ResponseRateRow>();

	ResponseRateRow row = getResponseRateRow(ReportConstants.ATI);
	if (row != null)
	{
	    responseRateRows.add(row);
	}

	row = getResponseRateRow(ReportConstants.AUTHENTIDATE);
	if (row != null)
	{
	    responseRateRows.add(row);
	}

	row = getResponseRateRow(ReportConstants.CARDIOCOM);
	if (row != null)
	{
	    responseRateRows.add(row);
	}

	row = getResponseRateRow(ReportConstants.HEALTH_HERO);
	if (row != null)
	{
	    responseRateRows.add(row);
	}

	row = getResponseRateRow(ReportConstants.VITEL_NET);
	if (row != null)
	{
	    responseRateRows.add(row);
	}

	row = getResponseRateRow(ReportConstants.VITERION);
	if (row != null)
	{
	    responseRateRows.add(row);
	}

    }

    public ResponseRateRow getResponseRateRow(String vendorName)
    {
	List<ResponseRateRecord> currentRow = recordsKeyedByVendor.get(vendorName);
	ResponseRateRow row = null;

	if (currentRow != null)
	{
	    row = new ResponseRateRow();
	    row.setVendorName(vendorName);
	    for (ResponseRateRecord record : currentRow)
	    {
		int responseCode = record.getResponseCode();

		switch (responseCode)
		{
		case RESPONDERS:
		    row.setNumResponders(record.getPatientCount());
		    break;
		case PARTIAL_RESPONDERS:
		    row.setPartialResponders(record.getPatientCount());
		    break;
		case NON_RESPONDERS:
		    row.setNonResponders(record.getPatientCount());
		    break;
		case UNKOWN:
		    row.setUnknown(record.getPatientCount());
		    break;
		case DATE_COLS:
		    row.setThirtyDaysNonResponse(record.getPatientCount());
		    row.setGreatestDaysNonResponse(record.getMaxDataAge());
		    break;
		}
	    }
	    row.setTotal();
	}

	return row;
    }

}
