package gov.va.med.ccht.persistent.hibernate;

import gov.va.med.ccht.model.coc.CategoryOfCareRecord;
import gov.va.med.ccht.model.coc.CategoryOfCareReportParameters;
import gov.va.med.ccht.model.coc.CategoryOfCareResult;
import gov.va.med.ccht.persistent.CategoryOfCareReportDAO;
import gov.va.med.fw.persistent.DAOException;
import gov.va.med.fw.persistent.hibernate.GenericDAOImpl;
import gov.va.med.fw.persistent.hibernate.QueryAndParams;
import gov.va.med.fw.ui.model.TermType;
import gov.va.med.fw.util.DateUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

public class CategoryOfCareReportDAOImpl extends GenericDAOImpl implements CategoryOfCareReportDAO
{

    @Override
    public CategoryOfCareResult getNationalReportAllVendors(CategoryOfCareReportParameters reportParams)
	    throws DAOException
    {
	CategoryOfCareResult result = new CategoryOfCareResult();
	QueryAndParams queryAndParams = getNationalReportAllVendorsQueryAndParams(reportParams);
		
	result = getNationalReport(queryAndParams);
	return result;
    }
    
    @Override
    public CategoryOfCareResult getNationalReportByVendor(CategoryOfCareReportParameters reportParams)
	    throws DAOException
    {
	CategoryOfCareResult result = new CategoryOfCareResult();
	QueryAndParams queryAndParams = getNationalReportByVendorQueryAndParams(reportParams);
		
	result = getNationalReport(queryAndParams);
	return result;
    }
    
    private CategoryOfCareResult getNationalReport(QueryAndParams queryAndParams)throws DAOException
    {
	CategoryOfCareResult result = new CategoryOfCareResult();
	List<CategoryOfCareRecord> records = new ArrayList<CategoryOfCareRecord>();
	List<?> report = executeSQLQuery(queryAndParams);
	int numIterations = report.size();
	
	for (int j = 0; j < numIterations; j++)
	{
	    CategoryOfCareRecord newRecord = new CategoryOfCareRecord();
	    Object[] record = (Object[])report.get(j);
	    newRecord.setVendorName((String)record[0]);
	    newRecord.setId((Integer)record[1]);
	    newRecord.setCategoryName((String)record[2]);
	    newRecord.setNumResults((Integer)record[3]);
	    records.add(newRecord);
	}
	
	if(numIterations > 0)
	{
	    result.setRecords(records);
	    result.setRecordsKeyedByVendor();
	    result.buildCategoryOfCareRows();
	    result.buildCategorOfCareTotalsRow();
	}

	return result;
    }
    
    private QueryAndParams getNationalReportAllVendorsQueryAndParams(CategoryOfCareReportParameters reportParams)
    {
	QueryAndParams retVal = new QueryAndParams();
	StringBuilder sql = new StringBuilder();
	sql.append("SELECT DISTINCT c.vendor_name, d.id, d.category_name, COUNT(distinct patient_id) AS [Num] ");
	sql.append("FROM census a, censuspatient b, vendor c, levelofcare d WHERE ");
	sql.append("a.census_id=b.census_id ");
	sql.append("AND a.report_start_date= :reportStartDate ");
	sql.append("AND a.vendor_id=c.vendor_id ");
	sql.append("AND b.level_of_care=d.id ");
	sql.append("AND (b.disenrollment_date IS NULL OR b.disenrollment_date >= :reportStartDate) ");
	sql.append("GROUP BY c.vendor_name, d.id, d.category_name ");
	sql.append("ORDER BY c.vendor_name, d.id ASC");
	
			
	String reportStartDate = getReportStartDate(reportParams.getReportStartDate());
	
	retVal.append(sql.toString());
	retVal.addParam("reportStartDate", reportStartDate);
	
	return retVal;
    }

    
    private QueryAndParams getNationalReportByVendorQueryAndParams(CategoryOfCareReportParameters reportParams)
    {
	QueryAndParams retVal = new QueryAndParams();
	StringBuilder sql = new StringBuilder();
	sql.append("SELECT DISTINCT c.vendor_name, d.id, d.category_name, COUNT(distinct patient_id) AS [Num] ");
	sql.append("FROM census a, censuspatient b, vendor c, levelofcare d WHERE ");
	sql.append("a.census_id=b.census_id ");
	sql.append("AND a.report_start_date= :reportStartDate ");
	sql.append("AND a.vendor_id=c.vendor_id ");
	sql.append("AND b.level_of_care=d.id ");
	sql.append("AND (b.disenrollment_date IS NULL OR b.disenrollment_date >= :reportStartDate) ");
	sql.append("AND c.vendor_id=:vendorId ");	 
	sql.append("GROUP BY c.vendor_name, d.id, d.category_name ");
	sql.append("ORDER BY c.vendor_name, d.id ASC");
	TermType vendor = reportParams.getVendor();
	String reportStartDate = getReportStartDate(reportParams.getReportStartDate());
	retVal.append(sql.toString());
	retVal.addParam("reportStartDate", reportStartDate);
	retVal.addParam("vendorId", vendor.getValue());
	
	
	return retVal;
    }

    private String getReportStartDate(Date reportEndDate)
    {
	String startDate = null;
	
	GregorianCalendar gc = new GregorianCalendar();
	gc.setTime(reportEndDate);
	
	gc.set(Calendar.DATE, (gc.get(Calendar.DATE) - 6 ));
	SimpleDateFormat format = new SimpleDateFormat(DateUtils.MMDDYYYY);			
	startDate = format.format(gc.getTime());
	return startDate;
    }
}
