package gov.va.med.ccht.persistent.hibernate;

import gov.va.med.ccht.model.car.CensusActivityRecord;
import gov.va.med.ccht.model.car.CensusActivityReportParameters;
import gov.va.med.ccht.model.car.CensusActivityReportResult;
import gov.va.med.ccht.model.car.CensusActivitySubTotalRecord;
import gov.va.med.ccht.model.car.CensusActivityVisnRecord;
import gov.va.med.ccht.model.car.VendorCensusReportStatus;
import gov.va.med.ccht.persistent.CensusActivityReportDAO;
import gov.va.med.fw.persistent.DAOException;
import gov.va.med.fw.persistent.hibernate.GenericDAOImpl;
import gov.va.med.fw.persistent.hibernate.QueryAndParams;
import gov.va.med.fw.ui.model.TermType;
import gov.va.med.fw.util.DateUtils;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


import org.springframework.transaction.annotation.Transactional;

public class CensusActivityReportDAOImpl extends GenericDAOImpl implements
		CensusActivityReportDAO {
	public final static String TOTAL_PATIENTS = "Total Patients";
	public final static String MESSAGING_TOTAL = "Messaging Total";
	public final static String MESSAGING_BY_DEVICE = "Messaging By Device";
	public final static String MESSAGING_BY_IVR = "Messaging By IVR";
	public final static String MESSAGING_BY_BROWSER = "Messaging By Browser";
	public final static String MESSAGING_UNKOWN = "Messaging (IVR, device, or browser not yet assigned)";
	public final static String PERIPHERAL_MEASUREMENTS = "Peripheral Measurements";
	public final static String VIDEO = "Video";
	public final static String CELLULAR_MODEM_TRANSMISSION = "Cellular Modem Transmission";
	public final static String SATELLITE_TRANSMISSION = "Satellite Transmission";
	public final static String DIGITAL_IMAGE_CAPTURE = "Digital Image Capture";
	
	public final static String COMPLETED = "completed";

	
	final int total_patients = 1;
	final int messaging_total = 2;
	final int device = 3;
	final int ivr = 4;
	final int browser = 5;
	final int unknown = 6;
	final int measurements = 7;
	final int video = 8;
	final int cell = 9;
	final int satellite = 10;
	final int digital = 11;

	@Transactional
	public CensusActivityRecord getAllVendorsReportStatus(String arg)
			throws DAOException {
		CensusActivityRecord retStatus = new CensusActivityRecord();
		
		GregorianCalendar reportEndDateAsCal = getReportEndDateAsCal();
		String reportEndDate = getCalendarAsString(reportEndDateAsCal);
		
		GregorianCalendar reportStartDateAsCal = getReportStartDateAsCal(new GregorianCalendar(reportEndDateAsCal.get(Calendar.YEAR), reportEndDateAsCal.get(Calendar.MONTH), reportEndDateAsCal.get(Calendar.DAY_OF_MONTH)));		
		String reportStartDate = getCalendarAsString(reportStartDateAsCal);
 
		retStatus.setReportStartDate(reportStartDateAsCal.getTime());
		retStatus.setReportEndDate(reportEndDateAsCal.getTime());
	
		retStatus.setLastDayOfReportWeek(reportEndDate);
		
		Map<String, VendorCensusReportStatus> activeVendorsForReportingWeek = getActiveVendorsForReportWeek(reportEndDate, reportStartDate);
		Map<String, VendorCensusReportStatus> activeVendorsStatusForReportingWeek = getActiveVendorsStatusForReportWeek(reportEndDate, reportStartDate);
		
		Map<String, VendorCensusReportStatus> combinedVendorStatusMaps = combineVendorStatusMaps(activeVendorsForReportingWeek, activeVendorsStatusForReportingWeek);
			
		Map<String, VendorCensusReportStatus> vendorsNotCompleted = removeCompletedVendors(combinedVendorStatusMaps, reportStartDateAsCal.getTime(), reportEndDateAsCal.getTime());
		retStatus.setVendorCensusReportStatusAsMap(vendorsNotCompleted);
		return retStatus;

	}
	
	
	/**
	 * 
	 * @param reportEndDate
	 * @param reportStartDate
	 * @return
	 * @throws DAOException
	 */
	public Map<String, VendorCensusReportStatus> getActiveVendorsStatusForReportWeek(String reportEndDate, String reportStartDate) throws DAOException
	{	
		Map<String, VendorCensusReportStatus> vendors = new HashMap<String, VendorCensusReportStatus>();
		StringBuilder sql = new StringBuilder();
		sql.append("SELECT Vendor.Vendor_Name, Vendor.Date_active,Vendor.Date_Inactive, COUNT(*) AS [PatientCount], ");
		sql.append("CASE WHEN HL7Activity.AckType IS NULL THEN '??' ELSE HL7Activity.AckType END AS [AckType], ");
		sql.append("CASE WHEN (HL7Activity.AckType = 'AA') AND (COUNT(*) > 0) THEN 'Completed' WHEN (HL7Activity.AckType is null) AND (COUNT(*) > 0) THEN 'In progress' ");
		sql.append("WHEN (HL7Activity.AckType <> 'AA') AND (COUNT(*) = 0) THEN 'Errors' ELSE 'Unknown' END AS [Status] ");
		sql.append("FROM Census INNER JOIN HL7Activity ON HL7Activity.ID = Census.HL7ActivityID ");
		sql.append("INNER JOIN CensusPatient ON Census.Census_ID = CensusPatient.Census_ID INNER JOIN Vendor ON Census.Vendor_ID = Vendor.Vendor_ID ");
		sql.append("WHERE (Census.Report_Start_Date >= :reportStartDate AND Census.Report_End_Date <= :reportEndDate) ");
		sql.append("AND (Vendor.Date_Active is not null OR Vendor.Date_Active <= :reportStartDate) ");
		sql.append("AND (Vendor.Date_Inactive is NULL OR (Vendor.Date_InActive >= :reportStartDate AND Vendor.Date_InActive <= :reportEndDate)) ");
		sql.append("GROUP BY Vendor.Vendor_Name, Vendor.Date_active, Vendor.Date_Inactive, HL7Activity.AckType ORDER BY Vendor.vendor_name ");
		
		
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.append(sql.toString());
		queryAndParams.addParam("reportStartDate", reportStartDate);
		queryAndParams.addParam("reportEndDate", reportEndDate);
	
		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();
		
		
		for (int j = 0; j < numIterations; j++) 
		{
			Object[] record = (Object[]) report.get(j);
			String vendorName = (String)record[0];
			Integer patientCount = (Integer)record[3];
			String ackType = (String)record[4];
			String status = (String)record[5];
		
			VendorCensusReportStatus currentRecord = new VendorCensusReportStatus();
			currentRecord.setVendorName(vendorName);
			currentRecord.setPatientCount(patientCount);
			currentRecord.setAckType(ackType);
			currentRecord.setEndDateAsString(reportEndDate);
			currentRecord.setStartDateAsString(reportStartDate);			
			currentRecord.setStatus(status);
			
			vendors.put(vendorName, currentRecord);
		}
		
		return vendors;
	}
	
	public Map<String, VendorCensusReportStatus> getActiveVendorsForReportWeek(String reportEndDate, String reportStartDate) throws DAOException
	{	
		Map<String, VendorCensusReportStatus> activeVendors = new HashMap<String, VendorCensusReportStatus>();
		StringBuilder sql = new StringBuilder();
		sql.append("SELECT vendor_name, date_active, date_inactive, -1 as PatientCount, '??' as AckType, 'No report' as Status FROM Vendor ");
		sql.append("WHERE (Date_Active is not null OR Date_Active <= :reportStartDate) ");
		sql.append("AND (Date_Inactive is NULL OR (Date_InActive >= :reportStartDate AND Date_InActive <= :reportEndDate)) ");
		sql.append("ORDER BY vendor_name");
		
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.append(sql.toString());
		queryAndParams.addParam("reportStartDate", reportStartDate);
		queryAndParams.addParam("reportEndDate", reportEndDate);
	
		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();
		
		
		for (int j = 0; j < numIterations; j++) 
		{
			Object[] record = (Object[]) report.get(j);
			String vendorName = (String)record[0];
			Integer patientCount = (Integer)record[3];
			String ackType = (String)record[4];
			String status = (String)record[5];
		
			VendorCensusReportStatus currentRecord = new VendorCensusReportStatus();
			currentRecord.setVendorName(vendorName);
			currentRecord.setPatientCount(patientCount);
			currentRecord.setAckType(ackType);
			currentRecord.setEndDateAsString(reportEndDate);
			currentRecord.setStartDateAsString(reportStartDate);
			currentRecord.setStatus(status);
			
			activeVendors.put(vendorName, currentRecord);
		}
		return activeVendors;
	}
	
	
	public Map<String, VendorCensusReportStatus> getNonReportingVendors(String reportEndDate, GregorianCalendar cal) throws DAOException
	{	
		StringBuilder sql = new StringBuilder();
		sql.append("select Vendor.Vendor_Name from Vendor where Vendor.Date_Inactive is null and Vendor.Vendor_ID NOT IN  ");
		sql.append("(select census.Vendor_ID from census where census.Report_End_Date = :reportEndDate)");
		
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.append(sql.toString());
		queryAndParams.addParam("reportEndDate", reportEndDate);
	
		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();
		Map<String, VendorCensusReportStatus> vendorCensusReportStatusMap = new HashMap<String, VendorCensusReportStatus>();
		
		Date startDate = cal.getTime();
		
		cal.set(Calendar.DATE, (cal.get(Calendar.DATE) + 6));
		Date endDate = cal.getTime();
		
		
		for (int j = 0; j < numIterations; j++) 
		{
			//Object[] record = (Object[]) report.get(j);
			String vendorName = (String) report.get(j);
			
			
			VendorCensusReportStatus newRecord = new VendorCensusReportStatus();
			newRecord.setVendorName(vendorName + " (No Report)");
			newRecord.setStartDate(startDate);
			newRecord.setEndDate(endDate);
			newRecord.setDateLoaded(null);
			newRecord.setAckType("AR");
			vendorCensusReportStatusMap.put(vendorName, newRecord);			
		}
		
		return vendorCensusReportStatusMap;
		
	}

	public CensusActivityRecord getLastReportWeek() throws DAOException {
		CensusActivityRecord lastReportWeekActivityRecord = new CensusActivityRecord();
		List<TermType> dates = new ArrayList<TermType>();
		String sql = "SELECT DISTINCT Rpt_Wk,CONVERT(varchar,Rpt_wk,101) AS [StrDate] FROM facility_activity ORDER BY Rpt_Wk DESC";
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.append(sql);

		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();

		for (int j = 0; j < numIterations; j++) {
			Object[] record = (Object[]) report.get(j);
			String dateAsString = (String) record[1];
			TermType date = new TermType();
			date.setLabel(dateAsString);
			date.setValue(dateAsString);
			dates.add(date);
		}
		lastReportWeekActivityRecord.setReportWeek(dates);

		return lastReportWeekActivityRecord;
	}

	@Override
	public void getFacilityReport(CensusActivityReportParameters reportParams)
			throws DAOException {
		// TODO Auto-generated method stub
		

	}

	@Override
	public List<CensusActivityRecord> getNationalReport(
			CensusActivityReportParameters reportParams) throws DAOException {
		List<CensusActivityRecord> returnResults = new ArrayList<CensusActivityRecord>();
		String sql = getNationalSQL();
		String weekEndingDate = reportParams.getReportDate(); // DateUtils.format(reportParams.getWeekEndingDate(),
																// DateUtils.MMDDYYYY);
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.append(sql);
		queryAndParams.addParam("weekEndingDate", weekEndingDate);

		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();

		for (int j = 0; j < numIterations; j++) {
			Object[] record = (Object[]) report.get(j);

			CensusActivityRecord tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Total Patients");
			tempRecord.setTotal((Integer) record[1]);
			tempRecord.setNationalTotal((Integer) record[1]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Messaging Total");
			tempRecord.setTotal((Integer) record[2]);
			tempRecord.setNationalTotal((Integer) record[2]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Messaging By Device");
			tempRecord.setTotal((Integer) record[3]);
			tempRecord.setNationalTotal((Integer) record[3]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Messaging By IVR");
			tempRecord.setTotal((Integer) record[4]);
			tempRecord.setNationalTotal((Integer) record[4]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Messaging By Browser");
			tempRecord.setTotal((Integer) record[5]);
			tempRecord.setNationalTotal((Integer) record[5]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord
					.setModalityName("Messaging (IVR, device or browser not yet assigned)");
			tempRecord.setTotal((Integer) record[6]);
			tempRecord.setNationalTotal((Integer) record[6]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Peripheral Measurements");
			tempRecord.setTotal((Integer) record[7]);
			tempRecord.setNationalTotal((Integer) record[7]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Video");
			tempRecord.setTotal((Integer) record[8]);
			tempRecord.setNationalTotal((Integer) record[8]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Cellular Modem Transmission");
			tempRecord.setTotal((Integer) record[9]);
			tempRecord.setNationalTotal((Integer) record[9]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Satellite Transmission");
			tempRecord.setTotal((Integer) record[10]);
			tempRecord.setNationalTotal((Integer) record[10]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Digital Image Capture");
			tempRecord.setTotal((Integer) record[11]);
			tempRecord.setNationalTotal((Integer) record[11]);
			returnResults.add(tempRecord);
		}
		return returnResults;
	}

	@Override
	public List<CensusActivityRecord> getNationalReportByVendor(
			CensusActivityReportParameters reportParams) throws DAOException {
		List<CensusActivityRecord> returnResults = new ArrayList<CensusActivityRecord>();
		String sql = getNationalSQLByVendor(reportParams.getVendor());
		String weekEndingDate = DateUtils.format(reportParams
				.getWeekEndingDate(), DateUtils.MMDDYYYY);
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.append(sql);
		queryAndParams.addParam("weekEndingDate", weekEndingDate);
		queryAndParams.addParam("vendorId", reportParams.getVendor()
				.getValue());

		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();

		for (int j = 0; j < numIterations; j++) {
			Object[] record = (Object[]) report.get(j);

			CensusActivityRecord tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Total Patients");
			tempRecord.setTotal((Integer) record[1]);
			tempRecord.setNationalTotal((Integer) record[1]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Messaging Total");
			tempRecord.setTotal((Integer) record[2]);
			tempRecord.setNationalTotal((Integer) record[2]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Messaging By Device");
			tempRecord.setTotal((Integer) record[3]);
			tempRecord.setNationalTotal((Integer) record[3]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Messaging By IVR");
			tempRecord.setTotal((Integer) record[4]);
			tempRecord.setNationalTotal((Integer) record[4]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Messaging By Browser");
			tempRecord.setTotal((Integer) record[5]);
			tempRecord.setNationalTotal((Integer) record[5]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord
					.setModalityName("Messaging (IVR, device or browser not yet assigned)");
			tempRecord.setTotal((Integer) record[6]);
			tempRecord.setNationalTotal((Integer) record[6]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Peripheral Measurements");
			tempRecord.setTotal((Integer) record[7]);
			tempRecord.setNationalTotal((Integer) record[7]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Video");
			tempRecord.setTotal((Integer) record[8]);
			tempRecord.setNationalTotal((Integer) record[8]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Cellular Modem Transmission");
			tempRecord.setTotal((Integer) record[9]);
			tempRecord.setNationalTotal((Integer) record[9]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Satellite Transmission");
			tempRecord.setTotal((Integer) record[10]);
			tempRecord.setNationalTotal((Integer) record[10]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Digital Image Capture");
			tempRecord.setTotal((Integer) record[11]);
			tempRecord.setNationalTotal((Integer) record[11]);
			returnResults.add(tempRecord);
		}
		return returnResults;
	}

	public List<CensusActivityRecord> getNationalReportByModality(
			CensusActivityReportParameters reportParams) throws DAOException {
		List<CensusActivityRecord> returnResults = new ArrayList<CensusActivityRecord>();

		String sql = getNationalSQLByModality(reportParams);
		String weekEndingDate = DateUtils.format(reportParams
				.getWeekEndingDate(), DateUtils.MMDDYYYY);
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.append(sql);
		queryAndParams.addParam("weekEndingDate", weekEndingDate);

		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();

		for (int j = 0; j < numIterations; j++) {
			Object[] record = (Object[]) report.get(j);

			CensusActivityRecord tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Total Patients");
			tempRecord.setTotal((Integer) record[1]);
			tempRecord.setNationalTotal((Integer) record[1]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName(reportParams.getModality().getValue());
			tempRecord.setTotal((Integer) record[2]);
			tempRecord.setNationalTotal((Integer) record[2]);
			returnResults.add(tempRecord);
		}

		return returnResults;
	}

	public List<CensusActivityRecord> getNationalReportByModalityAndVendor(
			CensusActivityReportParameters reportParams) throws DAOException {
		List<CensusActivityRecord> returnResults = new ArrayList<CensusActivityRecord>();

		String sql = getNationalSQLByModalityAndVendor(reportParams);
		String weekEndingDate = DateUtils.format(reportParams
				.getWeekEndingDate(), DateUtils.MMDDYYYY);
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.append(sql);
		queryAndParams.addParam("weekEndingDate", weekEndingDate);
		queryAndParams.addParam("vendorId", reportParams.getVendor()
				.getValue());

		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();

		for (int j = 0; j < numIterations; j++) {
			Object[] record = (Object[]) report.get(j);

			CensusActivityRecord tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName("Total Patients");
			tempRecord.setTotal((Integer) record[1]);
			tempRecord.setNationalTotal((Integer) record[1]);
			returnResults.add(tempRecord);

			tempRecord = new CensusActivityRecord();
			tempRecord.setModalityName(reportParams.getModality().getValue());
			tempRecord.setTotal((Integer) record[2]);
			tempRecord.setNationalTotal((Integer) record[2]);
			returnResults.add(tempRecord);
		}

		return returnResults;
	}

	private CensusActivityReportResult getCurrentRecord(Object[] record) {
		CensusActivityReportResult currentRecord = new CensusActivityReportResult();

		currentRecord.setNationalTotalsTitle((String) record[0]);
		currentRecord.setTotalPatients((Integer) record[1]);
		currentRecord.setMessagingTotal((Integer) record[2]);
		currentRecord.setMessagingByDevice((Integer) record[3]);
		currentRecord.setMessagingByIVR((Integer) record[4]);
		currentRecord.setMessagingByBrowser((Integer) record[5]);
		currentRecord.setMessagingByNotYetAssigned((Integer) record[6]);
		currentRecord.setPeripheralMeasurements((Integer) record[7]);
		currentRecord.setVideoTotal((Integer) record[8]);
		currentRecord.setCellularModemTransmission((Integer) record[9]);
		currentRecord.setSatelliteTransmission((Integer) record[10]);
		currentRecord.setDigitalImageCapture((Integer) record[11]);
		return currentRecord;
	}

	private CensusActivityReportResult getCurrentModalityRecord(Object[] record) {
		CensusActivityReportResult currentRecord = new CensusActivityReportResult();

		currentRecord.setNationalTotalsTitle((String) record[0]);
		currentRecord.setTotalPatients((Integer) record[1]);
		currentRecord.setTotalForSelectedModality((Integer) record[2]);

		return currentRecord;
	}

	@Override
	public List<CensusActivityVisnRecord> getAllVisnReport(
			CensusActivityReportParameters reportParams) throws DAOException {
		List<CensusActivityVisnRecord> returnResults = new ArrayList<CensusActivityVisnRecord>();
		
		
		String sql = null;
		String weekEndingDate = reportParams.getReportDate();
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.addParam("weekEndingDate", weekEndingDate);
		
		if(reportParams.getVisn() == null || reportParams.getVisn().getValue().equalsIgnoreCase("0"))
		{		
			sql = getAllVisnAllAllModalityAllVendorsSQL();
		}
		else
		{
			sql = getSingleVisnReportSQL();
			queryAndParams.addParam("visnId", reportParams.getVisn().getValue());
		}
		
		
		queryAndParams.append(sql);
		

		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();
		CensusActivityVisnRecord totalRecord = new CensusActivityVisnRecord();
		totalRecord.setVisnName("Total");
		for (int j = 0; j < numIterations; j++) {
			Object[] record = (Object[]) report.get(j);
			CensusActivityVisnRecord tempRecord = new CensusActivityVisnRecord();

			tempRecord.setVisnId((Integer) record[0]);
			tempRecord.setVisnName((String) record[1]);
			tempRecord.setPatientTotals((Integer) record[2]);
			tempRecord.setDialogue((Integer) record[3]);
			tempRecord.setMessagingByDevice((Integer) record[4]);
			tempRecord.setMessagingByIVR((Integer) record[5]);
			tempRecord.setBrowser((Integer) record[6]);
			tempRecord.setMessagingUnknown((Integer) record[7]);
			tempRecord.setMeasurement((Integer) record[8]);
			tempRecord.setVideo((Integer) record[9]);
			tempRecord.setCellModem((Integer) record[10]);
			tempRecord.setSatellite((Integer) record[11]);
			tempRecord.setDigitalImageCapture((Integer) record[12]);

			// Add the values of the current record to a
			// record that will represent a totals row
			totalRecord.add(tempRecord);

			returnResults.add(tempRecord);
		}

		returnResults.add(totalRecord);

		return returnResults;
	}

	@Override
	public List<CensusActivityVisnRecord> getSubTotaledAllVisnReport(
			CensusActivityReportParameters reportParams) throws DAOException {
		List<CensusActivityVisnRecord> returnResults = new ArrayList<CensusActivityVisnRecord>();
		String sql = null;
		String weekEndingDate = reportParams.getReportDate();
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.addParam("weekEndingDate", weekEndingDate);
		
		if(reportParams.getVisn() == null || reportParams.getVisn().getValue().equalsIgnoreCase("0"))
		{		
			sql = getSubTotaledVisnReportSQL();
		}
		else
		{
			sql = getSubTotaledSingleVisnReportSQL();
			queryAndParams.addParam("visnId", reportParams.getVisn().getValue());
		}
		
		
		queryAndParams.append(sql);

		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();
		CensusActivityVisnRecord totalRecord = new CensusActivityVisnRecord();
		totalRecord.setVisnName("Total");
		for (int j = 0; j < numIterations; j++) {
			Object[] record = (Object[]) report.get(j);
			CensusActivityVisnRecord tempRecord = new CensusActivityVisnRecord();

			tempRecord.setVisnId((Integer) record[0]);
			tempRecord.setVisnName((String) record[1]);
			tempRecord.setVendorName((String) record[2]);
			tempRecord.setPatientTotals((Integer) record[3]);
			tempRecord.setDialogue((Integer) record[4]);
			tempRecord.setMessagingByDevice((Integer) record[5]);
			tempRecord.setMessagingByIVR((Integer) record[6]);
			tempRecord.setBrowser((Integer) record[7]);
			tempRecord.setMessagingUnknown((Integer) record[8]);
			tempRecord.setMeasurement((Integer) record[9]);
			tempRecord.setVideo((Integer) record[10]);
			tempRecord.setCellModem((Integer) record[11]);
			tempRecord.setSatellite((Integer) record[12]);
			tempRecord.setDigitalImageCapture((Integer) record[13]);

			// Add the values of the current record to a
			// record that will represent a totals row
			totalRecord.add(tempRecord);

			returnResults.add(tempRecord);
		}

		returnResults.add(totalRecord);

		return returnResults;
	}
	
	
	@Override
	public List<CensusActivityVisnRecord> getSingleVisnReport(
			CensusActivityReportParameters reportParams) throws DAOException {
		List<CensusActivityVisnRecord> returnResults = new ArrayList<CensusActivityVisnRecord>();
		
		
		String sql = null;
		String weekEndingDate = reportParams.getReportDate();
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.addParam("weekEndingDate", weekEndingDate);
		
		sql = getSingleVisnReportSQL();
		queryAndParams.addParam("visnId", reportParams.getVisn().getValue());
		queryAndParams.append(sql);
		

		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();
		CensusActivityVisnRecord totalRecord = new CensusActivityVisnRecord();
		totalRecord.setVisnName("Total");
		for (int j = 0; j < numIterations; j++) {
			Object[] record = (Object[]) report.get(j);
			CensusActivityVisnRecord tempRecord = new CensusActivityVisnRecord();

			tempRecord.setVisnId((Integer) record[0]);
			tempRecord.setVisnName((String) record[1]);
			tempRecord.setPatientTotals((Integer) record[2]);
			tempRecord.setDialogue((Integer) record[3]);
			tempRecord.setMessagingByDevice((Integer) record[4]);
			tempRecord.setMessagingByIVR((Integer) record[5]);
			tempRecord.setBrowser((Integer) record[6]);
			tempRecord.setMessagingUnknown((Integer) record[7]);
			tempRecord.setMeasurement((Integer) record[8]);
			tempRecord.setVideo((Integer) record[9]);
			tempRecord.setCellModem((Integer) record[10]);
			tempRecord.setSatellite((Integer) record[11]);
			tempRecord.setDigitalImageCapture((Integer) record[12]);

			// Add the values of the current record to a
			// record that will represent a totals row
			totalRecord.add(tempRecord);

			returnResults.add(tempRecord);
		}

		returnResults.add(totalRecord);

		return returnResults;
	}

	@Override
	public List<CensusActivityVisnRecord> getSubTotaledSingleVisnReport(
			CensusActivityReportParameters reportParams) throws DAOException {
		List<CensusActivityVisnRecord> returnResults = new ArrayList<CensusActivityVisnRecord>();
		String sql = null;
		String weekEndingDate = reportParams.getReportDate();
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.addParam("weekEndingDate", weekEndingDate);
		
		if(reportParams.getVisn() == null || reportParams.getVisn().getValue().equalsIgnoreCase("0"))
		{		
			sql = getSubTotaledVisnReportSQL();
		}
		else
		{
			sql = getSubTotaledSingleVisnReportSQL();
			queryAndParams.addParam("visnId", reportParams.getVisn().getValue());
		}
		
		
		queryAndParams.append(sql);

		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();
		CensusActivityVisnRecord totalRecord = new CensusActivityVisnRecord();
		totalRecord.setVisnName("Total");
		for (int j = 0; j < numIterations; j++) {
			Object[] record = (Object[]) report.get(j);
			CensusActivityVisnRecord tempRecord = new CensusActivityVisnRecord();

			tempRecord.setVisnId((Integer) record[0]);
			tempRecord.setVisnName((String) record[1]);
			tempRecord.setVendorName((String) record[2]);
			tempRecord.setPatientTotals((Integer) record[3]);
			tempRecord.setDialogue((Integer) record[4]);
			tempRecord.setMessagingByDevice((Integer) record[5]);
			tempRecord.setMessagingByIVR((Integer) record[6]);
			tempRecord.setBrowser((Integer) record[7]);
			tempRecord.setMessagingUnknown((Integer) record[8]);
			tempRecord.setMeasurement((Integer) record[9]);
			tempRecord.setVideo((Integer) record[10]);
			tempRecord.setCellModem((Integer) record[11]);
			tempRecord.setSatellite((Integer) record[12]);
			tempRecord.setDigitalImageCapture((Integer) record[13]);

			// Add the values of the current record to a
			// record that will represent a totals row
			totalRecord.add(tempRecord);

			returnResults.add(tempRecord);
		}

		returnResults.add(totalRecord);

		return returnResults;
	}
	
	
	

	public List<CensusActivityVisnRecord> getVisnReportByVendor(CensusActivityReportParameters reportParams) throws DAOException 
	{
		List<CensusActivityVisnRecord> returnResults = new ArrayList<CensusActivityVisnRecord>();
		String sql = null;
		
		String reportWeek = reportParams.getReportDate();
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.addParam("reportWeek", reportWeek);
		String vendorId = reportParams.getVendor().getValue();
		if(reportParams.getVisn() == null || reportParams.getVisn().getValue().equalsIgnoreCase("0"))
		{			
			sql = getVisnReportByVendorSQL();			
		}
		else
		{
			//sql = getVisnReportByVisnIdAndVendorIdSQL();
			//queryAndParams.addParam("visnId", reportParams.getVisn().getValue());
		}
		
		queryAndParams.addParam("vendorId", vendorId);
		queryAndParams.append(sql);
		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();
		CensusActivityVisnRecord totalRecord = new CensusActivityVisnRecord();
		totalRecord.setVisnName("Total");
		for (int j = 0; j < numIterations; j++) {
			Object[] record = (Object[]) report.get(j);
			CensusActivityVisnRecord tempRecord = new CensusActivityVisnRecord();

			/*
			tempRecord.setVisnId((Integer) record[0]);
			tempRecord.setVisnName((String) record[1]);
			tempRecord.setVendorName((String) record[2]);
			tempRecord.setPatientTotals((Integer) record[3]);
			tempRecord.setDialogue((Integer) record[4]);
			tempRecord.setMessagingByDevice((Integer) record[5]);
			tempRecord.setMessagingByIVR((Integer) record[6]);
			tempRecord.setBrowser((Integer) record[7]);
			tempRecord.setMessagingUnknown((Integer) record[8]);
			tempRecord.setMeasurement((Integer) record[9]);
			tempRecord.setVideo((Integer) record[10]);
			tempRecord.setCellModem((Integer) record[11]);
			tempRecord.setSatellite((Integer) record[12]);
			tempRecord.setDigitalImageCapture((Integer) record[13]);
			*/
			
			tempRecord.setVisnId((Integer) record[0]);
			tempRecord.setVisnName((String) record[1]);
			tempRecord.setPatientTotals((Integer) record[2]);
			tempRecord.setDialogue((Integer) record[3]);
			tempRecord.setMessagingByDevice((Integer) record[4]);
			tempRecord.setMessagingByIVR((Integer) record[5]);
			tempRecord.setBrowser((Integer) record[6]);
			tempRecord.setMessagingUnknown((Integer) record[7]);
			tempRecord.setMeasurement((Integer) record[8]);
			tempRecord.setVideo((Integer) record[9]);
			tempRecord.setCellModem((Integer) record[10]);
			tempRecord.setSatellite((Integer) record[11]);
			tempRecord.setDigitalImageCapture((Integer) record[12]);

			// Add the values of the current record to a
			// record that will represent a totals row
			totalRecord.add(tempRecord);

			returnResults.add(tempRecord);
		}

		returnResults.add(totalRecord);

		return returnResults;
	}
	
	private String getVisnReportByVendorSQL()
	{
		/*
		 * select VISN.VISN_ID, sum(Facility_Activity.Patient_Totals) AS [Patient_Totals], sum(Facility_Activity.Dialogue) AS [Dialogue], 
		 * sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice], sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR], 
		 * sum(Facility_Activity.Browser) AS [Browser], sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown], 
		 * sum(Facility_Activity.Measurement) AS [Measurement], sum(Facility_Activity.Video) AS [Video], sum(Facility_Activity.CellModem) AS [CellModem], 
		 * sum(Facility_Activity.Satellite) AS [Satellite], sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] 
		 * from Facilities, Facility_Activity,Vendor, VISN 
		 * where Facility_Activity.Facility_ID=Facilities.ID 
		 * and Facility_Activity.Rpt_Wk='2015-01-24' 
		 * and Facility_Activity.Vendor_ID=Vendor.Vendor_ID 
		 * and Vendor.Vendor_ID='6' 
		 * and Facilities.VISN_ID=VISN.VISN_ID 
		 * group by VISN.VISN_ID, Vendor.Vendor_Name  order by 1 ASC
		 */
		StringBuilder sql = new StringBuilder();
		sql.append("select VISN.VISN_ID, cast(VISN.VISN_Name As VARCHAR) As visn_name, sum(Facility_Activity.Patient_Totals) AS [Patient_Totals], sum(Facility_Activity.Dialogue) AS [Dialogue], ");
		sql.append("sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice], sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR], ");
		sql.append("sum(Facility_Activity.Browser) AS [Browser], sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown], ");
		sql.append("sum(Facility_Activity.Measurement) AS [Measurement], sum(Facility_Activity.Video) AS [Video], sum(Facility_Activity.CellModem) AS [CellModem], ");
		sql.append("sum(Facility_Activity.Satellite) AS [Satellite], sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] ");
		sql.append("from Facilities, Facility_Activity,Vendor, VISN ");
		sql.append("where Facility_Activity.Facility_ID=Facilities.ID ");
		sql.append("and Facility_Activity.Rpt_Wk= :reportWeek ");
		sql.append("and Facility_Activity.Vendor_ID=Vendor.Vendor_ID ");
		sql.append("and Vendor.Vendor_ID= :vendorId ");
		sql.append("and Facilities.VISN_ID=VISN.VISN_ID ");
		sql.append("group by VISN.VISN_ID, visn_name, Vendor.Vendor_Name  order by 1 ASC ");
		return sql.toString();
	}
	
	private String getAllVisnAllAllModalityAllVendorsSQL() {
		StringBuilder sql = new StringBuilder();

		/*
		 * REPORT QUERY: select VISN.VISN_ID, VISN.VISN_Name,
		 * sum(Facility_Activity.Patient_Totals) AS [Patient_Totals],
		 * sum(Facility_Activity.Dialogue) AS [Dialogue],
		 * sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice],
		 * sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR],
		 * sum(Facility_Activity.Browser) AS [Browser],
		 * sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown],
		 * sum(Facility_Activity.Measurement) AS [Measurement],
		 * sum(Facility_Activity.Video) AS [Video],
		 * sum(Facility_Activity.CellModem) AS [CellModem],
		 * sum(Facility_Activity.Satellite) AS [Satellite],
		 * sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] from
		 * Facilities, Facility_Activity, VISN where
		 * Facility_Activity.Facility_ID=Facilities.ID and
		 * Facility_Activity.Rpt_Wk='2015-01-24' and
		 * Facilities.VISN_ID=VISN.VISN_ID group by VISN.VISN_ID order by 1 ASC
		 */

		sql
				.append("select VISN.VISN_ID, cast(VISN.VISN_Name As VARCHAR) As visn_name, sum(Facility_Activity.Patient_Totals) AS [Patient_Totals], sum(Facility_Activity.Dialogue) AS [Dialogue], ");
		sql
				.append("sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice], sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR], ");
		sql
				.append("sum(Facility_Activity.Browser) AS [Browser], sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown], sum(Facility_Activity.Measurement) AS [Measurement], ");
		sql
				.append("sum(Facility_Activity.Video) AS [Video], sum(Facility_Activity.CellModem) AS [CellModem], sum(Facility_Activity.Satellite) AS [Satellite], ");
		sql
				.append("sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] ");
		sql.append("from Facilities, Facility_Activity, VISN ");
		sql.append("where Facility_Activity.Facility_ID=Facilities.ID and ");
		sql.append("Facility_Activity.Rpt_Wk= :weekEndingDate and ");
		sql.append("Facilities.VISN_ID=VISN.VISN_ID ");
		sql
				.append("group by VISN.VISN_ID, visn_name  order by VISN.VISN_ID ASC");

		return sql.toString();
	}
	
	private String getSubTotaledVisnReportSQL() {
		StringBuilder sql = new StringBuilder();

		/*
		 * REPORT QUERY: select VISN.VISN_ID, VISN.VISN_Name,
		 * sum(Facility_Activity.Patient_Totals) AS [Patient_Totals],
		 * sum(Facility_Activity.Dialogue) AS [Dialogue],
		 * sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice],
		 * sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR],
		 * sum(Facility_Activity.Browser) AS [Browser],
		 * sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown],
		 * sum(Facility_Activity.Measurement) AS [Measurement],
		 * sum(Facility_Activity.Video) AS [Video],
		 * sum(Facility_Activity.CellModem) AS [CellModem],
		 * sum(Facility_Activity.Satellite) AS [Satellite],
		 * sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] from
		 * Facilities, Facility_Activity, VISN where
		 * Facility_Activity.Facility_ID=Facilities.ID and
		 * Facility_Activity.Rpt_Wk='2015-01-24' and
		 * Facilities.VISN_ID=VISN.VISN_ID group by VISN.VISN_ID order by 1 ASC
		 */

		sql
				.append("select VISN.VISN_ID, cast(VISN.VISN_Name As VARCHAR) As visn_name, Vendor.Vendor_Name, sum(Facility_Activity.Patient_Totals) AS [Patient_Totals], ");
		sql
				.append("sum(Facility_Activity.Dialogue) AS [Dialogue], sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice], ");
		sql
				.append("sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR], sum(Facility_Activity.Browser) AS [Browser], ");
		sql
				.append("sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown], sum(Facility_Activity.Measurement) AS [Measurement], ");
		sql
				.append("sum(Facility_Activity.Video) AS [Video], sum(Facility_Activity.CellModem) AS [CellModem], sum(Facility_Activity.Satellite) AS [Satellite], ");
		sql
				.append("sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] ");
		sql.append("from  Facilities ");
		sql
				.append("INNER JOIN Facility_Activity ON Facilities.ID = Facility_Activity.Facility_ID ");
		sql.append("INNER JOIN VISN ON Facilities.VISN_ID = VISN.VISN_ID ");
		sql
				.append("INNER JOIN Vendor ON Facility_Activity.Vendor_ID = Vendor.Vendor_ID ");
		sql
				.append("where Facility_Activity.Facility_ID=Facilities.ID and Facility_Activity.Rpt_Wk= :weekEndingDate ");
		sql
				.append("and Facilities.VISN_ID=VISN.VISN_ID group by VISN.VISN_ID, VISN.VISN_Name, Vendor.Vendor_Name ");
		sql.append("order by VISN.VISN_ID");

		return sql.toString();
	}
	
	private String getSingleVisnReportSQL() {
		StringBuilder sql = new StringBuilder();

		/*
		 * REPORT QUERY: select VISN.VISN_ID, VISN.VISN_Name,
		 * sum(Facility_Activity.Patient_Totals) AS [Patient_Totals],
		 * sum(Facility_Activity.Dialogue) AS [Dialogue],
		 * sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice],
		 * sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR],
		 * sum(Facility_Activity.Browser) AS [Browser],
		 * sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown],
		 * sum(Facility_Activity.Measurement) AS [Measurement],
		 * sum(Facility_Activity.Video) AS [Video],
		 * sum(Facility_Activity.CellModem) AS [CellModem],
		 * sum(Facility_Activity.Satellite) AS [Satellite],
		 * sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] from
		 * Facilities, Facility_Activity, VISN where
		 * Facility_Activity.Facility_ID=Facilities.ID and
		 * Facility_Activity.Rpt_Wk='2015-01-24' and
		 * Facilities.VISN_ID=VISN.VISN_ID and 
		 * VISN.VISN_ID=1
		 * group by VISN.VISN_ID order by 1 ASC
		 */

		sql
				.append("select VISN.VISN_ID, cast(VISN.VISN_Name As VARCHAR) As visn_name, sum(Facility_Activity.Patient_Totals) AS [Patient_Totals], sum(Facility_Activity.Dialogue) AS [Dialogue], ");
		sql
				.append("sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice], sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR], ");
		sql
				.append("sum(Facility_Activity.Browser) AS [Browser], sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown], sum(Facility_Activity.Measurement) AS [Measurement], ");
		sql
				.append("sum(Facility_Activity.Video) AS [Video], sum(Facility_Activity.CellModem) AS [CellModem], sum(Facility_Activity.Satellite) AS [Satellite], ");
		sql
				.append("sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] ");
		sql.append("from Facilities, Facility_Activity, VISN ");
		sql.append("where Facility_Activity.Facility_ID=Facilities.ID and ");
		sql.append("Facility_Activity.Rpt_Wk= :weekEndingDate and ");
		sql.append("Facilities.VISN_ID=VISN.VISN_ID and ");
		sql.append("VISN.VISN_ID= :visnId ");
		sql.append("group by VISN.VISN_ID, visn_name  order by VISN.VISN_ID ASC");

		return sql.toString();
	}
	
	
	private String getSubTotaledSingleVisnReportSQL() {
		StringBuilder sql = new StringBuilder();

		/*
		 * REPORT QUERY: select VISN.VISN_ID,cast(VISN.VISN_Name As VARCHAR) As visn_name, 
		 * sum(Facility_Activity.Patient_Totals) AS [Patient_Totals], 
		 * sum(Facility_Activity.Dialogue) AS [Dialogue], 
		 * sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice], 
		 * sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR], 
		 * sum(Facility_Activity.Browser) AS [Browser], 
		 * sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown], 
		 * sum(Facility_Activity.Measurement) AS [Measurement], 
		 * sum(Facility_Activity.Video) AS [Video], 
		 * sum(Facility_Activity.CellModem) AS [CellModem], 
		 * sum(Facility_Activity.Satellite) AS [Satellite], 
		 * sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] 
		 * from  Facilities 
		 * INNER JOIN Facility_Activity ON Facilities.ID = Facility_Activity.Facility_ID 
		 * INNER JOIN VISN ON Facilities.VISN_ID = VISN.VISN_ID 
		 * INNER JOIN Vendor ON Facility_Activity.Vendor_ID = Vendor.Vendor_ID  
		 * where Facility_Activity.Facility_ID=Facilities.ID and 
		 * Facility_Activity.Rpt_Wk='2015-01-24' and VISN.VISN_ID=2 and 
		 * Facilities.VISN_ID=VISN.VISN_ID 
		 * group by VISN.VISN_ID, VISN_Name, Vendor.Vendor_Name order by 1 ASC
		 */

		sql.append("select VISN.VISN_ID, cast(VISN.VISN_Name As VARCHAR) As visn_name, Vendor.Vendor_Name, sum(Facility_Activity.Patient_Totals) AS [Patient_Totals], sum(Facility_Activity.Dialogue) AS [Dialogue], ");
		sql.append("sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice], sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR], ");
		sql.append("sum(Facility_Activity.Browser) AS [Browser], sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown], sum(Facility_Activity.Measurement) AS [Measurement], ");
		sql.append("sum(Facility_Activity.Video) AS [Video], sum(Facility_Activity.CellModem) AS [CellModem], sum(Facility_Activity.Satellite) AS [Satellite], ");
		sql.append("sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] ");
		sql.append("from Facilities ");		
		sql.append("INNER JOIN Facility_Activity ON Facilities.ID = Facility_Activity.Facility_ID ");
		sql.append("INNER JOIN VISN ON Facilities.VISN_ID = VISN.VISN_ID ");				
		sql.append("INNER JOIN Vendor ON Facility_Activity.Vendor_ID = Vendor.Vendor_ID  ");
		sql.append("where Facility_Activity.Facility_ID=Facilities.ID and ");
		sql.append("Facility_Activity.Rpt_Wk= :weekEndingDate and VISN.VISN_ID= :visnId and "); 
		sql.append("Facilities.VISN_ID=VISN.VISN_ID ");
		sql.append("group by VISN.VISN_ID, VISN_Name, Vendor.Vendor_Name order by 1 ASC");

		return sql.toString();
	}
	
	
	private String getSingleVisnAllModalitySingleVendorReportSQL() {
		StringBuilder sql = new StringBuilder();

		/*
		 * REPORT QUERY: 
		 * select VISN.VISN_ID, cast(VISN.VISN_Name As VARCHAR) As visn_name, sum(Facility_Activity.Patient_Totals) AS [Patient_Totals], 
		 * sum(Facility_Activity.Dialogue) AS [Dialogue], sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice], 
		 * sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR], sum(Facility_Activity.Browser) AS [Browser], 
		 * sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown], sum(Facility_Activity.Measurement) AS [Measurement], 
		 * sum(Facility_Activity.Video) AS [Video], sum(Facility_Activity.CellModem) AS [CellModem], 
		 * sum(Facility_Activity.Satellite) AS [Satellite], sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] 
		 * from Facilities, Facility_Activity,Vendor, VISN 
		 * where Facility_Activity.Facility_ID=Facilities.ID 
		 * and Facility_Activity.Rpt_Wk='2015-06-13' 
		 * and Facility_Activity.Vendor_ID=Vendor.Vendor_ID 
		 * and Vendor.Vendor_ID='10' 
		 * and VISN.VISN_ID=1 
		 * and Facilities.VISN_ID=VISN.VISN_ID 
		 * group by VISN.VISN_ID, Visn_Name, Vendor.Vendor_Name  order by 1 ASC
		 */

		sql.append("select VISN.VISN_ID, cast(VISN.VISN_Name As VARCHAR) As visn_name, sum(Facility_Activity.Patient_Totals) AS [Patient_Totals], ");
		sql.append("sum(Facility_Activity.Dialogue) AS [Dialogue], sum(Facility_Activity.MessagingByDevice) AS [MessagingByDevice], ");
		sql.append("sum(Facility_Activity.MessagingByIVR) AS [MessagingByIVR], sum(Facility_Activity.Browser) AS [Browser], ");
		sql.append("sum(Facility_Activity.MessagingUnknown) AS [MessagingUnknown], sum(Facility_Activity.Measurement) AS [Measurement], ");
		sql.append("sum(Facility_Activity.Video) AS [Video], sum(Facility_Activity.CellModem) AS [CellModem], ");
		sql.append("sum(Facility_Activity.Satellite) AS [Satellite], sum(Facility_Activity.DigImageCapture) AS [DigImageCapture] ");
		sql.append("from Facilities, Facility_Activity,Vendor, VISN ");
		sql.append("where Facility_Activity.Facility_ID=Facilities.ID ");
		sql.append("and Facility_Activity.Rpt_Wk=:reportWeek ");
		sql.append("and Facility_Activity.Vendor_ID=Vendor.Vendor_ID ");
		sql.append("and Vendor.Vendor_ID=:vendorId ");
		sql.append("and VISN.VISN_ID=:visnId ");
		sql.append("and Facilities.VISN_ID=VISN.VISN_ID ");
		sql.append("group by VISN.VISN_ID,Visn_Name, Vendor.Vendor_Name  order by 1 ASC");
		

		return sql.toString();
	}
	
	
	
	
	private String getNationalSQL() {
		StringBuilder sql = new StringBuilder();

		sql
				.append("select 'National Totals' AS [National Totals], sum(Facility_Activity.Patient_Totals) AS [Total Patients], ");
		sql
				.append("sum(Facility_Activity.Dialogue) AS [Messaging Total], sum(Facility_Activity.MessagingByDevice) AS [Messaging By Device], ");
		sql
				.append("sum(Facility_Activity.MessagingByIVR) AS [Messaging By IVR], sum(Facility_Activity.Browser) AS [Messaging By Browser], ");
		sql
				.append("sum(Facility_Activity.MessagingUnknown) AS [Messaging (IVR, device, or browser not yet assigned)], ");
		sql
				.append("sum(Facility_Activity.Measurement) AS [Peripheral Measurements], sum(Facility_Activity.Video) AS [Video], ");
		sql
				.append("sum(Facility_Activity.CellModem) AS [Cellular Modem Transmission], sum(Facility_Activity.Satellite) AS [Satellite Transmission], ");
		sql
				.append("sum(Facility_Activity.DigImageCapture) AS [Digital Image Capture] ");
		sql.append("from Facility_Activity ");
		sql.append("where Facility_Activity.Rpt_Wk= :weekEndingDate ");
		sql.append("order by 1 ASC");

		return sql.toString();
	}

	private String getNationalSQLByModality(
			CensusActivityReportParameters reportParams) {
		/*
		 * REPORT QUERY: select 'National Totals',
		 * sum(Facility_Activity.Patient_Totals) AS [Total Patients],
		 * sum(Facility_Activity.Dialogue) AS [Messaging Total] from
		 * Facility_Activity where Facility_Activity.Rpt_Wk=@dtReportEndDate
		 * order by 1 ASC
		 */
		StringBuilder sql = new StringBuilder();
		sql
				.append("select 'National Totals', sum(Facility_Activity.Patient_Totals) AS [Total Patients], ");

		// Determine which column to sum for Messaging Total according to
		// modality chosen.
		String columnName = reportParams.getColumnNameForModality();
		sql.append("sum(Facility_Activity.");
		sql.append(columnName);
		sql.append(") AS [Messaging Total] ");

		sql.append("from Facility_Activity ");
		sql.append("where Facility_Activity.Rpt_Wk=:weekEndingDate ");

		return sql.toString();
	}

	private String getNationalSQLByVendor(TermType vendor) {
		/*
		 * REPORT QUERY: select 'National Totals for ATI',
		 * sum(Facility_Activity.Patient_Totals) AS [Total Patients],
		 * sum(Facility_Activity.Dialogue) AS [Messaging Total],
		 * sum(Facility_Activity.MessagingByDevice) AS [Messaging By Device],
		 * sum(Facility_Activity.MessagingByIVR) AS [Messaging By IVR],
		 * sum(Facility_Activity.Browser) AS [Messaging By Browser],
		 * sum(Facility_Activity.MessagingUnknown) AS [Messaging (IVR, device,
		 * or browser not yet assigned)], sum(Facility_Activity.Measurement) AS
		 * [Peripheral Measurements], sum(Facility_Activity.Video) AS [Video],
		 * sum(Facility_Activity.CellModem) AS [Cellular Modem Transmission],
		 * sum(Facility_Activity.Satellite) AS [Satellite Transmission],
		 * sum(Facility_Activity.DigImageCapture) AS [Digital Image Capture]
		 * from Facility_Activity,Vendor where
		 * Facility_Activity.Rpt_Wk=@dtReportEndDate and
		 * Facility_Activity.Vendor_ID=Vendor.Vendor_ID and Vendor.Vendor_ID=6
		 * GROUP BY Vendor.Vendor_Name order by 1 ASC
		 */


		String totalsColumnName = getTotalsColumnName(vendor);
		StringBuilder sql = new StringBuilder();
		sql.append("select 'National Totals for ");
		sql.append(totalsColumnName);
		sql
				.append("', sum(Facility_Activity.Patient_Totals) AS [Total Patients], ");

		sql
				.append("sum(Facility_Activity.Dialogue) AS [Messaging Total], sum(Facility_Activity.MessagingByDevice) AS [Messaging By Device], ");
		sql
				.append("sum(Facility_Activity.MessagingByIVR) AS [Messaging By IVR], sum(Facility_Activity.Browser) AS [Messaging By Browser], ");
		sql
				.append("sum(Facility_Activity.MessagingUnknown) AS [Messaging (IVR, device, or browser not yet assigned)], ");
		sql
				.append("sum(Facility_Activity.Measurement) AS [Peripheral Measurements], sum(Facility_Activity.Video) AS [Video], ");
		sql
				.append("sum(Facility_Activity.CellModem) AS [Cellular Modem Transmission], sum(Facility_Activity.Satellite) AS [Satellite Transmission], ");
		sql
				.append("sum(Facility_Activity.DigImageCapture) AS [Digital Image Capture]");
		sql.append("from Vendor ");
		sql
				.append("INNER JOIN Facility_Activity ON Facility_Activity.Vendor_ID = Vendor.Vendor_ID ");
		sql.append("where Facility_Activity.Rpt_Wk = :weekEndingDate ");

		sql.append("and Vendor.Vendor_Id = :vendorId ");
		sql.append("GROUP BY Vendor.Vendor_Name order by 1 ASC ");

		return sql.toString();
	}

	private String getNationalSQLByModalityAndVendor(
			CensusActivityReportParameters reportParams) {

		/*
		 * REPORT QUERY: select 'National Totals for ATI',
		 * sum(Facility_Activity.Patient_Totals) AS [Total Patients],
		 * sum(Facility_Activity.Dialogue) AS [Messaging Total] from
		 * Facility_Activity,Vendor where
		 * Facility_Activity.Rpt_Wk=@dtReportEndDate and
		 * Facility_Activity.Vendor_ID=Vendor.Vendor_ID and Vendor.Vendor_ID=6
		 * GROUP BY Vendor.Vendor_Name order by 1 ASC
		 */
		TermType vendor = reportParams.getVendor();

		String totalsColumnName = getTotalsColumnName(vendor);
		StringBuilder sql = new StringBuilder();
		sql.append("select 'National Totals for ");
		sql.append(totalsColumnName);
		sql
				.append("', sum(Facility_Activity.Patient_Totals) AS [Total Patients], ");

		// Determine which column to sum for Messaging Total according to
		// modality chosen.
		String columnName = reportParams.getColumnNameForModality();
		sql.append("sum(Facility_Activity.");
		sql.append(columnName);
		sql.append(") AS [Messaging Total] ");

		sql.append("from Vendor ");
		sql
				.append("INNER JOIN Facility_Activity ON Facility_Activity.Vendor_ID = Vendor.Vendor_ID ");
		sql.append("where Facility_Activity.Rpt_Wk = :weekEndingDate ");

		sql.append("and Vendor.Vendor_ID = :vendorId ");
		sql.append("GROUP BY Vendor.Vendor_Name order by 1 ASC ");

		return sql.toString();
	}

	private String getTotalsColumnName(TermType vendor) {
		StringBuilder columnName = new StringBuilder();

		if (vendor != null) {
			columnName.append(vendor.getLabel());
		}

		return columnName.toString();
	}

	@Override
	public List<CensusActivitySubTotalRecord> getSubTotaledNationalReport(
			CensusActivityReportParameters reportParams) throws DAOException {
		List<CensusActivitySubTotalRecord> retVal = null;
		String sql = getSubTotaledNationalReportSQL();
		String weekEndingDate = DateUtils.format(reportParams
				.getWeekEndingDate(), DateUtils.MMDDYYYY);
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.append(sql);
		queryAndParams.addParam("weekEndingDate", weekEndingDate);

		List<?> rows = executeSQLQuery(queryAndParams);

		if (rows != null || rows.size() > 0) {
			retVal = getNationalSubTotalPivotedList(rows);
		}

		return retVal;
	}
	
	
	public List<CensusActivityVisnRecord> getSingleVisnAllModalitySingleVendorReport(CensusActivityReportParameters reportParams) throws DAOException
	{
		
		List<CensusActivityVisnRecord> retVal = new ArrayList<CensusActivityVisnRecord>();
		String sql = getSingleVisnAllModalitySingleVendorReportSQL();
		
		String reportWeek = reportParams.getReportDate();
		String visnId = reportParams.getVisn().getValue();
		String vendorId = reportParams.getVendor().getValue();
		
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.addParam("reportWeek", reportWeek);
		queryAndParams.addParam("visnId", visnId);
		queryAndParams.addParam("vendorId", vendorId);
		
		queryAndParams.append(sql);
		

		List<?> report = executeSQLQuery(queryAndParams);
		int numIterations = report.size();
		CensusActivityVisnRecord totalRecord = new CensusActivityVisnRecord();
		totalRecord.setVisnName("Total");
		for (int j = 0; j < numIterations; j++) {
			Object[] record = (Object[]) report.get(j);
			CensusActivityVisnRecord tempRecord = new CensusActivityVisnRecord();

			tempRecord.setVisnId((Integer) record[0]);
			tempRecord.setVisnName(((String) record[1]).trim());
			tempRecord.setPatientTotals((Integer) record[2]);
			tempRecord.setDialogue((Integer) record[3]);
			tempRecord.setMessagingByDevice((Integer) record[4]);
			tempRecord.setMessagingByIVR((Integer) record[5]);
			tempRecord.setBrowser((Integer) record[6]);
			tempRecord.setMessagingUnknown((Integer) record[7]);
			tempRecord.setMeasurement((Integer) record[8]);
			tempRecord.setVideo((Integer) record[9]);
			tempRecord.setCellModem((Integer) record[10]);
			tempRecord.setSatellite((Integer) record[11]);
			tempRecord.setDigitalImageCapture((Integer) record[12]);

			// Add the values of the current record to a
			// record that will represent a totals row
			totalRecord.add(tempRecord);

			retVal.add(tempRecord);
		}

		retVal.add(totalRecord);
		
		
		return retVal;
	}
	
	public List<CensusActivityVisnRecord> getFacilityDrilldownData(CensusActivityReportParameters params) throws DAOException
	{
		QueryAndParams queryAndParams = new QueryAndParams();
		List<CensusActivityVisnRecord> retVal = new ArrayList<CensusActivityVisnRecord>();
		String weekEndingDate = DateUtils.format(params.getWeekEndingDate(), DateUtils.MMDDYYYY);
		String selectedVisnId = params.getVisn().getValue();
		queryAndParams.addParam("weekEndingDate", weekEndingDate);
		queryAndParams.addParam("selectedVisnId", selectedVisnId);
		
		String sql = "";
		String selectedVendorId = params.getVendor().getValue();
		if(selectedVendorId.equalsIgnoreCase("All Vendors"))
		{
			sql = getFacilityDrilldownDataSQL();
		}
		else
		{
			sql = getFacilityDrilldownDataByVendorSQL();			
			queryAndParams.addParam("selectedVendorId", selectedVendorId);
		}
		
		queryAndParams.append(sql);
		
		List<?> rows = executeSQLQuery(queryAndParams);
		
		int numRows = rows.size();
		CensusActivityVisnRecord totalRecord = new CensusActivityVisnRecord();
		totalRecord.setVisnName("Total");
		for (int j = 0; j < numRows; j++) {
			Object[] record = (Object[]) rows.get(j);
			CensusActivityVisnRecord tempRecord = new CensusActivityVisnRecord();

			tempRecord.setFacility((String) record[0]);		
			tempRecord.setVisnId((Integer) record[1]);
			tempRecord.setVisnName((String) record[2]);
	
			tempRecord.setPatientTotals((Integer) record[3]);
			tempRecord.setDialogue((Integer) record[4]);
			tempRecord.setMessagingByDevice((Integer) record[5]);
			tempRecord.setMessagingByIVR((Integer) record[6]);
			tempRecord.setBrowser((Integer) record[7]);
			tempRecord.setMessagingUnknown((Integer) record[8]);
			tempRecord.setMeasurement((Integer) record[9]);
			tempRecord.setVideo((Integer) record[10]);
			tempRecord.setCellModem((Integer) record[11]);
			tempRecord.setSatellite((Integer) record[12]);
			tempRecord.setDigitalImageCapture((Integer) record[13]);

			// Add the values of the current record to a
			// record that will represent a totals row
			totalRecord.add(tempRecord);

			retVal.add(tempRecord);
		}

		retVal.add(totalRecord);
		
		
		
		return retVal;
	}
	
	private String getFacilityDrilldownDataSQL() {
		
		/*
		 * select CONVERT(VARCHAR(10), Facilities.Facility_ID) + ' - ' +Facilities.Facility_Name AS [Facility],VISN.VISN_ID AS [VISN], 
		 * sum(cast(VISN.VISN_Name As VARCHAR) As visn_name, Facility_Activity.Patient_Totals) AS [Total Patients], ");   
		 * sum(Facility_Activity.Dialogue) AS [Messaging Total], 
		 * sum(Facility_Activity.MessagingByDevice) AS [Messaging By Device], sum(Facility_Activity.MessagingByIVR) AS [Messaging By IVR], 
		 * sum(Facility_Activity.Browser) AS [Messaging By Browser], 
		 * sum(Facility_Activity.MessagingUnknown) AS [Messaging (IVR, device, or browser not yet assigned)], 
		 * sum(Facility_Activity.Measurement) AS [Peripheral Measurements], sum(Facility_Activity.Video) AS [Video], 
		 * sum(Facility_Activity.CellModem) AS [Cellular Modem Transmission], sum(Facility_Activity.Satellite) AS [Satellite Transmission], 
		 * sum(Facility_Activity.DigImageCapture) AS [Digital Image Capture] 
		 * from Facilities, Facility_Activity, VISN 
		 * where Facility_Activity.Facility_ID=Facilities.ID and Facility_Activity.Rpt_Wk='2015-01-24' 
		 * and VISN.VISN_ID=1 and Facilities.VISN_ID=VISN.VISN_ID 
		 * group by Facilities.facility_Name,VISN.VISN_ID, VISN.VISN_NAME,Facilities.facility_id 
		 * order by Facilities.Facility_Name DESC
		 */
		StringBuilder sql = new StringBuilder();
		sql.append("select CONVERT(VARCHAR(10), Facilities.Facility_ID) + ' - ' +Facilities.Facility_Name AS [Facility],VISN.VISN_ID AS [VISN], ");
		sql.append("cast(VISN.VISN_Name As VARCHAR) As visn_name, sum(Facility_Activity.Patient_Totals) AS [Total Patients], "); 
		sql.append("sum(Facility_Activity.Dialogue) AS [Messaging Total], ");
		sql.append("sum(Facility_Activity.MessagingByDevice) AS [Messaging By Device], sum(Facility_Activity.MessagingByIVR) AS [Messaging By IVR], ");
		sql.append("sum(Facility_Activity.Browser) AS [Messaging By Browser], ");
		sql.append("sum(Facility_Activity.MessagingUnknown) AS [Messaging (IVR, device, or browser not yet assigned)], ");
		sql.append("sum(Facility_Activity.Measurement) AS [Peripheral Measurements], sum(Facility_Activity.Video) AS [Video], ");
		sql.append("sum(Facility_Activity.CellModem) AS [Cellular Modem Transmission], sum(Facility_Activity.Satellite) AS [Satellite Transmission], ");
		sql.append("sum(Facility_Activity.DigImageCapture) AS [Digital Image Capture] ");
		sql.append("from Facilities, Facility_Activity, VISN ");
		sql.append("where Facility_Activity.Facility_ID=Facilities.ID and Facility_Activity.Rpt_Wk= :weekEndingDate ");
		sql.append("and VISN.VISN_ID= :selectedVisnId and Facilities.VISN_ID=VISN.VISN_ID ");
		sql.append("group by Facilities.facility_Name,VISN.VISN_ID, VISN.VISN_NAME, Facilities.facility_id ");
		sql.append("order by Facilities.Facility_Name DESC");
		return sql.toString();
	}	
	private String getFacilityDrilldownDataByVendorSQL() {
		
		/*
		 *	select CONVERT(VARCHAR(10), Facilities.Facility_ID) + ' - ' +Facilities.Facility_Name AS [Facility],VISN.VISN_ID AS [VISN], cast(VISN.VISN_Name As VARCHAR) As visn_name, 
		 *	sum(Facility_Activity.Patient_Totals) AS [Total Patients], sum(Facility_Activity.Dialogue) AS [Messaging Total], 
		 *	sum(Facility_Activity.MessagingByDevice) AS [Messaging By Device], sum(Facility_Activity.MessagingByIVR) AS [Messaging By IVR], 
		 *	sum(Facility_Activity.Browser) AS [Messaging By Browser], sum(Facility_Activity.MessagingUnknown) AS [Messaging (IVR, device, or browser not yet assigned)], 
		 *	sum(Facility_Activity.Measurement) AS [Peripheral Measurements], sum(Facility_Activity.Video) AS [Video], 
		 *	sum(Facility_Activity.CellModem) AS [Cellular Modem Transmission], sum(Facility_Activity.Satellite) AS [Satellite Transmission], 
		 *	sum(Facility_Activity.DigImageCapture) AS [Digital Image Capture] from Facilities, Facility_Activity,Vendor, VISN 
		 *	where Facility_Activity.Facility_ID=Facilities.ID and Facility_Activity.Rpt_Wk='2015-01-24' 
		 *	and Facility_Activity.Vendor_ID=Vendor.Vendor_ID and Vendor.Vendor_ID='10' and VISN.VISN_ID=1 and Facilities.VISN_ID=VISN.VISN_ID 
		 *	group by Facilities.facility_Name,VISN.VISN_ID, VISN_Name, Facilities.facility_id, Vendor.Vendor_Name order by Facilities.Facility_Name DESC			
		 */
		StringBuilder sql = new StringBuilder();
		sql.append("select CONVERT(VARCHAR(10), Facilities.Facility_ID) + ' - ' +Facilities.Facility_Name AS [Facility],VISN.VISN_ID AS [VISN], cast(VISN.VISN_Name As VARCHAR) As visn_name, ");
		sql.append("sum(Facility_Activity.Patient_Totals) AS [Total Patients], sum(Facility_Activity.Dialogue) AS [Messaging Total], "); 
		sql.append("sum(Facility_Activity.MessagingByDevice) AS [Messaging By Device], sum(Facility_Activity.MessagingByIVR) AS [Messaging By IVR], ");
		sql.append("sum(Facility_Activity.Browser) AS [Messaging By Browser], sum(Facility_Activity.MessagingUnknown) AS [Messaging (IVR, device, or browser not yet assigned)], ");
		sql.append("sum(Facility_Activity.Measurement) AS [Peripheral Measurements], sum(Facility_Activity.Video) AS [Video], ");
		sql.append("sum(Facility_Activity.CellModem) AS [Cellular Modem Transmission], sum(Facility_Activity.Satellite) AS [Satellite Transmission], ");
		sql.append("sum(Facility_Activity.DigImageCapture) AS [Digital Image Capture] ");
		sql.append("from Facilities, Facility_Activity,Vendor, VISN ");
		sql.append("where Facility_Activity.Facility_ID=Facilities.ID ");
		sql.append("and Facility_Activity.Rpt_Wk=:weekEndingDate ");
		sql.append("and Facility_Activity.Vendor_ID=Vendor.Vendor_ID ");
		sql.append("and Vendor.Vendor_ID=:selectedVendorId ");
		sql.append("and VISN.VISN_ID=:selectedVisnId ");
		sql.append("and Facilities.VISN_ID=VISN.VISN_ID ");
		sql.append("group by Facilities.facility_Name,VISN.VISN_ID, Facilities.facility_id, visn_name, Vendor.Vendor_Name ");
		sql.append("order by Facilities.Facility_Name DESC ");
		return sql.toString();
	}	
	private String getFacilityDrilldownDataByModalitySQL() 
	{
		StringBuilder sql = new StringBuilder();
		
		return sql.toString();
	}	
	private String getFacilityDrilldownDataByVendorAndModalitySQL() 
	{
		StringBuilder sql = new StringBuilder();
		
		return sql.toString();
	}
	private String getSubTotaledNationalReportSQL() {
		StringBuilder sql = new StringBuilder();
		sql
				.append("select Vendor.Vendor_Name as [Vendor], sum(Facility_Activity.Patient_Totals) AS [Total Patients], ");
		sql
				.append("sum(Facility_Activity.Dialogue) AS [Messaging Total], sum(Facility_Activity.MessagingByDevice) AS [Messaging By Device], ");
		sql
				.append("sum(Facility_Activity.MessagingByIVR) AS [Messaging By IVR], sum(Facility_Activity.Browser) AS [Messaging By Browser], ");
		sql
				.append("sum(Facility_Activity.MessagingUnknown) AS [Messaging (IVR, device, or browser not yet assigned)], ");
		sql
				.append("sum(Facility_Activity.Measurement) AS [Peripheral Measurements], sum(Facility_Activity.Video) AS [Video], ");
		sql
				.append("sum(Facility_Activity.CellModem) AS [Cellular Modem Transmission], ");
		sql
				.append("sum(Facility_Activity.Satellite) AS [Satellite Transmission], ");
		sql
				.append("sum(Facility_Activity.DigImageCapture) AS [Digital Image Capture] ");
		sql
				.append("from Facility_Activity,Vendor where Facility_Activity.Rpt_Wk= :weekEndingDate ");
		sql
				.append("and Vendor.Vendor_Name <> 'None' and Facility_Activity.Vendor_ID=Vendor.Vendor_ID ");
		sql.append("GROUP BY Vendor.Vendor_Name order by 1 ASC ");
		return sql.toString();
	}

	@Override
	public List<CensusActivitySubTotalRecord> getSubTotaledNationalReportByModality(
			CensusActivityReportParameters reportParams) throws DAOException {
		List<CensusActivitySubTotalRecord> retVal = null;
		String sql = getSubTotaledNationalReportByModalitySQL(reportParams);

		String weekEndingDate = DateUtils.format(reportParams
				.getWeekEndingDate(), DateUtils.MMDDYYYY);
		QueryAndParams queryAndParams = new QueryAndParams();
		queryAndParams.append(sql);
		queryAndParams.addParam("weekEndingDate", weekEndingDate);

		List<?> rows = executeSQLQuery(queryAndParams);

		if (rows != null || rows.size() > 0) {
			retVal = getNationalSubTotalPivotedListByModality(rows,
					reportParams.getColumnNameForModality());
		}

		return retVal;
	}

	private String getSubTotaledNationalReportByModalitySQL(
			CensusActivityReportParameters reportParams) {
		/*
		 * select Vendor.Vendor_Name as [Vendor],
		 * sum(Facility_Activity.Patient_Totals) AS [Total Patients],
		 * sum(Facility_Activity.Dialogue) AS [Messaging Total] from
		 * Facility_Activity,Vendor where
		 * Facility_Activity.Rpt_Wk=@dtReportEndDate and Vendor.Vendor_Name <>
		 * 'None' and Facility_Activity.Vendor_ID=Vendor.Vendor_ID GROUP BY
		 * Vendor.Vendor_Name order by 1 ASC
		 */
		StringBuilder sql = new StringBuilder();

		sql.append("select Vendor.Vendor_Name as [Vendor],");
		sql
				.append("sum(Facility_Activity.Patient_Totals) AS [Total Patients],");

		// Determine which column to sum for Messaging Total according to
		// modality chosen.
		String columnName = reportParams.getColumnNameForModality();
		sql.append("sum(Facility_Activity.");
		sql.append(columnName);
		sql.append(") AS [Messaging Total] ");

		sql.append("from Facility_Activity,Vendor ");
		sql.append("where Facility_Activity.Rpt_Wk= :weekEndingDate ");
		sql.append("and Vendor.Vendor_Name <> 'None' ");
		sql.append("and Facility_Activity.Vendor_ID=Vendor.Vendor_ID ");
		sql.append("GROUP BY Vendor.Vendor_Name order by 1 ASC");
		return sql.toString();
	}

	private List<CensusActivitySubTotalRecord> getNationalSubTotalPivotedList(
			List<?> rows) {
		int numRows = rows.size();

		// Create a group of lists that will pivot the record array
		List<Integer> totalPatients = new ArrayList<Integer>();
		List<Integer> messagingTotals = new ArrayList<Integer>();
		List<Integer> deviceTotals = new ArrayList<Integer>();
		List<Integer> ivrTotals = new ArrayList<Integer>();
		List<Integer> browserTotals = new ArrayList<Integer>();
		List<Integer> unknownTotals = new ArrayList<Integer>();
		List<Integer> measureTotals = new ArrayList<Integer>();
		List<Integer> videoTotals = new ArrayList<Integer>();
		List<Integer> cellTotals = new ArrayList<Integer>();
		List<Integer> satTotals = new ArrayList<Integer>();
		List<Integer> digitalTotals = new ArrayList<Integer>();

		boolean hasAti = hasVendor(rows, "ATI");
		boolean hasAuthentidate = hasVendor(rows, "Authentidate");
		boolean hasCardiocom = hasVendor(rows, "Cardiocom");
		boolean hasHealthHero = hasVendor(rows, "Health Hero");
		boolean hasViTelNet = hasVendor(rows, "ViTel Net");
		boolean hasViterion = hasVendor(rows, "Viterion");

		for (int row = 0; row < numRows; row++) {
			Object[] record = (Object[]) rows.get(row);

			int numColumns = record.length;

			for (int col = 1; col < numColumns; col++) {
				Integer val = (Integer) record[col];

				if (val == null) {
					val = 0;
				}

				switch (col) {
				case total_patients:
					totalPatients.add(val);
					break;
				case messaging_total:
					messagingTotals.add(val);
					break;
				case device:
					deviceTotals.add(val);
					break;
				case ivr:
					ivrTotals.add(val);
					break;
				case browser:
					browserTotals.add(val);
					break;
				case unknown:
					unknownTotals.add(val);
					break;
				case measurements:
					measureTotals.add(val);
					break;
				case video:
					videoTotals.add(val);
					break;
				case cell:
					cellTotals.add(val);
					break;
				case satellite:
					satTotals.add(val);
					break;
				case digital:
					digitalTotals.add(val);
					break;
				}
			}

		}

		if (!hasAti) {

			totalPatients.add(0, 0);
			messagingTotals.add(0, 0);
			deviceTotals.add(0, 0);
			ivrTotals.add(0, 0);
			browserTotals.add(0, 0);
			unknownTotals.add(0, 0);
			measureTotals.add(0, 0);
			videoTotals.add(0, 0);
			cellTotals.add(0, 0);
			satTotals.add(0, 0);
			digitalTotals.add(0, 0);
		}

		if (!hasAuthentidate) {

			totalPatients.add(1, 0);
			messagingTotals.add(1, 0);
			deviceTotals.add(1, 0);
			ivrTotals.add(1, 0);
			browserTotals.add(1, 0);
			unknownTotals.add(1, 0);
			measureTotals.add(1, 0);
			videoTotals.add(1, 0);
			cellTotals.add(1, 0);
			satTotals.add(1, 0);
			digitalTotals.add(1, 0);
		}

		if (!hasCardiocom) {
			totalPatients.add(2, 0);
			messagingTotals.add(2, 0);
			deviceTotals.add(2, 0);
			ivrTotals.add(2, 0);
			browserTotals.add(2, 0);
			unknownTotals.add(2, 0);
			measureTotals.add(2, 0);
			videoTotals.add(2, 0);
			cellTotals.add(2, 0);
			satTotals.add(2, 0);
			digitalTotals.add(2, 0);
		}

		if (!hasHealthHero) {
			totalPatients.add(3, 0);
			messagingTotals.add(3, 0);
			deviceTotals.add(3, 0);
			ivrTotals.add(3, 0);
			browserTotals.add(3, 0);
			unknownTotals.add(3, 0);
			measureTotals.add(3, 0);
			videoTotals.add(3, 0);
			cellTotals.add(3, 0);
			satTotals.add(3, 0);
			digitalTotals.add(3, 0);
		}

		if (!hasViTelNet) {
			totalPatients.add(4, 0);
			messagingTotals.add(4, 0);
			deviceTotals.add(4, 0);
			ivrTotals.add(4, 0);
			browserTotals.add(4, 0);
			unknownTotals.add(4, 0);
			measureTotals.add(4, 0);
			videoTotals.add(4, 0);
			cellTotals.add(4, 0);
			satTotals.add(4, 0);
			digitalTotals.add(4, 0);
		}

		if (!hasViterion) {
			totalPatients.add(5, 0);
			messagingTotals.add(5, 0);
			deviceTotals.add(5, 0);
			ivrTotals.add(5, 0);
			browserTotals.add(5, 0);
			unknownTotals.add(5, 0);
			measureTotals.add(5, 0);
			videoTotals.add(5, 0);
			cellTotals.add(5, 0);
			satTotals.add(5, 0);
			digitalTotals.add(5, 0);
		}
		CensusActivitySubTotalRecord patientTotalsRecord = getCensusActivitySubTotalRecord(
				totalPatients, TOTAL_PATIENTS);
		CensusActivitySubTotalRecord messagingTotalsRecord = getCensusActivitySubTotalRecord(
				messagingTotals, MESSAGING_TOTAL);
		CensusActivitySubTotalRecord deviceTotalsRecord = getCensusActivitySubTotalRecord(
				deviceTotals, MESSAGING_BY_DEVICE);
		CensusActivitySubTotalRecord ivrTotalsRecord = getCensusActivitySubTotalRecord(
				ivrTotals, MESSAGING_BY_IVR);
		CensusActivitySubTotalRecord browserTotalsRecord = getCensusActivitySubTotalRecord(
				browserTotals, MESSAGING_BY_BROWSER);
		CensusActivitySubTotalRecord unknownTotalsRecord = getCensusActivitySubTotalRecord(
				unknownTotals, MESSAGING_UNKOWN);
		CensusActivitySubTotalRecord measureTotalsRecord = getCensusActivitySubTotalRecord(
				measureTotals, PERIPHERAL_MEASUREMENTS);
		CensusActivitySubTotalRecord videoTotalsRecord = getCensusActivitySubTotalRecord(
				videoTotals, VIDEO);
		CensusActivitySubTotalRecord cellTotalsRecord = getCensusActivitySubTotalRecord(
				cellTotals, CELLULAR_MODEM_TRANSMISSION);
		CensusActivitySubTotalRecord satTotalsRecord = getCensusActivitySubTotalRecord(
				satTotals, SATELLITE_TRANSMISSION);
		CensusActivitySubTotalRecord digitalTotalsRecord = getCensusActivitySubTotalRecord(
				digitalTotals, DIGITAL_IMAGE_CAPTURE);

		List<CensusActivitySubTotalRecord> returnVal = new ArrayList<CensusActivitySubTotalRecord>();
		returnVal.add(patientTotalsRecord);
		returnVal.add(messagingTotalsRecord);
		returnVal.add(deviceTotalsRecord);
		returnVal.add(ivrTotalsRecord);
		returnVal.add(browserTotalsRecord);
		returnVal.add(unknownTotalsRecord);
		returnVal.add(measureTotalsRecord);
		returnVal.add(videoTotalsRecord);
		returnVal.add(cellTotalsRecord);
		returnVal.add(satTotalsRecord);
		returnVal.add(digitalTotalsRecord);
		return returnVal;
	}

	private List<CensusActivitySubTotalRecord> getNationalSubTotalPivotedListByModality(
			List<?> rows, String selectedModalityName) {
		int numRows = rows.size();

		// Create a group of lists that will pivot the record array
		List<Integer> totalPatients = new ArrayList<Integer>();
		List<Integer> selectedModality = new ArrayList<Integer>();

		boolean hasAti = hasVendor(rows, "ATI");
		boolean hasAuthentidate = hasVendor(rows, "Authentidate");
		boolean hasCardiocom = hasVendor(rows, "Cardiocom");
		boolean hasHealthHero = hasVendor(rows, "Health Hero");
		boolean hasViTelNet = hasVendor(rows, "ViTel Net");
		boolean hasViterion = hasVendor(rows, "Viterion");

		for (int row = 0; row < numRows; row++) {
			Object[] record = (Object[]) rows.get(row);

			int numColumns = record.length;

			for (int col = 1; col < numColumns; col++) {
				Integer val = (Integer) record[col];

				if (val == null) {
					val = 0;
				}

				switch (col) {
				case 1:
					totalPatients.add(val);
					break;
				case 2:
					selectedModality.add(val);
					break;
				}
			}

		}

		if (!hasAti) {

			totalPatients.add(0, 0);
			selectedModality.add(0, 0);
		}

		if (!hasAuthentidate) {

			totalPatients.add(1, 0);
			selectedModality.add(1, 0);
		}

		if (!hasCardiocom) {
			totalPatients.add(2, 0);
			selectedModality.add(2, 0);
		}

		if (!hasHealthHero) {
			totalPatients.add(3, 0);
			selectedModality.add(3, 0);
		}

		if (!hasViTelNet) {
			totalPatients.add(4, 0);
			selectedModality.add(4, 0);
		}

		if (!hasViterion) {
			totalPatients.add(5, 0);
			selectedModality.add(5, 0);
		}

		CensusActivitySubTotalRecord patientTotalsRecord = getCensusActivitySubTotalRecord(
				totalPatients, TOTAL_PATIENTS);
		CensusActivitySubTotalRecord selectedModalitysRecord = getCensusActivitySubTotalRecord(
				selectedModality, selectedModalityName);

		List<CensusActivitySubTotalRecord> returnVal = new ArrayList<CensusActivitySubTotalRecord>();
		returnVal.add(patientTotalsRecord);
		returnVal.add(selectedModalitysRecord);

		return returnVal;
	}

	private CensusActivitySubTotalRecord getCensusActivitySubTotalRecord(
			List<Integer> row, String modalityName) {
		CensusActivitySubTotalRecord retVal = new CensusActivitySubTotalRecord();
		final int ati_Total = 0;
		final int authentidate_Total = 1;
		final int cardiocomTotal = 2;
		final int healthHeroTotal = 3;
		final int vitelNetTotal = 4;
		final int viterionTotal = 5;

		retVal.setModalityName(modalityName);
		for (int col = 0; col < row.size(); col++) {
			switch (col) {
			case ati_Total:
				retVal.setAtiTotal(row.get(col));
				break;
			case authentidate_Total:
				retVal.setAuthentidateTotal(row.get(col));
				break;
			case cardiocomTotal:
				retVal.setCardiocomTotal(row.get(col));
				break;
			case healthHeroTotal:
				retVal.setHealthHeroTotal(row.get(col));
				break;
			case vitelNetTotal:
				retVal.setVitelNetTotal(row.get(col));
				break;
			case viterionTotal:
				retVal.setViterionTotal(row.get(col));
				break;
			}
		}

		return retVal;
	}

	private boolean hasVendor(List<?> rows, String vendor) {
		boolean hasVendor = false;

		int numRows = rows.size();

		for (int col = 0; col < numRows; col++) {
			Object[] row = (Object[]) rows.get(col);
			String curVendor = (String) row[0];

			if (curVendor.equalsIgnoreCase(vendor)) {
				return true;
			}
		}

		return hasVendor;
	}

	private String getAckTypeLongDescription(String ackType) {
		String retVal = "";
		if (ackType != null && ackType.equalsIgnoreCase("AA")) {
			retVal = "";
		} else if (ackType != null && ackType.equalsIgnoreCase("AR")) {
			retVal = " (No Report)";
		} else if (ackType != null && ackType.equalsIgnoreCase("IP")) {
			retVal = " (In Progress)";
		}

		return retVal;
	}
	
	
	private Integer getIntFromDBCol(Object col)
	{
		Integer retVal = 0;
		
		if(col != null)
		{
			retVal = (Integer)col;
		}
		
		
		return retVal;
		
	}


	private GregorianCalendar getReportEndDateAsCal()
	{
		GregorianCalendar cal = new GregorianCalendar();
		int dayOfWeek = cal.get(Calendar.DAY_OF_WEEK);

		switch (dayOfWeek) {
		case Calendar.SUNDAY:
			cal.set(Calendar.DATE, cal.get(Calendar.DATE) - 1);
			break;
		case Calendar.MONDAY:
			cal.set(Calendar.DATE, cal.get(Calendar.DATE) - 2);
			break;
		case Calendar.TUESDAY:
			cal.set(Calendar.DATE, cal.get(Calendar.DATE) - 3);
			break;
		case Calendar.WEDNESDAY:
			cal.set(Calendar.DATE, cal.get(Calendar.DATE) - 4);
			break;
		case Calendar.THURSDAY:
			cal.set(Calendar.DATE, cal.get(Calendar.DATE) - 5);
			break;
		case Calendar.FRIDAY:
			cal.set(Calendar.DATE, cal.get(Calendar.DATE) - 6);
			break;
		}
		
		return cal;
	}
	
	private GregorianCalendar getReportStartDateAsCal(GregorianCalendar endDate)
	{
	
		GregorianCalendar cal = new GregorianCalendar();
		
		cal = endDate;
		cal.set(Calendar.DATE, endDate.get(Calendar.DATE)-6);		
		
		return cal;
	}
	
	private String getCalendarAsString(GregorianCalendar cal)
	{
		if(cal != null)
		{
			return (cal.get(Calendar.MONTH) + 1) + "/" + (cal.get(Calendar.DATE)) + "/" + cal.get(Calendar.YEAR);
		}
		else
		{
			return null;
		}
	}
	
	 
	
	private Map<String, VendorCensusReportStatus> combineVendorStatusMaps(Map<String, VendorCensusReportStatus> activeVendorsForReportingWeek, Map<String, VendorCensusReportStatus> activeVendorsStatusForReportingWeek)
	{
		int numIterations = activeVendorsStatusForReportingWeek.keySet().size();
		
		for(int i = 0; i < numIterations; i++)
		{
			String key = (String)activeVendorsStatusForReportingWeek.keySet().toArray()[i];
			activeVendorsForReportingWeek.put(key, activeVendorsStatusForReportingWeek.get(key));
		}
		
		return activeVendorsForReportingWeek;
	}
	
	private Map<String, VendorCensusReportStatus> removeCompletedVendors(Map<String, VendorCensusReportStatus> activeVendorsForReportingWeek, Date startDate, Date endDate)
	{
		int numIterations = activeVendorsForReportingWeek.keySet().size();
		Map<String, VendorCensusReportStatus> retVal = new HashMap<String, VendorCensusReportStatus>();
		for(int i = 0; i < numIterations; i++)
		{
			String key = (String)activeVendorsForReportingWeek.keySet().toArray()[i];
			
			VendorCensusReportStatus temp = activeVendorsForReportingWeek.get(key);
			
			if(!temp.getStatus().equalsIgnoreCase(COMPLETED))
			{
				String vendorName = temp.getVendorName() + " (" + temp.getStatus() + ")";
				temp.setVendorName(vendorName);
				temp.setStartDate(startDate);
				temp.setEndDate(endDate);
				retVal.put(key, temp);
			}
			
		}
		
		return retVal;
	}

}
