package gov.va.med.ccht.persistent.hibernate;

import gov.va.med.ccht.model.ReportConstants;
import gov.va.med.ccht.model.responserate.ResponseRateParameters;
import gov.va.med.ccht.model.responserate.ResponseRateRecord;
import gov.va.med.ccht.model.responserate.ResponseRateResults;
import gov.va.med.ccht.model.responserate.ResponseRateRow;
import gov.va.med.ccht.persistent.ResponseRateReportDAO;
import gov.va.med.fw.persistent.DAOException;
import gov.va.med.fw.persistent.hibernate.GenericDAOImpl;
import gov.va.med.fw.persistent.hibernate.QueryAndParams;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

public class ResponseRateReportDAOImpl extends GenericDAOImpl implements ResponseRateReportDAO
{
    public ResponseRateResults getNationalReport(ResponseRateParameters reportParams) throws DAOException
    {
	ResponseRateResults retVal = new ResponseRateResults();
	String reportDate = reportParams.getReportEndDate();
	QueryAndParams queryAndParams = getQueryAndParams(reportParams);
	List<?> report = executeSQLQuery(queryAndParams);
	int numIterations = report.size();
	List<ResponseRateRecord> records = new ArrayList<ResponseRateRecord>();
	for (int j = 0; j < numIterations; j++)
	{
	    ResponseRateRecord nextRecord = new ResponseRateRecord();
	    Object[] record = (Object[]) report.get(j);
	    Integer id = (Integer)record[0];
	    Integer dummy = (Integer)record[1];
	    String vendorName = (String)record[2];
	    Integer responseCode = (Integer)record[3];
	    Integer patientCount = (Integer)record[4];
	    BigDecimal maxDataAgeAsBd = (BigDecimal)record[5];
	    
	    Integer maxDataAge = 0;
	    
	    if(maxDataAgeAsBd != null)
	    {
		maxDataAge = maxDataAgeAsBd.intValueExact();
	    }
		
	    
	    
	    nextRecord.setId(id);
	    nextRecord.setDummy(dummy);
	    nextRecord.setVendorName(vendorName);
	    nextRecord.setResponseCode(responseCode);
	    nextRecord.setPatientCount(patientCount);
	    nextRecord.setMaxDataAge(maxDataAge);
	    	    
	    records.add(nextRecord);
	}
	

	
	if(numIterations > 0)
	{
	    retVal.setResponseRateRecords(records);
	    retVal.setRecordsKeyedByVendor();
	    retVal.buildCategoryOfCareRows();
	    retVal.setTotalsRow();
	}
	
	/*
	List<ResponseRateRecord> responseRateRecords = getTempRecords();
	retVal.setResponseRateRecords(responseRateRecords);
	retVal.setTotalsRow();
	*/
	return retVal;
    }    
    
    private QueryAndParams getQueryAndParams(ResponseRateParameters reportParams)
    {
	QueryAndParams queryAndParams = new QueryAndParams();

	/*
	 * SELECT 0 as [id], 0 as [Dummy], b.vendor_name, a.responsecode, count
	 * (DISTINCT a.PatientID) PatientCnt, MAX (a.data_age) MaxDataAge -- ,
	 * 'National Report' Mode FROM (SELECT b.vendor_id AS VendorID,
	 * dbo.GetWeeklyCompliance3 (a.rr_res, a.data_age) AS responsecode,
	 * CONVERT ( NUMERIC (5), (REPLACE (REPLACE (a.data_age, 'NA', '0'),
	 * '-', '0'))) AS data_age, a.patient_id AS PatientID FROM censuspatient
	 * a, census b WHERE a.census_id = b.census_id AND (
	 * a.disenrollment_date IS NULL OR a.disenrollment_date >= dateadd (day,
	 * -6, '06/14/2014')) AND a.enrollment_date <= '06/14/2014' AND
	 * b.report_start_date = dateadd (day, -6, '06/14/2014') UNION SELECT
	 * b2.vendor_id AS VendorID, 5 AS responsecode, CONVERT ( NUMERIC (5),
	 * (REPLACE (REPLACE (a2.data_age, 'NA', '0'), '-', '0'))) AS data_age,
	 * a2.patient_id AS PatientID FROM censuspatient a2, census b2 WHERE
	 * a2.census_id = b2.census_id AND ( a2.disenrollment_date IS NULL OR
	 * a2.disenrollment_date >= dateadd (day, -6, '06/14/2014')) AND
	 * a2.enrollment_date <= '06/14/2014' AND b2.report_start_date = dateadd
	 * (day, -6, '06/14/2014') AND CONVERT ( NUMERIC (5), (REPLACE (REPLACE
	 * (a2.data_age, 'NA', '0'), '-', '0'))) > 30) a JOIN vendor b ON
	 * a.VendorID = b.Vendor_ID GROUP BY b.vendor_name, a.responsecode ORDER
	 * BY b.vendor_name, a.responsecode ASC
	 */

	StringBuilder sql = new StringBuilder();
	 sql.append(" SELECT 0 as [id], 0 as [Dummy], b.vendor_name, a.responsecode, count");
	 sql.append(" (DISTINCT a.PatientID) PatientCnt, MAX (a.data_age) MaxDataAge ");
	 sql.append(" FROM (SELECT b.vendor_id AS VendorID,");
	 sql.append(" dbo.GetWeeklyCompliance3 (a.rr_res, a.data_age) AS responsecode,");
	 sql.append(" CONVERT ( NUMERIC (5), (REPLACE (REPLACE (a.data_age, 'NA', '0'),");
	 sql.append(" '-', '0'))) AS data_age, a.patient_id AS PatientID FROM censuspatient");
	 sql.append(" a, census b WHERE a.census_id = b.census_id AND (");
	 sql.append(" a.disenrollment_date IS NULL OR a.disenrollment_date >= dateadd (day,");
	 sql.append(" -6, :reportEndDate)) AND a.enrollment_date <= :reportEndDate AND");
	 sql.append(" b.report_start_date = dateadd (day, -6, :reportEndDate) UNION SELECT");
	 sql.append(" b2.vendor_id AS VendorID, 5 AS responsecode, CONVERT ( NUMERIC (5),");
	 sql.append(" (REPLACE (REPLACE (a2.data_age, 'NA', '0'), '-', '0'))) AS data_age,");
	 sql.append(" a2.patient_id AS PatientID FROM censuspatient a2, census b2 WHERE");
	 sql.append(" a2.census_id = b2.census_id AND ( a2.disenrollment_date IS NULL OR");
	 sql.append(" a2.disenrollment_date >= dateadd (day, -6, :reportEndDate)) AND");
	 sql.append(" a2.enrollment_date <= :reportEndDate AND b2.report_start_date = dateadd");
	 sql.append(" (day, -6, :reportEndDate) AND CONVERT ( NUMERIC (5), (REPLACE (REPLACE");
	 sql.append(" (a2.data_age, 'NA', '0'), '-', '0'))) > 30) a JOIN vendor b ON");
	 sql.append(" a.VendorID = b.Vendor_ID GROUP BY b.vendor_name, a.responsecode ORDER");
	 sql.append(" BY b.vendor_name, a.responsecode ASC");
	
	
	queryAndParams.append(sql.toString());
	queryAndParams.addParam("reportEndDate", reportParams.getReportEndDate());
	return queryAndParams;
    }

    private List<ResponseRateRecord> getTempDatabaseRecords()
    {
	List<ResponseRateRecord> rows = new ArrayList<ResponseRateRecord>();
	
	rows.addAll(getTempDatabaseATIRecords());
	rows.addAll(getTempDatabaseAuthentidateRecords());
	rows.addAll(getTempDatabaseCardiocomRecords());
	rows.addAll(getTempDatabaseHealthHeroRecords());
	rows.addAll(getTempDatabaseVitelRecords());
	rows.addAll(getTempDatabaseViterionRecords());
	
	return rows;
    }
    
    private List<ResponseRateRecord> getTempDatabaseATIRecords()
    {
	List<ResponseRateRecord> rows = new ArrayList<ResponseRateRecord>();
	
	ResponseRateRecord row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.ATI);
	row.setResponseCode(1);
	row.setPatientCount(26);
	row.setMaxDataAge(0);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.ATI);
	row.setResponseCode(2);
	row.setPatientCount(100);
	row.setMaxDataAge(6);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.ATI);
	row.setResponseCode(3);
	row.setPatientCount(100);
	row.setMaxDataAge(2993);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.ATI);
	row.setResponseCode(5);
	row.setPatientCount(29);
	row.setMaxDataAge(2993);
	rows.add(row);
	
	
	return rows;
    }
        
    private List<ResponseRateRecord> getTempDatabaseAuthentidateRecords()
    {
	List<ResponseRateRecord> rows = new ArrayList<ResponseRateRecord>();
	
	ResponseRateRecord row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.AUTHENTIDATE);
	row.setResponseCode(1);
	row.setPatientCount(234);
	row.setMaxDataAge(1);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.AUTHENTIDATE);
	row.setResponseCode(2);
	row.setPatientCount(1560);
	row.setMaxDataAge(7);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.AUTHENTIDATE);
	row.setResponseCode(3);
	row.setPatientCount(5);
	row.setMaxDataAge(9);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.AUTHENTIDATE);
	row.setResponseCode(4);
	row.setPatientCount(29);
	row.setMaxDataAge(42);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.AUTHENTIDATE);
	row.setResponseCode(5);
	row.setPatientCount(1);
	row.setMaxDataAge(42);
	rows.add(row);
	
	
	return rows;
    }
    
    private List<ResponseRateRecord> getTempDatabaseCardiocomRecords()
    {
	List<ResponseRateRecord> rows = new ArrayList<ResponseRateRecord>();
	
	ResponseRateRecord row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.CARDIOCOM);
	row.setResponseCode(1);
	row.setPatientCount(14127);
	row.setMaxDataAge(8);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.CARDIOCOM);
	row.setResponseCode(2);
	row.setPatientCount(36908);
	row.setMaxDataAge(8);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.CARDIOCOM);
	row.setResponseCode(3);
	row.setPatientCount(6226);
	row.setMaxDataAge(457);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.CARDIOCOM);
	row.setResponseCode(5);
	row.setPatientCount(562);
	row.setMaxDataAge(457);
	rows.add(row);
	
	
	return rows;
    }
    
    private List<ResponseRateRecord> getTempDatabaseHealthHeroRecords()
    {
	List<ResponseRateRecord> rows = new ArrayList<ResponseRateRecord>();
	
	ResponseRateRecord row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.HEALTH_HERO);
	row.setResponseCode(1);
	row.setPatientCount(5086);
	row.setMaxDataAge(0);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.HEALTH_HERO);
	row.setResponseCode(2);
	row.setPatientCount(19263);
	row.setMaxDataAge(7);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.HEALTH_HERO);
	row.setResponseCode(3);
	row.setPatientCount(1813);
	row.setMaxDataAge(1453);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.HEALTH_HERO);
	row.setResponseCode(4);
	row.setPatientCount(757);
	row.setMaxDataAge(953);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.HEALTH_HERO);
	row.setResponseCode(5);
	row.setPatientCount(293);
	row.setMaxDataAge(1453);
	rows.add(row);
	
	
	return rows;
    }
    
    private List<ResponseRateRecord> getTempDatabaseVitelRecords()
    {
	List<ResponseRateRecord> rows = new ArrayList<ResponseRateRecord>();
	
	ResponseRateRecord row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.VITEL_NET);
	row.setResponseCode(1);
	row.setPatientCount(687);
	row.setMaxDataAge(0);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.VITEL_NET);
	row.setResponseCode(2);
	row.setPatientCount(2479);
	row.setMaxDataAge(7);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.VITEL_NET);
	row.setResponseCode(3);
	row.setPatientCount(107);
	row.setMaxDataAge(537);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.VITEL_NET);
	row.setResponseCode(5);
	row.setPatientCount(26);
	row.setMaxDataAge(537);
	rows.add(row);
	
	
	return rows;
    }
        
    private List<ResponseRateRecord> getTempDatabaseViterionRecords()
    {
	List<ResponseRateRecord> rows = new ArrayList<ResponseRateRecord>();
	
	ResponseRateRecord row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.VITERION);
	row.setResponseCode(1);
	row.setPatientCount(1008);
	row.setMaxDataAge(1);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.VITERION);
	row.setResponseCode(2);
	row.setPatientCount(2579);
	row.setMaxDataAge(7);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.VITERION);
	row.setResponseCode(3);
	row.setPatientCount(52);
	row.setMaxDataAge(1498);
	rows.add(row);
	
	row = new ResponseRateRecord();
	row.setId(0);
	row.setDummy(0);	
	row.setVendorName(ReportConstants.VITERION);
	row.setResponseCode(5);
	row.setPatientCount(20);
	row.setMaxDataAge(1498);
	rows.add(row);
	
	
	return rows;
    }
    
 

    private List<ResponseRateRow> getTempRecords()
    {
	List<ResponseRateRow> responseRateRecords = new ArrayList<ResponseRateRow>();

	ResponseRateRow recordOne = new ResponseRateRow();
	recordOne.setVendorName(ReportConstants.ATI);
	recordOne.setNumResponders(26);
	recordOne.setPartialResponders(100);
	recordOne.setNonResponders(100);
	recordOne.setUnknown(0);
	recordOne.setTotal();
	recordOne.setThirtyDaysNonResponse(229);
	recordOne.setGreatestDaysNonResponse(2993);
	responseRateRecords.add(recordOne);

	recordOne = new ResponseRateRow();
	recordOne.setVendorName(ReportConstants.AUTHENTIDATE);
	recordOne.setNumResponders(234);
	recordOne.setPartialResponders(1560);
	recordOne.setNonResponders(5);
	recordOne.setUnknown(29);
	recordOne.setTotal();
	recordOne.setThirtyDaysNonResponse(1);
	recordOne.setGreatestDaysNonResponse(42);
	responseRateRecords.add(recordOne);

	recordOne = new ResponseRateRow();
	recordOne.setVendorName(ReportConstants.CARDIOCOM);
	recordOne.setNumResponders(14127);
	recordOne.setPartialResponders(36908);
	recordOne.setNonResponders(6226);
	recordOne.setUnknown(0);
	recordOne.setTotal();
	recordOne.setThirtyDaysNonResponse(562);
	recordOne.setGreatestDaysNonResponse(457);
	responseRateRecords.add(recordOne);

	recordOne = new ResponseRateRow();
	recordOne.setVendorName(ReportConstants.HEALTH_HERO);
	recordOne.setNumResponders(5086);
	recordOne.setPartialResponders(19263);
	recordOne.setNonResponders(1813);
	recordOne.setUnknown(757);
	recordOne.setTotal();
	recordOne.setThirtyDaysNonResponse(293);
	recordOne.setGreatestDaysNonResponse(1453);
	responseRateRecords.add(recordOne);

	recordOne = new ResponseRateRow();
	recordOne.setVendorName(ReportConstants.VITEL_NET);
	recordOne.setNumResponders(687);
	recordOne.setPartialResponders(2479);
	recordOne.setNonResponders(107);
	recordOne.setUnknown(0);
	recordOne.setTotal();
	recordOne.setThirtyDaysNonResponse(26);
	recordOne.setGreatestDaysNonResponse(537);
	responseRateRecords.add(recordOne);

	recordOne = new ResponseRateRow();
	recordOne.setVendorName(ReportConstants.VITERION);
	recordOne.setNumResponders(1008);
	recordOne.setPartialResponders(2579);
	recordOne.setNonResponders(52);
	recordOne.setUnknown(0);
	recordOne.setTotal();
	recordOne.setThirtyDaysNonResponse(20);
	recordOne.setGreatestDaysNonResponse(1498);
	responseRateRecords.add(recordOne);

	return responseRateRecords;
    }
}
