$(function() {
	$("#scheduledJobTabContainer").tabs({
		heightStyle : "content",
		activate: function( event, ui ) {
			if (ui.newTab.text() == 'Active') {
				refreshActiveJobs()
			} else {
				refreshCurrentJobs()
			}
		}
	})

	buildCurrentJobsTable()
	buildActiveJobsTable()
	buildConfigPopup()
	buildSchedulePopup()
	buildHistoryPopup()
	refreshCurrentJobs()

})

/* ---------------------------------- Current Jobs table & buttons */

var allJobs = {}
function refreshCurrentJobs() {
	var currentJob = getCurrentJob()
	
	var table = $('#currentJobs').DataTable()
	table.clear()
	
	$.ajax({
		url : 'ajax/scheduledJobs',
		dataType : 'json',
		data : {
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			var selectedRowIndex = -1
			
			allJobs = {}
			for (var i = 0; i < r.jobs.length; i++) {
				var job = r.jobs[i]
				allJobs['' + job.id] = job
				if (currentJob && currentJob.id == job.id)
					selectedRowIndex = i
			}
			
			table.rows.add(r.jobs)
			table.draw()
			
			if (selectedRowIndex != -1)
				table.row(selectedRowIndex).select()
				
			updateResumeAndPauseButtons()
			forceFocusOnInfoMessage()
		}
	})
}

function showButtons() {
	$('#execute').show()
	$('#config').show()
	$('#schedule').show()
	$('#history').show()
	updateResumeAndPauseButtons()
}

function updateResumeAndPauseButtons() {
	var table = $('#currentJobs').DataTable()
	var tableData = table.data()
	
	if (tableData == null || !tableData.length)
		return

	var hasPausedItem = false
	var hasRunningItem = false
	for (var i = 0; i < tableData.length; i++) {
		hasPausedItem = hasPausedItem || (tableData[i].status == 'PAUSED')
		hasRunningItem = hasRunningItem || (tableData[i].status == 'WAITING')
	}
	
	$('#resumeAll').toggle(hasPausedItem)
	$('#pauseAll').toggle(hasRunningItem)
}

function forceFocusOnInfoMessage() {
	
	if($('#informationMessageDiv').is(':visible')){
		$('#informationMessageDiv p').focus()
	}
}

function buildCurrentJobsTable() {
	var table = $('#currentJobs').DataTable({
		"columnDefs" : [{
			"targets" : 0,
			"data" : function(row, type, set, meta) {
				return row.jobName
			}
		}, {
			"targets" : 1,
			"data" : function(row, type, set, meta) {
				return row.jobDescription
			}
		}, {
			"targets" : 2,
			"data" : function(row, type, set, meta) {
				return row.jobScheduleDescription
			}
		}, {
			"targets" : 3,
			"data" : function(row, type, set, meta) {
				return row.nextFireTime.formattedDate
			}
		}, {
			"targets" : 4,
			"data" : function(row, type, set, meta) {
				return row.status
			}
		}],
		scrollY : "250px",
		paging : true,
		autowidth : true,
		searching : false,
		select : 'single'
	})

	$('#currentJobs tbody').on('click', 'tr', function() {
		var data = table.row(this).data()
		$("#currentJobs").data('selectedJobName', data.jobName)
		showButtons(data)
	})
}

/* ---------------------------------- Active Jobs table & buttons */

var activeJobs = {}
function refreshActiveJobs() {
	var table = $('#activeJobs').DataTable()
	table.clear()
	
	$.ajax({
		url : 'ajax/scheduledJobs/active',
		dataType : 'json',
		error : commonAjaxErrorHandler,
		success : function(r) {
			activeJobs = {}
			for (var i = 0; i < r.jobs.length; i++) {
				var job = r.jobs[i]
				activeJobs['' + job.id] = job
			}
			
			table.rows.add(r.jobs)
			table.draw()
		}
	})
}

function buildActiveJobsTable() {
	var table = $('#activeJobs').DataTable({
		"columnDefs" : [{
			"targets" : 0,
			"data" : function(row, type, set, meta) {
				return row.jobName
			}
		}, {
			"targets" : 1,
			"data" : function(row, type, set, meta) {
				return row.formattedStartDate
			}
		}, {
			"targets" : 2,
			"data" : function(row, type, set, meta) {
				return row.runBy
			}
		}, {
			"targets" : 3,
			"data" : function(row, type, set, meta) {
				return row.status
			}
		}],
		scrollY : "250px",
		paging : true,
		autowidth : true,
		searching : false,
		select : 'single'
	})
}

/* ---------------------------------- Execute button */

function submitJobExecution() {
	doubleClickSafeguard($("#execute"))
	var jobName = $("#currentJobs").data('selectedJobName')
	if (!jobName) return
	
	$.ajax({
		url : 'ajax/scheduledJobs/execute',
		dataType : 'json',
		data : {
			jobName : jobName
		},
		error : commonAjaxErrorHandler,
		success : function() {
			showInformationMessage(jobName + " has been scheduled. Go to the History page to check the job completion status.")
			refreshCurrentJobs()
		}
	})
}

/* ---------------------------------- Config dialog */

function buildConfigPopup() {
	$("#configFieldsWrapper").dialog({
		autoOpen : false,
		modal : true,
		width : 850,
		height : 400,
		closeOnEscape : true,
		draggable : true,
		resizable : true,
		buttons : [{
           id: "configPopupSubmit",
           text: "Submit",
           click: submitJobConfigUpdate
		}, {
           id: "configPopupCancel",
           text: "Cancel",
           click: function() {
   				$(this).dialog('close')
           }
		}]
	})
}

function popupConfigEdit() {
	var job = getCurrentJob()
	
	$("#configJobName").text(job ? job.jobName : '')
	$("#configJobNameInput").val(job ? job.jobName : '')
	$("#configGroupName").val(job ? job.groupName : '')
	$("#configJobDescription").text(job ? job.jobDescription : '')
	$("#configJobNextFireTime").text(job && job.nextFireTime ? job.nextFireTime.formattedDate : '')
	$("#configJobExistingStatus").text(job ? job.status : '')
	$("#configJobEmails").val(job ? job.emailDistributionList : '')
	$("#configJobStatus").val(job ? job.status : '')
	$("#configJobPreviousStatus").val(job ? job.previousTriggerStatus : '')
	$("#configFieldsWrapper").dialog('open')
}

function submitJobConfigUpdate() {
	doubleClickSafeguard($("#configPopupSubmit"))
	var jobName = $("#configJobNameInput").val()
	
	$.ajax({
		url : 'ajax/scheduledJobs/configUpdate',
		dataType : 'json',
		data : {
			jobName : jobName,
			groupName : $("#configGroupName").val(),
			emails : $("#configJobEmails").val(),
			jobStatus : $("#configJobStatus").val(),
			previousJobStatus : $("#configJobPreviousStatus").val()
		},
		error : commonAjaxErrorHandler,
		success : function() {
			showInformationMessage(jobName + " updated successfully.")
			$("#configFieldsWrapper").dialog('close')
			refreshCurrentJobs()
		}
	})
}

/* ---------------------------------- Schedule dialog */

function buildSchedulePopup() {
	$("#scheduleFieldsWrapper").dialog({
		autoOpen : false,
		modal : true,
		width : 850,
		height : 400,
		closeOnEscape : true,
		draggable : true,
		resizable : true,
		buttons : [{
           id: "schedulePopupSubmit",
           text: "Submit",
           click: submitJobScheduleUpdate
       }, {
           id: "schedulePopupCancel",
           text: "Cancel",
           click: function() {
   				$(this).dialog('close')
   			}
		}]
	})
	
	$("#scheduleChangeWrapper").dialog({
		autoOpen : false,
		modal : true,
		width : 600,
		height : 300,
		closeOnEscape : true,
		draggable : true,
		resizable : true,
		buttons : [{
           id: "scheduleChangePopupSubmit",
           text: "Submit",
           click: submitJobScheduleValidation
       },{
           id: "scheduleChangePopupReset",
           text: "Reset",
           click: function() {
   				setScheduleChangeFields()
   			}
		}, {
           id: "scheduleChangePopupCancel",
           text: "Cancel",
           click: function() {
   				$(this).dialog('close')
   			}
		}]
	})
	
	$("#otherDate").datepicker()
	$("#scheduleChangeFrequency").change(function() {
		scheduleChangeFrequencyUpdated(true)
	})
}

function popupScheduleEdit() {
	var job = getCurrentJob()
	
	$("#scheduleJobName").text(job ? job.jobName : '')
	$("#scheduleJobDescription").text(job ? job.jobDescription : '')
	$("#scheduleJobNextFireTime").text(job && job.nextFireTime ? job.nextFireTime.formattedDate : '')
	$("#scheduleJobExistingStatus").text(job ? job.status : '')
	$("#scheduleJobScheduleDescriptionSpan").text(job ? job.jobScheduleDescription : '')
	
	$("#scheduleJobNameInput").val(job ? job.jobName : '')
	$("#scheduleGroupNameInput").val(job ? job.groupName : '')
	
	setScheduleChangeFields()
	
	$("#scheduleTriggerNameInput").val(job ? job.triggerName : '')
	$("#scheduleTriggerGroupInput").val(job ? job.triggerGroup : '')
	
	$("#scheduleFieldsWrapper").dialog('open')
}

function submitJobScheduleUpdate() {
	doubleClickSafeguard($("#schedulePopupSubmit"))
	var jobName = $("#scheduleJobNameInput").val()
	
	$.ajax({
		url : 'ajax/scheduledJobs/scheduleUpdate',
		dataType : 'json',
		data : {
			jobName : jobName,
			groupName : $("#scheduleGroupNameInput").val(),
			triggerName : $("#scheduleTriggerNameInput").val(),
			triggerGroup : $("#scheduleTriggerGroupInput").val(),
			scheduleJobCronInput : $("#scheduleJobCronInput").val(),
			jobScheduleDescription : $("#scheduleJobScheduleDescriptionInput").val(),
			scheduleJobScheduleParamsCSV : $("#scheduleJobScheduleParamsCSV").val()
		},
		error : commonAjaxErrorHandler,
		success : function() {
			$("#scheduleFieldsWrapper").dialog('close')
			refreshCurrentJobs()
		}
	})
}

function popupScheduleChange() {
	$("#scheduleChangeWrapper").dialog('open')
}

function submitJobScheduleValidation() {
	doubleClickSafeguard($("#scheduleChangePopupSubmit"))
	var job = getCurrentJob()
	
	var DAILY_CODE = '1'
	var MONTHLY_DAY_OF_MONTH_CODE = '2'
	var QUARTERLY_CODE = '3'
	var MONTHLY_WEEK_DAY_CODE = '4'
	var YEARLY_WEEK_DAY_MONTH_CODE = '5'
	var YEARLY_MONTH_DAY_CODE = '6'
	var OTHER_CODE = '7'
		
	var scheduleTypeCode = $("input[name='scheduleType']:checked").val()
	
	var dayOfWeekCSV = buildDaysOfWeekCSV()
	var relativeDayOfMonthVal = $("#relativeDayOfMonthValue").val()
	var weekOfMonthVal = $("#weekOfMonth").val()
	var dayOfWeekVal = $("#dayOfWeek").val()
	var otherDateVal = $("#otherDate").val()
	var quarterlyDateVal = $("#quarterlyDate").val()
	var yearlyMonthVal = $("#yearlyMonth").val()
	var yearlyMonthFixedDayVal = $("#yearlyMonthFixedDay").val()
	var yearlyWeekOfMonthVal = $("#yearlyWeekOfMonth").val()
	var yearlyDayOfWeekVal = $("#yearlyDayOfWeek").val()
	var yearlyRelativeMonthVal = $("#yearlyRelativeMonth").val()
	
	var errors = new Array()
	
	if (scheduleTypeCode == DAILY_CODE && $.trim(dayOfWeekCSV) == '') {
		errors.push('<span tabindex="0">Please select at least one day.</span>')
	} else if (scheduleTypeCode == MONTHLY_DAY_OF_MONTH_CODE && $.trim(relativeDayOfMonthVal) == '') {
		errors.push('<span tabindex="0">Please select the day of the month.</span>')
	} else if (scheduleTypeCode == MONTHLY_WEEK_DAY_CODE) {
		if ($.trim(weekOfMonthVal) == '')
			errors.push('<span tabindex="0">Please select the week of the month.</span>')
		if ($.trim(dayOfWeekVal) == '')
			errors.push('<span tabindex="0">Please select the day of the week.</span>')
	} else if (scheduleTypeCode == OTHER_CODE) {
		if ($.trim(otherDateVal) == '') {
			errors.push('<span tabindex="0">Please enter the day.</span>')
		} else if (!validateDate(otherDateVal)) {
			errors.push('<span tabindex="0">Please enter the day in MM/DD/YYYY format.</span>')
		}
	} else if (scheduleTypeCode == QUARTERLY_CODE && $.trim(quarterlyDateVal) == '') {
		errors.push('<span tabindex="0">Please enter the day of the quarter.</span>')
	} else if (scheduleTypeCode == YEARLY_MONTH_DAY_CODE) {
		if ($.trim(yearlyMonthVal) == '')
			errors.push('<span tabindex="0">Please select the month.</span>')
		if ($.trim(yearlyMonthFixedDayVal) == '' || !isValidDayOfMonthNum(yearlyMonthFixedDayVal))
			errors.push('<span tabindex="0">Please enter a day of the month between 1 and 31.</span>')
	} else if (scheduleTypeCode == YEARLY_WEEK_DAY_MONTH_CODE) {
		if ($.trim(yearlyWeekOfMonthVal) == '')
			errors.push('<span tabindex="0">Please select the week of the month.</span>')
		if ($.trim(yearlyDayOfWeekVal) == '')
			errors.push('<span tabindex="0">Please select the day of the week.</span>')
		if ($.trim(yearlyRelativeMonthVal) == '')
			errors.push('<span tabindex="0">Please select the month.</span>')
	}
	
	if (errors.length > 0) {
		displayAttentionDialog("<span tabindex='0'>Please correct the following errors:</span><ul><li>"
				+ errors.join("</li><li>") + "</li></ul>");
		return
	}
	
	$.ajax({
		url : 'ajax/scheduledJobs/scheduleValidate',
		dataType : 'json',
		data : {
			jobName : job.jobName,
			runFrequency : $("#scheduleChangeFrequency").val(),
			scheduleType : scheduleTypeCode,
			dayOfMonth : relativeDayOfMonthVal,
			weekOfMonth : scheduleTypeCode == MONTHLY_WEEK_DAY_CODE ? weekOfMonthVal :
				scheduleTypeCode == YEARLY_WEEK_DAY_MONTH_CODE ? yearlyWeekOfMonthVal :
					'',
			dayOfWeek : scheduleTypeCode == MONTHLY_WEEK_DAY_CODE ? dayOfWeekVal :
				scheduleTypeCode == YEARLY_WEEK_DAY_MONTH_CODE ? yearlyDayOfWeekVal :
					'',
			dayOfQuarter : quarterlyDateVal,
			month : scheduleTypeCode == YEARLY_MONTH_DAY_CODE ? yearlyMonthVal :
				scheduleTypeCode == YEARLY_WEEK_DAY_MONTH_CODE ? yearlyRelativeMonthVal :
					'',
			numericDayOfMonth : yearlyMonthFixedDayVal,
			dateToGenerate : otherDateVal,
			hour : $("#scheduleChangeStartTimeHour").val(),
			minute : $("#scheduleChangeStartTimeMinute").val(),
			daysOfWeekCSV : dayOfWeekCSV
		},
		error : commonAjaxErrorHandler,
		success : function(result) {
			var r = result.reportSchedule
			var jobScheduleParamsCSV = result.jobScheduleParamsCSV
			
			$("#scheduleJobNextFireTime").text(r.nextFireTime ? r.nextFireTime.formattedDate : '')
			$("#scheduleJobScheduleDescriptionSpan").text(r.friendlyDescription)
			
			$("#scheduleJobCronInput").val(r.cronExpression)
			$("#scheduleJobScheduleDescriptionInput").val(r.friendlyDescription)
			$("#scheduleJobScheduleParamsCSV").val(jobScheduleParamsCSV)
			
			$("#scheduleChangeWrapper").dialog('close')
		}
	})
}

function scheduleChangeFrequencyUpdated(selectFirstScheduleType) {
	var runFrequency = $("#scheduleChangeFrequency").val()
	$("#dailyWeeklyOptions").toggle(runFrequency == '1')
	$("#monthlyOptions").toggle(runFrequency == '2')
	$("#quarterlyOptions").toggle(runFrequency == '3')
	$("#yearlyOptions").toggle(runFrequency == '4')
	$("#otherOptions").toggle(runFrequency == '5')
	
	if (selectFirstScheduleType)
		$("div.scheduleTypeFields:visible input[name='scheduleType']:first").prop('checked', true)
}

function setScheduleChangeFields(selectFirstScheduleType) {
	var job = getCurrentJob()
	
	/* cron */
	$("#scheduleJobCronInput").val(job ? job.jobSchedule : '')
	$("#scheduleJobScheduleDescriptionInput").val(job ? job.jobScheduleDescription : '')
	
	var runFrequency = job && job.reportSchedule && job.reportSchedule.runFrequency ? job.reportSchedule.runFrequency.value : ''
	$("#scheduleChangeFrequency").val(runFrequency)
	
	var scheduleTypeVal = job && job.reportSchedule && job.reportSchedule.scheduleType ? job.reportSchedule.scheduleType.value : ''
	if (scheduleTypeVal != '')
		$("input[name='scheduleType'][value='" + scheduleTypeVal + "']").prop('checked', true)
	
	// days of week
	$("input[name='dailyWeeklyOptions']").prop('checked', false)
	if (job && job.reportSchedule && job.reportSchedule.daysOfWeek) {
		for (var i = 0; i < job.reportSchedule.daysOfWeek.length; i++)
			$("input[name='dailyWeeklyOptions'][value='" + job.reportSchedule.daysOfWeek[i].value + "']").prop('checked', true)
	}
	
	// monthly
	$("#relativeDayOfMonthValue").val(job && job.reportSchedule && job.reportSchedule.dayOfMonth ? job.reportSchedule.dayOfMonth.value : '')
	$("#weekOfMonth").val(job && job.reportSchedule && job.reportSchedule.weekOfMonth ? job.reportSchedule.weekOfMonth.value : '')
	$("#dayOfWeek").val(job && job.reportSchedule && job.reportSchedule.dayOfWeek ? job.reportSchedule.dayOfWeek.value : '')
	$("#quarterlyDate").val(job && job.reportSchedule && job.reportSchedule.dayOfQuarter ? job.reportSchedule.dayOfQuarter.value : '')
	
	// yearly
	$("#yearlyMonth").val(job && job.reportSchedule && job.reportSchedule.month ? job.reportSchedule.month.value : '')
	$("#yearlyMonthFixedDay").val(job && job.reportSchedule && job.reportSchedule.numericDayOfMonth != 0 ? job.reportSchedule.numericDayOfMonth : '')
	$("#yearlyWeekOfMonth").val(job && job.reportSchedule ? job.reportSchedule.weekOfMonth : '')
	$("#yearlyDayOfWeek").val(job && job.reportSchedule && job.reportSchedule.dayOfWeek ? job.reportSchedule.dayOfWeek.value : '')
	$("#yearlyRelativeMonth").val(job && job.reportSchedule ? job.reportSchedule.month : '')
	
	// other
	$("#otherDate").val(job && job.reportSchedule && job.reportSchedule.dateToGenerate ? job.reportSchedule.dateToGenerate.date : '')
	
	$("#scheduleChangeStartTimeHour").val(job && job.reportSchedule ? job.reportSchedule.hour : '')
	$("#scheduleChangeStartTimeMinute").val(job && job.reportSchedule ? job.reportSchedule.minute : '')
	
	scheduleChangeFrequencyUpdated(selectFirstScheduleType)
}

function getCurrentJob() {
	var jobName = $("#currentJobs").data('selectedJobName')
	if (!jobName) return
	
	var job = null
	if (jobName && allJobs[jobName])
		job = allJobs[jobName]
	return job
}

function buildDaysOfWeekCSV() {
	var daysOfWeekCSV = []
	$("input[name='dailyWeeklyOptions']:checked").each(function() {
		daysOfWeekCSV.push($(this).val())
	})
	return daysOfWeekCSV.join(',')
}

/* ---------------------------------- History dialog */

function buildHistoryPopup() {
	$("#historyFieldsWrapper").dialog({
		autoOpen : false,
		modal : true,
		width : 950,
		height : 750,
		closeOnEscape : true,
		draggable : true,
		resizable : true,
		buttons : [{
           id: "historyPopupStatisticsButton",
           text: "Statistics",
           click: popupHistoryStatistics,
           "class": 'statisticsButtonClass'
		}, {
           id: "historyPopupCancelButton",
           text: "Cancel",
           click: function() {
   				$(this).dialog('close')
           }
		}]
	})
	
	$("#historyStatsFieldsWrapper").dialog({
		autoOpen : false,
		modal : true,
		width : 650,
		height : 400,
		closeOnEscape : true,
		draggable : true,
		resizable : true,
		buttons : {
		   "Close" : function() {
   				$(this).dialog('close')
           }
		}
	})
	
	var table = $('#jobHistoryTable').DataTable({
		"columnDefs" : [{
			"targets" : 0,
			"data" : function(row, type, set, meta) {
				return row.formattedStartDate
			}
		}, {
			"targets" : 1,
			"data" : function(row, type, set, meta) {
				return row.formattedEndDate
			}
		}, {
			"targets" : 2,
			"data" : function(row, type, set, meta) {
				return row.runBy
			}
		}, {
			"targets" : 3,
			"data" : function(row, type, set, meta) {
				return row.status
			}
		}, {
			"targets" : 4,
			"data" : function(row, type, set, meta) {
				return row.exitStatus
			}
		}],
		scrollY : "370px",
		paging : true,
		autowidth : true,
		searching : false,
		select : 'single'
	})

	$('#jobHistoryTable tbody').on('click', 'tr', function() {
		var data = table.row(this).data()
		$("#historyFieldsWrapper").data('selectedHistoryEntry', data)
		$(".statisticsButtonClass").show()
	})
}

function popupHistory() {
	var currentJob = getCurrentJob()
	$("#historyFieldsJobName").text(currentJob.jobName)
	$("#historyFieldsDescription").text(currentJob.jobDescription)
	$("#historyFieldsNextScheduledTime").text(currentJob.nextFireTime ? currentJob.nextFireTime.formattedDate : '')
	$("#historyFieldsStatus").text(currentJob.status)
	
	$("#historyFieldsWrapper").dialog('open')
	refreshJobHistory()
}

function refreshJobHistory() {
	var currentJob = getCurrentJob()
	
	var table = $('#jobHistoryTable').DataTable()
	table.clear()
	
	$.ajax({
		url : 'ajax/scheduledJobs/jobHistory',
		dataType : 'json',
		data : {
			jobName : currentJob.jobName,
			jobGroup : currentJob.groupName
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			table.rows.add(r.history)
			table.draw()
		}
	})
}

function popupHistoryStatistics() {
	var job = getCurrentJob()
	var historyEntry = $("#historyFieldsWrapper").data('selectedHistoryEntry')
	$("#statsName").text(job.jobName)
	$("#statsDetails").val(historyEntry.statistics)
	$("#historyStatsFieldsWrapper").dialog('open')
}

/* ---------------------------------- Pause All / Resume All buttons */

function submitPauseAll() {
	doubleClickSafeguard($("#pauseAll"))
	
	confirmDialog('Do you want to pause all the scheduled jobs?', function() {
			$.ajax({
				url : 'ajax/scheduledJobs/pauseAll',
				dataType : 'json',
				error : commonAjaxErrorHandler,
				success : function() {
					showInformationMessage("All the scheduled jobs have been paused successfully.")
					refreshCurrentJobs()
				}
			})
		}, {
			title : 'Pause Scheduled Jobs',
			okButtonLabel : 'Yes',
			cancelButtonLabel : 'No'
		})
}

function submitResumeAll() {
	doubleClickSafeguard($("#resumeAll"))
	
	confirmDialog('Do you want to resume all the scheduled jobs?', function() {
			$.ajax({
				url : 'ajax/scheduledJobs/resumeAll',
				dataType : 'json',
				error : commonAjaxErrorHandler,
				success : function() {
					showInformationMessage("All the scheduled jobs have been resumed successfully.")
					refreshCurrentJobs()
				}
			})
		}, {
			title : 'Resume Scheduled Jobs',
			okButtonLabel : 'Yes',
			cancelButtonLabel : 'No'
		})
}