package gov.va.med.ccht.controller;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import gov.va.med.ccht.model.car.CensusActivityModalityRecord;
import gov.va.med.ccht.model.car.CensusActivityRecord;
import gov.va.med.ccht.model.car.CensusActivityReportParameters;
import gov.va.med.ccht.model.car.CensusActivityReportResult;
import gov.va.med.ccht.model.car.CensusActivitySubTotalRecord;
import gov.va.med.ccht.model.car.CensusActivityVisnRecord;
import gov.va.med.ccht.service.dmpreport.CensusActivityReportService;
import gov.va.med.ccht.ui.common.AbstractController;

@Controller
public class CensusActivityReportController extends AbstractController {

	@Autowired
	private CensusActivityReportService censusActivityReportService;

	public CensusActivityRecord getAllVendorsReportStatus(String reportEndDate) throws Exception {
		CensusActivityRecord retVal = censusActivityReportService.getAllVendorsReportStatus(reportEndDate);
		return retVal;
	}

	public List<CensusActivityReportResult> getReport(CensusActivityReportParameters params) throws Exception {
		int reportType = params.getReportType();
		List<CensusActivityReportResult> results = new ArrayList<CensusActivityReportResult>();
		switch (reportType) {
		case CensusActivityReportParameters.NATIONAL:
			// results =
			// censusActivityReportService.getNationalReport(params);
			break;

		case CensusActivityReportParameters.VISN:
			censusActivityReportService.getAllVisnReport(params);
			break;

		case CensusActivityReportParameters.FACILITY:
			censusActivityReportService.getFacilityReport(params);
			break;
		}
		return results;
	}

	/**
	 * National report when no vendor or modality has been chosen.
	 * 
	 * @param params
	 * @throws Exception
	 */
	public List<CensusActivityRecord> getNationalReport(CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getNationalReport(params);
	}

	public List<CensusActivitySubTotalRecord> getSubTotaledNationalReport(CensusActivityReportParameters params)
			throws Exception {
		return censusActivityReportService.getSubTotaledNationalReport(params);
	}

	/**
	 * National report when a vendor has been selected but no modality has been
	 * chosen.
	 * 
	 * @param params
	 * @throws Exception
	 */
	public List<CensusActivityRecord> getNationalReportByVendor(CensusActivityReportParameters params)
			throws Exception {
		return censusActivityReportService.getNationalReportByVendor(params);
	}

	/**
	 * National report when a vendor has not been selected but a modality has
	 * been chosen.
	 * 
	 * @param params
	 * @throws Exception
	 */
	public List<CensusActivityRecord> getNationalReportByModality(CensusActivityReportParameters params)
			throws Exception {
		return censusActivityReportService.getNationalReportByModality(params);
	}

	public List<CensusActivitySubTotalRecord> getSubTotaledNationalReportByModality(
			CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getSubTotaledNationalReportByModality(params);
	}

	public List<CensusActivityRecord> getNationalReportByModalityAndVendor(CensusActivityReportParameters params)
			throws Exception {
		return censusActivityReportService.getNationalReportByModalityAndVendor(params);
	}

	// VISN LEVEL REPORTS

	// ALL VISN
	/**
	 * VISN report when no vendor or modality has been chosen.
	 * 
	 * @param params
	 * @throws Exception
	 */
	public List<CensusActivityVisnRecord> getAllVisnReport(CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getAllVisnReport(params);
	}

	/**
	 * Subtotaled VISN report when no vendor or modality has been chosen.
	 * 
	 * @param params
	 * @throws Exception
	 */
	public List<CensusActivityVisnRecord> getSubTotaledAllVisnReport(CensusActivityReportParameters params)
			throws Exception {
		return censusActivityReportService.getSubTotaledAllVisnReport(params);
	}

	/**
	 * VISN report when a vendor has been chosen but no modality has been
	 * chosen.
	 * 
	 * @param params
	 * @throws Exception
	 */
	public List<CensusActivityVisnRecord> getVisnReportByVendor(CensusActivityReportParameters params)
			throws Exception {
		return censusActivityReportService.getVisnReportByVendor(params);
	}

	/**
	 * VISN report when a Modality has been chosen, but no Vendor has been
	 * chosen.
	 */
	public List<CensusActivityModalityRecord> getSingleModalityAllVendorAllVisnReport(
			CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getSingleModalityAllVendorAllVisnReport(params);

	}

	/**
	 * Subtotaled VISN report when a Modality has been chosen, but no Vendor has
	 * been chosen.
	 */
	public List<CensusActivityModalityRecord> getSubTotaledSingleModalityAllVendorAllVisnReport(
			CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getSingleModalitySubtotaledVisnReport(params);

	}

	// A SINGLE, SPECIFIC VISN HAS BEEN CHOSEN
	/**
	 * VISN report when no vendor or modality has been chosen.
	 * 
	 * @param params
	 * @throws Exception
	 */
	public List<CensusActivityVisnRecord> getSingleVisnReport(CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getSingleVisnReport(params);
	}

	/**
	 * Subtotaled VISN report when no vendor or modality has been chosen.
	 * 
	 * @param params
	 * @throws Exception
	 */
	public List<CensusActivityVisnRecord> getSubTotaledSingleVisnReport(CensusActivityReportParameters params)
			throws Exception {
		return censusActivityReportService.getSubTotaledSingleVisnReport(params);
	}

	public List<CensusActivityModalityRecord> getSingleVisnSingleModalityAllVendorVisnReport(
			CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getSingleVisnSingleModalityAllVendorVisnReport(params);

	}

	/**
	 * Get a sub totaled report for a specific modality and all vendors
	 */
	public List<CensusActivityModalityRecord> getSingleVisnSingleModalitySubtotaledVisnReport(
			CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getSingleVisnSingleModalitySubtotaledVisnReport(params);
	}

	/**
	 * Get a non sub totaled report for a specific modality and a specific
	 * vendor
	 */
	public List<CensusActivityModalityRecord> getSingleModalitySingleVendorVisnReport(
			CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getSingleModalitySingleVendorVisnReport(params);

	}

	public List<CensusActivityVisnRecord> getSingleVisnAllModalitySingleVendorReport(
			CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getSingleVisnAllModalitySingleVendorReport(params);
	}

	public List<CensusActivityModalityRecord> getSingleVisnSingleModalitySingleVendorReport(
			CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getSingleVisnSingleModalitySingleVendorReport(params);
	}

	/**
	 * Facility drill down data for a selected visn & report week
	 * 
	 * @return
	 * @throws Exception
	 */
	public List<CensusActivityVisnRecord> getFacilityDrillDownData(CensusActivityReportParameters params)
			throws Exception {
		return censusActivityReportService.getFacilityDrilldownData(params);
	}

	/**
	 * Facility drill down data for a selected visn & report week
	 * 
	 * @return
	 * @throws Exception
	 */
	public List<CensusActivityModalityRecord> getDrillDownData(CensusActivityReportParameters params) throws Exception {
		return censusActivityReportService.getModalityFacilityDrilldownData(params);
	}

	public CensusActivityRecord getLastReportWeek() throws Exception {
		return censusActivityReportService.getLastReportWeek();
	}
}
