package gov.va.med.ccht.controller;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.multipart.MaxUploadSizeExceededException;
import org.springframework.web.servlet.ModelAndView;


import gov.va.med.ccht.model.CCHTPermissions;
import gov.va.med.ccht.model.CCHTRoles;
import gov.va.med.ccht.model.User;
import gov.va.med.ccht.model.inventory.Facility;
import gov.va.med.ccht.model.inventory.Vendor;
import gov.va.med.ccht.model.inventory.Visn;
import gov.va.med.ccht.model.terminology.InactiveReason;
import gov.va.med.ccht.model.terminology.RegistrationDeniedReason;
import gov.va.med.ccht.model.terminology.RegistrationStatus;
import gov.va.med.ccht.persistent.InventoryDAO;
import gov.va.med.ccht.persistent.SecurityDAO;
import gov.va.med.ccht.service.common.SecurityService;
import gov.va.med.ccht.service.common.TerminologyCache;
import gov.va.med.ccht.service.common.TerminologyException;
import gov.va.med.ccht.service.inventory.InventoryService;
import gov.va.med.ccht.service.qir.QIRService;
import gov.va.med.ccht.service.report.ScheduleConversionService;
import gov.va.med.ccht.ui.ajax.AjaxRequestHandler;
import gov.va.med.ccht.ui.common.AbstractController;
import gov.va.med.ccht.ui.common.UserNotifier;
import gov.va.med.ccht.ui.interceptor.CommonReferenceDataInterceptor;
import gov.va.med.ccht.ui.model.QIRForm;
import gov.va.med.ccht.ui.model.UserForm;
import gov.va.med.fw.batchprocess.BatchProcessInvoker;
import gov.va.med.fw.batchprocess.BatchProcessService;
import gov.va.med.fw.model.lookup.Lookup;
import gov.va.med.fw.scheduling.SchedulingService;
import gov.va.med.fw.security.SecurityContextHelper;
import gov.va.med.fw.security.SimpleRole;
import gov.va.med.fw.security.UserPrincipal;
import gov.va.med.fw.service.jms.MessageProducerService;
import gov.va.med.fw.ui.model.TermType;


public abstract class CommonCchtController {

	public final static String NEGATIVE_ONE = "-1";

	public static final TermType NATIONAL_ADMIN_ROLE = new TermType(AbstractController.NATIONAL_ADMIN,
			AbstractController.NATIONAL_ADMIN);
	public static final TermType VISN_ADMIN_ROLE = new TermType(AbstractController.VISN_ADMIN,
			AbstractController.VISN_ADMIN);
	public static final TermType FACILITY_ADMIN_ROLE = new TermType(AbstractController.FACILITY_ADMIN,
			AbstractController.FACILITY_ADMIN);
	public static final TermType APPLICATION_ADMIN_ROLE = new TermType(AbstractController.APPLICATION_ADMIN,
			AbstractController.APPLICATION_ADMIN);
	public static final TermType MANAGEMENT_ROLE = new TermType(AbstractController.MANAGEMENT,
			AbstractController.MANAGEMENT);
	public static final TermType DMP_ADMIN_ROLE = new TermType(AbstractController.DMP_ADMIN,
			AbstractController.DMP_ADMIN);
	public static final TermType DMP_ROLE = new TermType(AbstractController.DMP, AbstractController.DMP);
	public static final TermType VENDOR_ROLE = new TermType(AbstractController.VENDOR, AbstractController.VENDOR);
	public static final TermType NAC_POC_ROLE = new TermType(AbstractController.NAC, AbstractController.NAC);
	public static final TermType SYSTEM_ADMIN_ROLE = new TermType(AbstractController.SYSTEM_ADMIN,
			AbstractController.SYSTEM_ADMIN);
	public static final TermType CARE_COORDINATOR_ROLE = new TermType(AbstractController.CARE_COORDINATOR,
			AbstractController.CARE_COORDINATOR);
	public static final TermType PROGRAM_ASSISTANT_ROLE = new TermType(AbstractController.PROGRAM_ASSISTANT,
			AbstractController.PROGRAM_ASSISTANT);
	public static final TermType QIR_ORIGINATOR_ROLE = new TermType(AbstractController.QIR_ORIGINATOR,
			AbstractController.QIR_ORIGINATOR);
	
	public static final TermType REPORTS_ONLY_ROLE = new TermType(AbstractController.REPORTS_ONLY,
			AbstractController.REPORTS_ONLY); 
	
	@Autowired
	protected AjaxRequestHandler ajaxRequestHandler;
	@Autowired
	protected BatchProcessService batchProcessService;
	@Autowired
	protected BatchProcessInvoker batchProcessInvoker;
	@Autowired
	@Resource(name = "common.jms.producer.batchProcessEvents")
	protected MessageProducerService batchProcessMessageService;
	@Autowired
	protected InventoryDAO inventoryDAO;
	@Autowired
	protected InventoryService inventoryService;
	@Autowired
	protected QIRService qirService = null;
	@Autowired
	protected ReportConversionService reportConversionService;
	@Autowired
	protected ScheduleConversionService scheduleConversionService;
	@Autowired
	protected SchedulingService schedulingService;
	@Autowired
	protected SecurityConversionService securityConversionService;
	@Autowired
	protected SecurityDAO securityDAO;
	@Autowired
	protected SecurityService securityService;
	@Autowired
	protected TerminologyCache terminologyCache;
	@Autowired
	protected TerminologyCacheController terminologyCacheController;
	@Autowired
	protected TerminologyController terminologyController;
	@Autowired
	protected UserNotifier userNotifier;


	@Value("#{cchtProperties['basePath']}")
	protected String basePath;

	protected boolean isAjax(HttpServletRequest r) {
		return AjaxRequestHandler.isAjax(r);
	}

	@ExceptionHandler(MaxUploadSizeExceededException.class)
	public String handleIOException(MaxUploadSizeExceededException ex, HttpServletRequest request) {
		return "test";
	}

	@ExceptionHandler(Throwable.class)
	public ModelAndView processError(Throwable ex, HttpServletRequest request, HttpServletResponse response) {
		if (isAjax(request)) {
			/*
			 * Necessary to trigger the jQuery error() handler as opposed to the
			 * success() handler - CPB
			 */
			response.setStatus(HttpStatus.SC_BAD_REQUEST);
			return ajaxRequestHandler.getExceptionModelAndView(ex, request);
		} else {
			return new ModelAndView("error", "exceptionStackTrace", ExceptionUtils.getFullStackTrace(ex));
		}
	}

	/* Convenience methods */

	public static User getCurrentUser() {
		return CommonReferenceDataInterceptor.getCurrentUser();
	}

	public static String getCurrentUsername() {
		return CommonReferenceDataInterceptor.getCurrentUsername();
	}

	public static String getUserName() {
		return CommonReferenceDataInterceptor.getCurrentUserDisplayName();
	}

	public Map<String, RegistrationDeniedReason> getRegistrationDeniedReasons(boolean insertSelect) throws Exception {
		
		Map<String, RegistrationDeniedReason> retVal =  new LinkedHashMap<String, RegistrationDeniedReason>();
				
		
		if(insertSelect) {
			RegistrationDeniedReason val = new RegistrationDeniedReason();
			val.setName("Select");
			retVal.put(NEGATIVE_ONE, val);
		}
		
		Map<String, RegistrationDeniedReason> regDeniedReasons = terminologyCacheController.getRegistrationDeniedLookUpMap();
		
		retVal.putAll(regDeniedReasons);
		return retVal;
		
	}
	
	public Map<String, Vendor> getVendors(boolean insertSelect) throws Exception {
		Map<String, Vendor> retVal = null;

		List<Vendor> vendorList = inventoryService.getVendors();
		retVal = new LinkedHashMap<String, Vendor>();
		if (insertSelect) {
			Vendor selectVendor = new Vendor();
			selectVendor.setName("Select Vendor");
			retVal.put(NEGATIVE_ONE, selectVendor);
		}
		for (Vendor vendor : vendorList) {
			retVal.put(vendor.getName(), vendor);
		}

		return retVal;
	}

	public Map<String, Visn> getVisns(boolean insertSelect) throws Exception {
		Map<String, Visn> retVal = null;
		List<Visn> visns = inventoryService.getVisns();
		retVal = new LinkedHashMap<String, Visn>();

		if (insertSelect) {
			Visn selectVisn = new Visn();
			selectVisn.setName("Select VISN");
			retVal.put(NEGATIVE_ONE, selectVisn);
		}

		for (Visn visn : visns) {
			retVal.put(visn.getCode().trim(), visn);
		}

		return retVal;
	}

	public Map<String, Facility> getFacilities(boolean insertSelect) throws Exception {
		Map<String, Facility> retVal = null;

		SortedSet<Facility> list = inventoryService.getFacilities();
		retVal = new LinkedHashMap<String, Facility>();

		// new facility
		if (insertSelect) {
			Facility selecFacility = new Facility();
			selecFacility.setName("Select Facility");
			retVal.put(NEGATIVE_ONE, selecFacility);
		}
		// get facility
		for (Facility facility : list) {
			retVal.put(facility.getCode().trim(), facility);
		}

		return retVal;
	}
	
	
	public Map<String, InactiveReason> getInactiveReasons(boolean insertSelect) throws Exception {
		
		Map<String, InactiveReason> retVal = new LinkedHashMap<String, InactiveReason>();
		
		if (insertSelect) {
			InactiveReason reason = new InactiveReason();
			reason.setName("Select");
			retVal.put(NEGATIVE_ONE, reason);
		}
		
		List<InactiveReason> inactiveReasons = terminologyCacheController.getInactivationReasons();
		
		for(InactiveReason item : inactiveReasons) {
			retVal.put(item.getCode(), item);
		}
		
		
		return retVal;
	}
	

	public Map<String, Vendor> getVendors() throws Exception {
		Map<String, Vendor> retVal = null;

		List<Vendor> vendorList = inventoryService.getVendors();
		retVal = new LinkedHashMap<String, Vendor>();
		for (Vendor vendor : vendorList) {
			retVal.put(vendor.getCode() , vendor);
		}

		return retVal;
	}

	public Map<String, Visn> getVisns() throws Exception {
		Map<String, Visn> retVal = null;
		List<Visn> visns = inventoryService.getVisns();
		retVal = new LinkedHashMap<String, Visn>();

		for (Visn visn : visns) {
			retVal.put(visn.getCode(), visn);
		}

		return retVal;
	}

	public Map<String, Facility> getFacilities() throws Exception {
		Map<String, Facility> retVal = null;

		SortedSet<Facility> list = inventoryService.getFacilities();
		retVal = new LinkedHashMap<String, Facility>();
		for (Facility facility : list) {
			retVal.put(facility.getCode(), facility);
		}

		return retVal;
	}

	public Map<Long, String> getAllRoles() throws Exception {
		Map<Long, String> retVal = null;
		List<SimpleRole> simpleRoles = securityService.getAllSimpleRoles();
		retVal = new LinkedHashMap<Long, String>();

		for (SimpleRole simpleRole : simpleRoles) {
			retVal.put(simpleRole.getId(), simpleRole.getName());
		}

		return retVal;
	}

	/**
	 * Returns the available roles that the logged in user can assign.
	 * 
	 * @return Map<String, String>
	 * @throws Exception
	 */
	public Map<String, String> getAvailableRoles() throws Exception {
		Map<String, String> retVal = new LinkedHashMap<String, String>();

		UserForm userForm = getUser(getCurrentUser().getUsername());
		List<TermType> availableRoles = userForm.getAvailableRoles();

		for (TermType role : availableRoles) {
			String roleId = role.getValue();
			String roleName = role.getLabel();
			retVal.put(roleId, roleName);
		}

		// Add assigned roles to available roles 
		List<TermType> assignedRoles = userForm.getRoles();
		for(TermType role : assignedRoles) {
			String roleId = role.getValue();
			String roleName = role.getLabel();
			retVal.put(roleId, roleName);
		}
		return retVal;
	}

	public Map<String, String> getUserGroups() throws Exception {
		return terminologyCacheController.getUserGroups();
	}

	public Map<String, String> getTerms(String className) throws TerminologyException {
		Map<String, String> retVal = new LinkedHashMap<String, String>();
		List<? extends Lookup> terms = terminologyCache.getTerms(className);

		for (Lookup term : terms) {
			retVal.put(term.getCode(), term.getName());
		}

		return retVal;
	}

	public Map<String, RegistrationStatus> getRegistrationStatuses() throws Exception {
		return terminologyCacheController.getRegistrationStatusLookUpMap();
	}

	public Map<String, String> getJobDescriptions() throws Exception {

		return terminologyCacheController.getJobDescriptions();
	}

	public boolean canEditIsDmpIssue(QIRForm form) {
		boolean canSet = false;
		UserPrincipal up = SecurityContextHelper.getCurrentUser();

		if (!up.isPermissionGranted(CCHTPermissions.NATIONAL_ADMIN)) {
			return canSet;
		}
		if (form.getQirStatusType().getLabel().equalsIgnoreCase("Closed")
				|| form.getQirStatusType().getLabel().equalsIgnoreCase("Withdrawn")) {
			return canSet;
		}

		return true;
	}

	public boolean isVendor() {
		UserPrincipal up = SecurityContextHelper.getCurrentUser();

		if (up != null) {
			return up.isPermissionGranted(CCHTPermissions.VENDOR);
		} else {
			return false;
		}

	}

	public boolean isNationalAdmin() {
		UserPrincipal up = SecurityContextHelper.getCurrentUser();

		if (up != null) {
			return up.isPermissionGranted(CCHTPermissions.NATIONAL_ADMIN);
		} else {
			return false;
		}
	}
	
	
	/**
	 * Check to see if the logged in APP user is an app admin
	 * @param currentUser
	 * @return
	 */
	public boolean isApplicationAdmin(User currentUser) {		
		return currentUser.isPermissionGranted(CCHTPermissions.APPLICATION_ADMIN);		
	}

	public UserForm getUser(String userName) throws Exception {
		UserPrincipal currentUser = getCurrentUser();
		User user = securityService.getUser(userName);
		UserForm userForm = new UserForm();
		securityConversionService.convert(user, userForm);
		List<TermType> assignedRoles = userForm.getRoles();
		List<TermType> availableRoles = new ArrayList<TermType>(); // getAllRoles();
		List<TermType> allRoles = new ArrayList<TermType>(); // getAllRoles();
		List<SimpleRole> roles;
		if (currentUser.isPermissionGranted(CCHTRoles.DMP_ADMIN)
				&& currentUser.isPermissionGranted(CCHTRoles.NATIONAL_ADMIN)) {
			roles = securityService.getAllSimpleRoles();
		} else if (currentUser.isPermissionGranted(CCHTRoles.DMP_ADMIN)) {
			roles = securityService.getAllDMPRoles();
		} else {
			roles = securityService.getAllSimpleRoles();
		}
		for (SimpleRole role : roles) {
			allRoles.add(new TermType(role.getName(), role.getName()));
		}
		// filter the available roles based on the user role
		if (currentUser.isPermissionGranted(CCHTRoles.NATIONAL_ADMIN)) {
			// no filtering
			availableRoles = allRoles;
		} else {
			// availableRoles.remove(NATIONAL_ADMIN_ROLE);
			// availableRoles.remove(APPLICATION_ADMIN_ROLE);
			// availableRoles.remove(MANAGEMENT_ROLE);
			User loginUser = (User) currentUser;
			boolean canAddRole = false;
			if (currentUser.isPermissionGranted(CCHTRoles.FACILITY_ADMIN)) {

				if (loginUser.getFacility() != null) {
					if (user.getFacility() != null && (loginUser.getFacility().getId().longValue() == user.getFacility()
							.getId().longValue())) {
						canAddRole = true;
					} else if (user.getSecondaryFacility() != null && (loginUser.getFacility().getId()
							.longValue() == user.getSecondaryFacility().getId().longValue())) {
						canAddRole = true;
					}
				} else if (loginUser.getSecondaryFacility() != null) {
					if (user.getFacility() != null && (loginUser.getSecondaryFacility().getId().longValue() == user
							.getFacility().getId().longValue())) {
						canAddRole = true;
					} else if (user.getSecondaryFacility() != null && (loginUser.getSecondaryFacility().getId()
							.longValue() == user.getSecondaryFacility().getId().longValue())) {
						canAddRole = true;
					}
				}
				if (canAddRole) {
					availableRoles.add(FACILITY_ADMIN_ROLE);
					availableRoles.add(CARE_COORDINATOR_ROLE);
					availableRoles.add(PROGRAM_ASSISTANT_ROLE);
					//availableRoles.add(APPLICATION_ADMIN_ROLE);
					availableRoles.add(QIR_ORIGINATOR_ROLE);
					
					//TODO : Enforcing Reports Only being viewable for Facility Admin
					availableRoles.add(REPORTS_ONLY_ROLE);
				}
			}
			canAddRole = false;
			if (currentUser.isPermissionGranted(CCHTRoles.VISN_ADMIN)) {
				// keep Facility Administrator, Care Coordinator, QIR
				// Originator, Program Support Assistant, Application
				// Administrator
				// if user and current user are in the same VISN, otherwise,
				// remove all
				if (loginUser.getVisn() != null && user.getVisn() != null
						&& (loginUser.getVisn().getId().longValue() == user.getVisn().getId().longValue())) {
					canAddRole = true;
				}
				if (canAddRole) {
					if (!availableRoles.contains(VISN_ADMIN_ROLE))
						availableRoles.add(VISN_ADMIN_ROLE);
					if (!availableRoles.contains(FACILITY_ADMIN_ROLE))
						availableRoles.add(FACILITY_ADMIN_ROLE);
					if (!availableRoles.contains(CARE_COORDINATOR_ROLE))
						availableRoles.add(CARE_COORDINATOR_ROLE);
					if (!availableRoles.contains(QIR_ORIGINATOR_ROLE))
						availableRoles.add(QIR_ORIGINATOR_ROLE);
					if (!availableRoles.contains(PROGRAM_ASSISTANT_ROLE))
						availableRoles.add(PROGRAM_ASSISTANT_ROLE);
					
					//TODO : Enforcing Reports Only being viewable for VISN Admin
					if (!availableRoles.contains(REPORTS_ONLY_ROLE))
					availableRoles.add(REPORTS_ONLY_ROLE);
				}
			}
			if (currentUser.isPermissionGranted(CCHTRoles.DMP_ADMIN)) {
				if (!availableRoles.contains(DMP_ADMIN_ROLE))
					availableRoles.add(DMP_ADMIN_ROLE);
				if (!availableRoles.contains(DMP_ROLE))
					availableRoles.add(DMP_ROLE);
			}
		}

		availableRoles.removeAll(assignedRoles);
		userForm.setAvailableRoles(availableRoles);

		return userForm;
	}

}
