package gov.va.med.ccht.controller;

import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.naming.SizeLimitExceededException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.context.request.WebRequest;

import gov.va.med.ccht.controller.validators.QIRFormValidator;
import gov.va.med.ccht.model.CCHTPermissions;
import gov.va.med.ccht.model.User;
import gov.va.med.ccht.model.qir.QIR;
import gov.va.med.ccht.model.qir.QIRStatusType;
import gov.va.med.ccht.model.qir.QIRType;
import gov.va.med.ccht.ui.model.QIRForm;
import gov.va.med.fw.model.EntityKey;
import gov.va.med.fw.model.EntityKeyFactory;
import gov.va.med.fw.model.lookup.Lookup;
import gov.va.med.fw.service.ServiceException;
import gov.va.med.fw.ui.conversion.ConversionServiceException;
import gov.va.med.fw.ui.model.TermType;
import gov.va.med.fw.util.DateUtils;

@Controller
@SessionAttributes(value = { "command" })
public class EditQIRContoller extends CommonCchtController {

	@Autowired
	private QIRConversionService qiRConversionService;
	@Autowired
	private QIRFormValidator validator;
	@Value("#{cchtProperties['qir.maxCombinedAttachmentBytes']}")
	private int maxCombinedAttachmentLength;

    @InitBinder
    public void initBinder(WebDataBinder binder)
    {
    	String[] allowedFields = {"isDmpIssue", "dmpQirType", "complaint" , "remarks", "vendorActions", "attachments"};
    	binder.setAllowedFields(allowedFields);
    }
    
    
	@RequestMapping(value = "/manageQirs.html", method = RequestMethod.GET)
	public String showEditQirForm(@RequestParam String selectedQIR, @RequestParam String fromPage,
			@RequestParam String editMode, Model model) throws Exception {
		QIRForm qirForm = buildForm(selectedQIR, fromPage, "View".equals(editMode));
		model.addAttribute("command", qirForm);
		addReferenceData(model, qirForm);
		return "editQir";
	}

	@RequestMapping(value = "/manageQirs.html", params = { "PDF" }, method = RequestMethod.GET)
	public String doPDFRequest(@RequestParam String selectedQIR, Model model) throws Exception {
		return "CCHT";
	}

	@RequestMapping(value = "/editQir.html", method = { RequestMethod.POST }, params = { "Update" })
	public String submitUpdateQIR(@ModelAttribute("command") @Valid QIRForm command, BindingResult result, Model model,
			SessionStatus status, HttpSession session, HttpServletRequest request) throws Exception {
		validator.validateEditQir(command, result);
		return processSubmission(command, result, model, status, request, null);
	}

	@RequestMapping(value = "/editQir.html", method = { RequestMethod.POST }, params = { "Agree" })
	public String submitAgreeQIR(@ModelAttribute("command") QIRForm command, BindingResult result, Model model,
			SessionStatus status, HttpServletRequest request) throws Exception {
		validator.validateAgreeQir(command, result);
		return processSubmission(command, result, model, status, request, new TermType("Agreed", QIRStatusType.AGREED));
	}

	@RequestMapping(value = "/editQir.html", method = { RequestMethod.POST }, params = { "Approve" })
	public String submitApproveQIR(@ModelAttribute("command") QIRForm command, BindingResult result, Model model,
			SessionStatus status, HttpServletRequest request) throws Exception {
		validator.validateApproveQir(command, result);
		return processSubmission(command, result, model, status, request,
				new TermType("Approved", QIRStatusType.APPROVED));
	}

	@RequestMapping(value = "/editQir.html", method = { RequestMethod.POST, RequestMethod.GET }, params = {
			"Withdraw" })
	public String submitWithdrawQIR(@ModelAttribute("command") QIRForm command, BindingResult result, Model model,
			SessionStatus status, HttpServletRequest request) throws Exception {
		return processSubmission(command, result, model, status, request,
				new TermType("Withdrawn", QIRStatusType.WITHDRAWN));
	}

	@RequestMapping(value = "/editQir.html", method = { RequestMethod.POST }, params = { "Reply" })
	public String submitReplyQIR(@ModelAttribute("command") QIRForm command, BindingResult result, Model model,
			SessionStatus status, HttpServletRequest request) throws Exception {
		validator.validateReplyQir(command, result);
		return processSubmission(command, result, model, status, request,
				new TermType("Replied", QIRStatusType.REPLIED));
	}

	@RequestMapping(value = "/editQir.html", method = { RequestMethod.POST }, params = { "Close" })
	public String submitCloseQIR(@ModelAttribute("command") QIRForm command, BindingResult result, Model model,
			SessionStatus status, HttpServletRequest request) throws Exception {
		return processSubmission(command, result, model, status, request, new TermType("Closed", QIRStatusType.CLOSED));
	}

	private String processSubmission(QIRForm command, BindingResult result, Model model, SessionStatus status,
			HttpServletRequest request, TermType newStatus)
			throws Exception, ServiceException, ConversionServiceException, SizeLimitExceededException {
		if (result.hasErrors()) {
			addReferenceData(model, command);
			return "editQir";
		}

		EntityKey<QIR> entityKey = EntityKeyFactory.createEntityKey(new Long(command.getId()),
				new Integer(command.getVersionNumber()), QIR.class);
		QIR qir = qirService.getQIR(entityKey);
		if (qir != null) {
			if (newStatus != null)
				command.setQirStatusType(newStatus);

			qiRConversionService.convert(command, qir);
			qirService.updateQIR(qir);

			try {
				NewQIRController.addAttachments(command, qir, terminologyCache, qirService, request,
						maxCombinedAttachmentLength);
			} catch (SizeLimitExceededException e) {
				result.reject("fileSizeExceeded");
				addReferenceData(model, command);
				return "editQir";
			}
		}

		userNotifier.notifyUserOnceWithMessage(request, "QIR updated successfully.");
		status.setComplete();
		return "redirect:" + basePath + command.getFromPage();
	}

	@RequestMapping(value = "/editQir.html", method = { RequestMethod.POST, RequestMethod.GET }, params = { "PDF" })
	public String showPDF(Model model) {
		return "CCHT";
	}

	private boolean enableUpdate(QIRForm qir) {
		User user = getCurrentUser();
		if (canApprove())
			// national admin
			return true;

		if (qir.isSameSubmittedBy(user) && (canCreate() || canWithdraw() || canUpdate()))
			return true;

		if (isVendor() && canUpdate() && qir.isSameVendor(user))
			return true;

		if (isVisnAdmin() && qir.isSameVisn(user))
			return true;

		if (isFacilityAdmin() && qir.isSameFacility(user))
			return true;

		return false;
	}

	private void addReferenceData(Model model, QIRForm qirForm) throws Exception {
		User user = getCurrentUser();

		boolean enableSubmit = false;
		boolean enableReply = false;
		boolean enableApprove = false;
		boolean enableWithdraw = false;
		boolean enableAgree = false;
		boolean enableClose = false;
		boolean enableVendorResponseDate = false;
		boolean enableRemarks = false;
		boolean enableActions = false;

		Date vendorResponseDueDate = qirForm.getVendorResponseDueDate();
		if (qirForm.isClosed() || qirForm.isWithdrawn()) {
			// leave them all false
		} else {
			enableSubmit = enableUpdate(qirForm);
			enableVendorResponseDate = vendorResponseDueDate != null;

			if (qirForm.isNew()) {
				enableApprove = canApprove();
				enableWithdraw = user.isPermissionGranted(CCHTPermissions.NATIONAL_ADMIN)
						|| (canWithdraw() && qirForm.isSameSubmittedBy(user));
			} else if (qirForm.isApproved()) {
				if (user.getVendor() != null) {
					enableReply = canReply() && qirForm.isSameVendor(user);
				}
				enableClose = canClose();
			} else if (qirForm.isReplied()) {
				enableActions = canReply() && qirForm.isSameVendor(user);
				enableAgree = canAgree() && qirForm.isSameSubmittedBy(user);
				enableClose = canClose();
				enableReply = canReply() && qirForm.isSameVendor(user);
				enableVendorResponseDate = vendorResponseDueDate != null;
			} else if (qirForm.isAgreed()) {
				enableClose = canClose();
				enableReply = canReply() && qirForm.isSameVendor(user);
				enableVendorResponseDate = vendorResponseDueDate != null;
			}

			enableActions |= enableReply;

			// if vendor - disable recs and remarks
			enableRemarks = !enableReply || !enableActions;
		}
		model.addAttribute("canSubmit", enableSubmit);
		model.addAttribute("canApprove", enableApprove);
		model.addAttribute("canWithdraw", enableWithdraw);
		model.addAttribute("canAgree", enableAgree);
		model.addAttribute("canReply", enableReply);
		model.addAttribute("canClose", enableClose);

		model.addAttribute("userIsVendor", isVendor());

		model.addAttribute("title", buildEditFormTitle(qirForm));

		model.addAttribute("enableRemarks", enableRemarks);
		model.addAttribute("canEditIsDmpIssue", canEditIsDmpIssue(qirForm));

		model.addAttribute("showVendorResponseDueDate", enableVendorResponseDate);
		if (enableVendorResponseDate) {
			String vendorResponseDueDateAsString = DateUtils.format(vendorResponseDueDate, DateUtils.MMDDYYYY);
			model.addAttribute("vendorResponseDueDateAsString", vendorResponseDueDateAsString);
		}

	}

	private QIRForm buildForm(String selectedQIR, String fromPage, boolean readOnly) throws Exception {
		EntityKey<QIR> entityKey = EntityKeyFactory.createEntityKey(new Long(selectedQIR), QIR.class);
		QIR qir = qirService.getQIR(entityKey);

		QIRForm qirForm = new QIRForm();
		qirForm.setId(selectedQIR);
		qiRConversionService.convert(qir, qirForm);
		qirForm.setFromPage(fromPage);
		qirForm.setReadOnly(readOnly);
		qirForm.setSubmittedDateAsString(DateUtils.format(qir.getSubmittedDate(), DateUtils.MMDDYYYY));

		List<? extends Lookup> terms = terminologyCache.getTerms("gov.va.med.ccht.model.qir.DmpQirType");
		Map<String, String> dmpQirTypes = new LinkedHashMap<String, String>();
		for (Lookup dmpQirType : terms) {
			gov.va.med.ccht.model.qir.DmpQirType t = (gov.va.med.ccht.model.qir.DmpQirType) dmpQirType;
			dmpQirTypes.put(t.getCode().toString(), t.getName());
		}
		qirForm.setDmpQirTypes(dmpQirTypes);

		List<QIRType> qirTypes = terminologyCache.getTerms(QIRType.class);
		Map<String, String> qirTypeMap = new LinkedHashMap<String, String>();
		for (Lookup t : qirTypes) {
			qirTypeMap.put(t.getCode().toString(), t.getName());
		}
		qirForm.setQirTypes(qirTypeMap);

		// model.addAttribute("approvedStatus", isApprovedStatus(qirForm));

		return qirForm;
	}

	private String buildEditFormTitle(QIRForm qirForm) {
		StringBuilder title = new StringBuilder();
		title.append("TN: ");
		title.append(qirForm.getId());
		title.append(" - ");

		title.append(qirForm.getQirStatusType().getLabel());
		title.append(" - ");

		title.append(qirForm.getHeadline());
		title.append("; ");

		title.append(getUserName());
		title.append(" ");

		title.append(qirForm.getFacility().getLabel());
		return title.toString();
	}

	/* Helper CRUD functions */

	private boolean canApprove() {
		return getCurrentUser().isPermissionGranted(CCHTPermissions.APPROVE_QIR);
	}

	private boolean canCreate() {
		return getCurrentUser().isPermissionGranted(CCHTPermissions.CREATE_QIR);
	}

	private boolean canWithdraw() {
		return getCurrentUser().isPermissionGranted(CCHTPermissions.WITHDRAW_QIR);
	}

	private boolean canReply() {
		return getCurrentUser().isPermissionGranted(CCHTPermissions.REPLY_QIR);
	}

	private boolean canUpdate() {
		return getCurrentUser().isPermissionGranted(CCHTPermissions.UPDATE_QIR);
	}

	private boolean canClose() {
		return getCurrentUser().isPermissionGranted(CCHTPermissions.CLOSE_QIR);
	}

	private boolean canAgree() {
		return getCurrentUser().isPermissionGranted(CCHTPermissions.AGREE_QIR);
	}

	public boolean canEditIsDmpIssue(QIRForm form) {
		return isNationalAdmin() && !form.isClosed() && !form.isWithdrawn();
	}

	// ------- Roles

	public boolean isNationalAdmin() {
		return getCurrentUser().isPermissionGranted(CCHTPermissions.NATIONAL_ADMIN);
	}

	private boolean isFacilityAdmin() {
		return getCurrentUser().isPermissionGranted(CCHTPermissions.FACILITY_ADMIN);
	}

	private boolean isVisnAdmin() {
		return getCurrentUser().isPermissionGranted(CCHTPermissions.VISN_ADMIN);
	}

	public boolean isVendor() {
		return getCurrentUser().isPermissionGranted(CCHTPermissions.VENDOR);
	}

}
