package gov.va.med.ccht.controller;

import javax.servlet.http.HttpSession;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import gov.va.med.ccht.model.CCHTRoles;
import gov.va.med.fw.security.SecurityContext;
import gov.va.med.fw.security.SecurityContextHelper;
import gov.va.med.fw.security.UserPrincipal;

@Controller
public class HomeController extends CommonCchtController {
	private static final Logger log = LoggerFactory.getLogger(HomeController.class);

	@Value("#{cchtProperties['session.expirationSeconds']}")
	private int sessionExpirationSeconds;

	@RequestMapping("/keepAlive")
	public @ResponseBody boolean keepAlive() {
		log.debug("Received request to keep-alive the session...");
		return true;
	}

	@RequestMapping(value = "/home.html", method = RequestMethod.GET)
	public String goHome(HttpSession session) throws Exception {
		session.setMaxInactiveInterval(sessionExpirationSeconds);
		return "home";
	}

	@RequestMapping(value = "/error.html", method = RequestMethod.GET)
	public String showError() throws Exception {
		return "error";
	}

	@RequestMapping(value = "/errorFileSize.html", method = RequestMethod.GET)
	public String showErrorFileSizeExceeded() throws Exception {
		return "errorFileSize";
	}
	
	
	@RequestMapping(value = "/help.html", method = RequestMethod.GET)
	public String showHelp(Model model) throws Exception {
		
		SecurityContext securityContext = SecurityContextHelper
				.getSecurityContext();
		UserPrincipal currentUser = securityContext.getUserPrincipal();
		boolean showAdminHelp = true;
		if (currentUser.isPermissionGranted(CCHTRoles.VENDOR)) {
			showAdminHelp = false;
		}
		boolean showInventoryHelp = true;
		if (currentUser.isPermissionGranted(CCHTRoles.VENDOR)) {
			showInventoryHelp = false;
		}
		boolean showQirHelp = true;
		boolean showDmpHelp = false;
		
		if (currentUser.isPermissionGranted(CCHTRoles.DMP_ADMIN) ||  currentUser.isPermissionGranted(CCHTRoles.DMP)) {
			showDmpHelp = true;
		}	
		
		// Forced back to false while DMP is inactivated. 
		showDmpHelp = false;
		
		boolean showReportsOnlyHelp = false;
		if (currentUser.isPermissionGranted(CCHTRoles.REPORTS_ONLY)) {
			//showReportsOnlyHelp = true;
			showAdminHelp = false;
			showQirHelp = false;
			showInventoryHelp = false;
		}
		
	model.addAttribute("showAdminHelp", showAdminHelp);
	model.addAttribute("showInventoryHelp", showInventoryHelp);
	model.addAttribute("showQirHelp", showQirHelp);
	model.addAttribute("showDmpHelp", showDmpHelp);
	model.addAttribute("showReportsOnlyHelp", showReportsOnlyHelp);
	// Forced to false until we deploy HT Reports
	model.addAttribute("showReportsHelp", false);
		
		
		return "help";
	}
	
	
	
}
