package gov.va.med.ccht.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;

import org.springframework.stereotype.Controller;

import gov.va.med.ccht.model.inventory.DIReportData;
import gov.va.med.ccht.model.inventory.DIReportParameters;
import gov.va.med.ccht.model.inventory.Device;
import gov.va.med.ccht.model.inventory.DeviceDetail;
import gov.va.med.ccht.model.inventory.DeviceRequirement;
import gov.va.med.ccht.model.inventory.DeviceSearchParameters;
import gov.va.med.ccht.model.inventory.DeviceSearchResult;
import gov.va.med.ccht.model.inventory.DeviceType;
import gov.va.med.ccht.model.inventory.Facility;
import gov.va.med.ccht.model.inventory.ReportWeek;
import gov.va.med.ccht.model.inventory.SimpleFacility;
import gov.va.med.ccht.model.inventory.SimpleVisn;
import gov.va.med.ccht.model.inventory.VCReportData;
import gov.va.med.ccht.model.inventory.Vendor;
import gov.va.med.ccht.model.inventory.Visn;
import gov.va.med.ccht.service.report.ReportConstants;
import gov.va.med.ccht.ui.common.AbstractController;
import gov.va.med.ccht.ui.common.ControllerException;
import gov.va.med.ccht.ui.common.ControllerResult;
import gov.va.med.ccht.ui.model.DIReportParameterForm;
import gov.va.med.ccht.ui.model.DeviceDetailForm;
import gov.va.med.ccht.ui.model.DeviceRequirementForm;
import gov.va.med.fw.model.EntityKey;
import gov.va.med.fw.model.EntityKeyFactory;
import gov.va.med.fw.service.ServiceOptimisticLockException;
import gov.va.med.fw.ui.model.TermType;
import gov.va.med.fw.util.StringUtils;

@Controller
public class InventoryController extends AbstractController implements ReportConstants {

	public List<DIReportData> searchDeviceBySerialNumber(DeviceSearchParameters deviceSearchParameters)
			throws Exception {
		List<Object[]> data = inventoryService.searchDevice(deviceSearchParameters);
		List<DIReportData> results = new ArrayList<DIReportData>();
		String searchType = deviceSearchParameters.getSearchType().getValue();
		if (DeviceSearchParameters.DEVICE_SEARCH_TYPE_SERIAL_NUMBER.equals(searchType)) {
			for (Object[] objs : data) {
				DIReportData record = new DIReportData();
				record.deviceSerialNumber = (String) objs[0];
				record.facilityName = (String) objs[1];
				record.vendorName = (String) objs[2];
				record.deviceCount = (Integer) objs[3];
				results.add(record);
			}
		} else {
			throw new Exception("Invalid Device search type" + searchType);
		}
		return results;
	}

	public List<DeviceSearchResult> searchDevice(DeviceSearchParameters deviceSearchParameters) throws Exception {
		List<Object[]> data = inventoryService.searchDevice(deviceSearchParameters);
		List<DeviceSearchResult> results = new ArrayList<DeviceSearchResult>();
		String searchType = deviceSearchParameters.getSearchType().getValue();
		if (DeviceSearchParameters.DEVICE_SEARCH_TYPE_PATIENT.equals(searchType)) {
			for (Object[] objs : data) {
				DeviceSearchResult record = new DeviceSearchResult();
				record.vendorName = (String) objs[0];
				record.deviceName = (String) objs[1];
				record.deviceSerialNumber = (String) objs[2];
				record.activationDate = (Date) objs[3];
				record.lastName = (String) objs[4];
				record.firstName = (String) objs[5];
				record.middleName = (String) objs[6];
				record.ssn = (String) objs[7];
				record.icn = (String) objs[8];
				results.add(record);
			}
		} else if (DeviceSearchParameters.DEVICE_SEARCH_TYPE_ACTIVATION_DATE.equals(searchType)) {
			for (Object[] objs : data) {
				DeviceSearchResult record = new DeviceSearchResult();
				record.vendorName = (String) objs[0];
				record.deviceName = (String) objs[1];
				record.deviceSerialNumber = (String) objs[2];
				record.activationDate = (Date) objs[3];
				results.add(record);
			}
		} else if (DeviceSearchParameters.DEVICE_SEARCH_TYPE_STATUS.equals(searchType)) {
			for (Object[] objs : data) {
				DeviceSearchResult record = new DeviceSearchResult();
				record.vendorName = (String) objs[0];
				record.deviceName = (String) objs[1];
				record.deviceSerialNumber = (String) objs[2];
				record.deviceStatus = (String) objs[3];
				results.add(record);
			}
		} else if (DeviceSearchParameters.DEVICE_SEARCH_TYPE_GENERIC.equals(searchType)) {
			for (Object[] objs : data) {
				DeviceSearchResult record = new DeviceSearchResult();
				record.id = String.valueOf(objs[0]);
				record.deviceName = (String) objs[1];
				record.deviceSerialNumber = (String) objs[2];
				record.multiPatientUsage = (Boolean) objs[3];
				record.statusChangeDate = (Date) objs[4];
				record.deviceStatus = (String) objs[5];
				record.lastName = (String) objs[6];
				record.careCoordinator = (String) objs[7];
				results.add(record);
			}
		} else {
			throw new Exception("Invalid Device search type" + searchType);
		}
		return results;
	}

	public DeviceDetailForm getDeviceDetail(String id) throws Exception {
		EntityKey<DeviceDetail> entityKey = EntityKeyFactory.createEntityKey(new Long(id), DeviceDetail.class);
		DeviceDetail dd = inventoryService.getDeviceDetail(entityKey);
		DeviceDetailForm ddForm = new DeviceDetailForm();
		getInventoryConversionService().convert(dd, ddForm);
		return ddForm;
	}

	public ControllerResult addDeviceDetail(DeviceDetailForm deviceDetailForm) throws Exception {
		// Verify whether the device with same model and serial number
		// exists
		DeviceSearchParameters deviceSearchParameters = new DeviceSearchParameters();
		deviceSearchParameters.setDeviceType(deviceDetailForm.getDeviceType());
		deviceSearchParameters.setSerialNumber(deviceDetailForm.getSerialNumber());
		DeviceDetail dd = inventoryService.getDeviceDetail(deviceSearchParameters);
		if (dd == null) {
			dd = new DeviceDetail();
			getInventoryConversionService().convert(deviceDetailForm, dd);
			inventoryService.updateDeviceDetail(dd);
			return new ControllerResult(ControllerResult.SUCCESS);
		} else {
			return new ControllerResult(ControllerResult.ALREADY_EXISTS);
		}
	}

	public DeviceDetailForm updateDeviceDetail(DeviceDetailForm deviceDetailForm) throws Exception {
		try {
			// update an existing device
			EntityKey<DeviceDetail> entityKey = EntityKeyFactory.createEntityKey(new Long(deviceDetailForm.getId()),
					new Integer(deviceDetailForm.getVersionNumber()), DeviceDetail.class);
			DeviceDetail dd = inventoryService.getDeviceDetail(entityKey);
			getInventoryConversionService().convert(deviceDetailForm, dd);
			inventoryService.updateDeviceDetail(dd);
			dd = inventoryService.getDeviceDetail(entityKey);
			DeviceDetailForm ddForm = new DeviceDetailForm();
			getInventoryConversionService().convert(dd, ddForm);
			return ddForm;
		} catch (ServiceOptimisticLockException ople) {
			throw new ControllerException(
					"Database operation failed because data was changed by another session. You will have to re-load it and re-apply your changes.");
		}
	}

	public List<DIReportData> generateVCCurrentReport(DeviceSearchParameters deviceSearchParameters) throws Exception {
		List<Object[]> data = inventoryService.generateVCCurrentReport(deviceSearchParameters);
		List<DIReportData> results = new ArrayList<DIReportData>();
		for (Object[] objs : data) {
			DIReportData record = new DIReportData();
			record.vendorName = (String) objs[0];
			record.deviceName = (String) objs[1];
			record.deviceCount = (Integer) objs[2];
			record.deviceCountNoSerialNumber = (Integer) objs[3];
			results.add(record);
		}
		return results;
	}

	public List<VCReportData> generateVCHistoryReport(DeviceSearchParameters deviceSearchParameters) throws Exception {
		List<Object[]> data = inventoryService.generateVCHistoryReport(deviceSearchParameters);
		List<VCReportData> results = new ArrayList<VCReportData>();
		for (Object[] objs : data) {
			VCReportData record = new VCReportData();
			record.reportEndDate = (Date) objs[0];
			record.vendorName = (String) objs[1];
			record.deviceCount = (Integer) objs[2];
			record.modelNameCount = (Integer) objs[3];
			record.serialNumberCount = (Integer) objs[4];
			results.add(record);
		}
		return results;
	}

	public List<VCReportData> generateVCReportCharts(DeviceSearchParameters deviceSearchParameters) throws Exception {
		return generateVCHistoryReport(deviceSearchParameters);
	}

	public void saveInventoryReport(DIReportParameterForm dIReportParameterForm) throws Exception {
		DIReportParameters repParameters = new DIReportParameters();
		repParameters.setReportSummaryType(dIReportParameterForm.getReportSummaryType().getValue());
		repParameters.setSubTotalType(dIReportParameterForm.getSubTotalType().getValue());
		repParameters.setReportWeek(dIReportParameterForm.getReportWeek());
		if (dIReportParameterForm.getVisn() != null) {
			repParameters.setVisn(inventoryService.getSimpleVisn(dIReportParameterForm.getVisn().getValue()));
		}
		if (dIReportParameterForm.getFacility() != null) {
			repParameters.setFacility(inventoryService.getFacility(dIReportParameterForm.getFacility().getValue()));
		}
		// save
	}

	public DeviceRequirementForm getDeviceRequirement(String id) throws Exception {
		EntityKey<DeviceRequirement> entityKey = EntityKeyFactory.createEntityKey(new Long(id),
				DeviceRequirement.class);
		DeviceRequirement dr = inventoryService.getDeviceRequirement(entityKey);
		DeviceRequirementForm drform = new DeviceRequirementForm();
		getInventoryConversionService().convert(dr, drform);
		return drform;
	}

	public List<DeviceRequirementForm> getDeviceRequirements(TermType facility) throws Exception {
		SimpleFacility simpleFacility = inventoryService.getSimpleFacility(facility.getValue());
		List<DeviceRequirementForm> list = new ArrayList<DeviceRequirementForm>();
		List<DeviceRequirement> drs = inventoryService.getDeviceRequirements(simpleFacility);

		for (DeviceRequirement dr : drs) {
			DeviceRequirementForm drform = new DeviceRequirementForm();
			getInventoryConversionService().convert(dr, drform);
			list.add(drform);
		}
		return list;
	}

	public List<DeviceRequirementForm> updateDeviceRequirements(List<DeviceRequirementForm> deviceRequirementForms)
			throws Exception {
		TermType facility = null;
		List<DeviceRequirement> deviceRequirements = new ArrayList<DeviceRequirement>();
		try {
			for (DeviceRequirementForm df : deviceRequirementForms) {
				DeviceRequirement dr = null;
				facility = df.getFacility();

				if (StringUtils.isNotEmpty(df.getId())) {
					EntityKey<DeviceRequirement> entityKey = EntityKeyFactory.createEntityKey(new Long(df.getId()),
							new Integer(df.getVersionNumber()), DeviceRequirement.class);
					// retrieve
					dr = inventoryService.getDeviceRequirement(entityKey);
				} else {
					dr = new DeviceRequirement();
				}
				// conversion and merge update only if the value has been
				// changed
				if (!df.getDevicesNeeded().equals(dr.getDevicesNeeded())) {
					getInventoryConversionService().convert(df, dr);
					deviceRequirements.add(dr);
				}
			}
			// save
			if (deviceRequirements.size() > 0) {
				inventoryService.updateDeviceRequirements(deviceRequirements);
			}
			if (facility != null) {
				return getDeviceRequirements(facility);
			} else {
				return new ArrayList<DeviceRequirementForm>();
			}
		} catch (ServiceOptimisticLockException ople) {
			throw new ControllerException(
					"Database operation failed because data was changed by another session. You will have to re-load it and re-apply your changes.");
		}
	}

	public List<Device> getDevices() throws Exception {
		return inventoryService.getDevices();
	}

	public Map<Long, DeviceType> getDevicesLookUpMap(List<DeviceType> devices) throws Exception {

		// List<Device> devices = getDevices();
		Map<Long, DeviceType> devicesMap = new LinkedHashMap<Long, DeviceType>();

		for (DeviceType device : devices) {
			devicesMap.put(device.getIdentifier(), device);
		}

		return devicesMap;
	}

	public Map<Long, SimpleFacility> getFacilitiesLookUpMap() throws Exception {
		SortedSet<SimpleFacility> facilitiesList = getSortedSimpleFacilities();
		Map<Long, SimpleFacility> facilitiesMap = new LinkedHashMap<Long, SimpleFacility>();

		for (SimpleFacility facility : facilitiesList) {
			facilitiesMap.put(facility.getId(), facility);
		}

		return facilitiesMap;
	}

	public Map<String, SimpleFacility> getFacilitiesLookUp() throws Exception {
		SortedSet<SimpleFacility> facilitiesList = getSortedSimpleFacilities();
		Map<String, SimpleFacility> facilitiesMap = new LinkedHashMap<String, SimpleFacility>();

		for (SimpleFacility facility : facilitiesList) {
			facilitiesMap.put(facility.getStationNumber(), facility);
		}

		return facilitiesMap;
	}

	public Map<Long, String> getFacilitiesMap(Map<Long, SimpleFacility> facilityMap) throws Exception {
		Map<Long, String> facilitiesMap = new LinkedHashMap<Long, String>();
		facilitiesMap.put(new Long(-1), "Select Facility");
		// for(Facility facility : facilitiesList)
		for (Map.Entry<Long, SimpleFacility> entry : facilityMap.entrySet()) {
			SimpleFacility facility = entry.getValue();

			facilitiesMap.put(facility.getId(),
					facility.getVisn().getName() + " " + facility.getStationNumber() + " " + facility.getName());
		}

		return facilitiesMap;
	}

	public SortedSet<SimpleFacility> getSortedSimpleFacilities() throws Exception {
		return inventoryService.getSortedSimpleFacilities();
	}

	public SortedSet<SimpleFacility> getSimpleFacilities() throws Exception {
		return inventoryService.getSimpleFacilities();
	}

	public Map<Long, Vendor> getVendorsLookUpMap() throws Exception {
		List<Vendor> vendorsList = getVendors();
		Map<Long, Vendor> vendorsMap = new HashMap<Long, Vendor>();

		for (Vendor vendor : vendorsList) {
			vendorsMap.put(vendor.getId(), vendor);
		}

		return vendorsMap;
	}

	public Map<Long, String> getVendorsMap(Map<Long, Vendor> vendorLookUp) throws Exception {
		Map<Long, String> vendorsMap = new HashMap<Long, String>();

		vendorsMap.put(new Long(-1), "Select Vendor");
		for (Map.Entry<Long, Vendor> entry : vendorLookUp.entrySet()) {
			Vendor vendor = entry.getValue();
			vendorsMap.put(vendor.getId(), vendor.getName());
		}

		return vendorsMap;
	}

	public List<Vendor> getVendors() throws Exception {
		return inventoryService.getVendors();
	}

	public Map<Long, SimpleVisn> getVisnLookUpMap() throws Exception {
		List<SimpleVisn> visnList = getSimpleVisns();
		Map<Long, SimpleVisn> visnMap = new HashMap<Long, SimpleVisn>();

		for (SimpleVisn visn : visnList) {
			visnMap.put(visn.getId(), visn);
		}

		return visnMap;
	}

	public Map<Long, String> getVisnsMap(Map<Long, SimpleVisn> visnMap) throws Exception {
		Map<Long, String> visnLookUpMap = new HashMap<Long, String>();
		visnLookUpMap.put(new Long(-1), "Select VISN");
		for (Map.Entry<Long, SimpleVisn> entry : visnMap.entrySet()) {
			SimpleVisn visn = entry.getValue();
			visnLookUpMap.put(visn.getId(), visn.getName());
		}

		return visnLookUpMap;
	}

	public List<Visn> getVisns() throws Exception {
		return inventoryService.getVisns();
	}

	public List<SimpleVisn> getSimpleVisns() throws Exception {
		return inventoryService.getSimpleVisns();
	}

	public List<ReportWeek> getReportWeeks() throws Exception {
		return inventoryService.getReportWeeks();
	}

	private InventoryConversionService inventoryConversionService = null;

	public InventoryConversionService getInventoryConversionService() {
		return inventoryConversionService;
	}

	public void setInventoryConversionService(InventoryConversionService inventoryConversionService) {
		this.inventoryConversionService = inventoryConversionService;
	}

}
