package gov.va.med.ccht.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;

import edu.emory.mathcs.backport.java.util.Arrays;
import gov.va.med.ccht.ui.model.RoleForm;
import gov.va.med.fw.model.EntityKeyFactory;
import gov.va.med.fw.security.Permission;
import gov.va.med.fw.security.Role;
import gov.va.med.fw.security.SimpleRole;
import gov.va.med.fw.service.ServiceException;

@Controller
@SessionAttributes(value = { "command" })
public class ManageRolesController extends CommonCchtController {

	@RequestMapping("/roles")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody Map<String, Object> roleList() throws Exception {
		Map<String, Object> results = new HashMap<>();
		List<SimpleRole> simpleRoles = securityService.getAllSimpleRoles();
		results.put("roles", simpleRoles);
		return results;
	}

	@RequestMapping(value = "/roleSelect.html")
	public String roleSelect() throws Exception {
		return "roleSelect";
	}

	@RequestMapping(value = "/roleEdit.html", method = RequestMethod.GET)
	public String roleNewOrEdit(ModelMap model, @RequestParam(required = false) Long roleId) throws Exception {
		Role role = new Role();
		if (roleId != null) {
			role = securityService.getRole(EntityKeyFactory.createEntityKey(roleId, Role.class));
		}

		RoleForm form = new RoleForm(role);
		model.put("command", form);

		//populateReferenceData(model);
		populatePermissionsCheckboxGroup(model); 

		return "roleEdit";
	}

	private void populateReferenceData(ModelMap model) throws ServiceException {
		Map<Long, String> allPermissions = new LinkedHashMap<>();
		for (Permission p : securityService.getAllPermissions()) {
			allPermissions.put(p.getId(), p.getName());
		}
		model.put("allRolesMap", allPermissions);
	}
	
	private void populatePermissionsCheckboxGroup(ModelMap model) throws ServiceException {
		
		/**
		 * Just a test for building checkbox group
		 */
		Map<String, String> availRoles = null;
		List<Permission> allPermissions = securityService.getAllPermissions();
		Permission[] permissions = new Permission[allPermissions.size()];
		int index = 0;
		for(Permission p : allPermissions) {
			String name = p.getName();
			
			permissions[index] = p;
			index++;
		}
		
		model.addAttribute("permissionCheckBoxes", permissions);
			
	}

	@RequestMapping(value = "/roleEdit.html", method = RequestMethod.POST)
	public String submitRoleNewOrEdit(@ModelAttribute("command") RoleForm command, BindingResult result, ModelMap model,
			HttpServletRequest request) throws Exception {
		
		// Create Role that will be edited and then saved
		Role role = new Role();
		if (command.getId() != null) {
			role = securityService
					.getRole(EntityKeyFactory.createEntityKey(Long.parseLong(command.getId()), Role.class));
		}

		role.setName(command.getName());
		role.setDescription(command.getDescription());

		Set<Permission> permissions = new HashSet<>();
		List<Permission> allPermissions = securityService.getAllPermissions();
		List<String> selectedPermissions = new ArrayList<>( Arrays.asList(command.getSelectedPermissions()));
		
		for (Permission p : allPermissions) {			
			if (selectedPermissions.contains(p.getName()))
				permissions.add(p);
		}
		role.setPermissions(permissions);

		List<SimpleRole> allRoles = securityDAO.findAllSimpleRoles();
		for (SimpleRole r : allRoles) {
			if (r.getName().trim().equalsIgnoreCase(role.getName())
					&& (role.getId() == null || !role.getId().equals(r.getId())))
				result.rejectValue("name", "role.error.duplicateRoleFound", new Object[] { role.getName() }, null);
		}

		if (result.hasErrors()) {
			populateReferenceData(model);
			return "roleEdit";
		}

		securityService.updateRole(role);
		userNotifier.notifyUserOnceWithMessage(request, "The role was updated successfully.");
		return "redirect:" + basePath + "roleSelect.html";
	}

}
