package gov.va.med.ccht.controller;


import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.util.UriUtils;

import gov.va.med.ccht.controller.validators.ManageUserEditFormValidator;
import gov.va.med.ccht.model.CCHTPermissions;
import gov.va.med.ccht.model.User;
import gov.va.med.ccht.model.inventory.SimpleFacility;
import gov.va.med.ccht.model.inventory.SimpleVisn;
import gov.va.med.ccht.model.terminology.RegistrationStatus;
import gov.va.med.ccht.service.common.RegistrationService;
import gov.va.med.ccht.ui.model.UserForm;
import gov.va.med.fw.ui.model.TermType;
import gov.va.med.fw.util.DateUtils;

@Controller
@SessionAttributes(value = { "command" })
public class ManageUsersEditController extends CommonCchtController {

	@Autowired
	RegistrationService registrationService;
	@Autowired
	ManageUserEditFormValidator editFormValidator;
	
	
	
	@RequestMapping(value = "/manageUsersEdit.html", method = { RequestMethod.GET })
	public String buildForm(ModelMap model, @RequestParam String userName, @RequestParam String fromPage) throws Exception {

		User currentUser = getCurrentUser();
		User user = securityService.getUser(userName);
		
		// Build userForm based on acquired user to pass
		// to form to edit
		UserForm userForm = getUser(userName);
		securityConversionService.convert(user, userForm);
		completeUserForm(userForm);
		
		userForm.setFromPage(fromPage);
		// Get available options to assign to user.
		model.addAttribute("userGroups", getUserGroups());
		model.addAttribute("registrationStatuses", getRegistrationStatuses());
		model.addAttribute("registrationDeniedReasons", getRegistrationDeniedReasons(true));
		model.addAttribute("jobDescriptions", getJobDescriptions());
		model.addAttribute("facilities", getFacilities(true));
		model.addAttribute("vendors", getVendors(true));
		model.addAttribute("visns", getVisns(true));
		model.addAttribute("inactiveReasons", getInactiveReasons(true));
		//model.addAttribute("rolesMap", getAvailableRoles());
		
		
		
		// Will have to be reworked to more elegant solution when time allows
		setRoles(model, userForm);
		
		model.addAttribute("canEdit", canEdit(currentUser, user));
		model.addAttribute("isCurrentUserAppAdmin", isApplicationAdmin(currentUser));	
		
		
		if(fromPage.equalsIgnoreCase("userSearch"))
		{
			userForm.setReturnPage("manageUsersSearch.html");			
		}
		else {
			
			TermType registrationStatus = new TermType();
	
			if(fromPage.equalsIgnoreCase("approveReg")) {
					model.addAttribute("pageTitle", "Approve Registration");
					userForm.setAssignedRegistrationStatusId("2");
					registrationStatus.setValue("2");
					registrationStatus.setLabel("Approved");
					userForm.setRegistrationStatus(registrationStatus);
					userForm.setRegistrationDeniedReason(null);
					model.addAttribute("showRegistrationDenied", false);
					model.addAttribute("isDeny", false);				
			}
			else {
				model.addAttribute("showRegistrationDenied", true);
				model.addAttribute("pageTitle", "Deny Registration");
				model.addAttribute("isDeny", true);
				model.addAttribute("disableRegDeniedDropdown", false);
				userForm.setAssignedRegistrationStatusId("3");
				registrationStatus.setValue("3");
				registrationStatus.setLabel("Denied");
				userForm.setRegistrationStatus(registrationStatus);				
			}
			userForm.setReturnPage("manageRegistrations.html");			
		}
		model.addAttribute("returnPage", userForm.getReturnPage());
		
		TermType regDeniedReason = userForm.getRegistrationDeniedReason();
		
		if(regDeniedReason != null) {
			model.addAttribute("showRegistrationDenied", true);
			model.addAttribute("disableRegDeniedDropdown", true);
			userForm.setRegistrationDeniedReasonId(regDeniedReason.getValue());
		}
		
		
		/**
		 * Just a test for building checkbox group
		 */
		Map<String, String> availRoles = getAvailableRoles();
		if(availRoles != null){
			
			// Get logged in users available roles
			String[] temp = new String[availRoles.size()];
			int i = 0;
			Iterator iter = availRoles.keySet().iterator();
			while(iter.hasNext())
			{
				temp[i] = (String)iter.next();
				i++;
			}
			model.addAttribute("rolesMap", temp);
			
			
			// Get roles of user being edited
			List<String> assignedRoles = userForm.getAssignedRoles();
			String[] selectedRoles = new String[assignedRoles.size()];
			i = 0;
			for(String role : assignedRoles) {
				selectedRoles[i] = role;
				i++;
			}
			userForm.setSelectedRoles(selectedRoles);
			
			
		}
		
		model.addAttribute("command", userForm);
		
		return "manageUsersEdit";
		
	}

	@RequestMapping(value = "/manageUsersEdit.html", method = { RequestMethod.POST, RequestMethod.GET })
	public String submit(@ModelAttribute("command") @Valid UserForm command, BindingResult result, Model model)
			throws Exception {


		// Step one, ensure all user entries are
		// represented in a UserForm object.
		buildValidatedUserForm(command);

		User user = securityService.getUser(command.getUserName());
		String oldRegStatusCode = user.getRegistrationStatus() == null ? "" : user.getRegistrationStatus().getCode();

		// Step two, convert UserForm to User by merging
		// new info into User object
		securityConversionService.convert(command, user);

		// Step three, save.
		securityService.updateUser(user);

		String newRegStatusCode = user.getRegistrationStatus() == null ? "" : user.getRegistrationStatus().getCode();
		if (!oldRegStatusCode.equals(newRegStatusCode)) {
			if (RegistrationStatus.APPROVED.equals(newRegStatusCode)) {
				registrationService.sendWelcomeMail(user);
			} else if (RegistrationStatus.REJECTED.equals(newRegStatusCode)) {
				registrationService.sendDeniedMail(user);
			} else {
				// submitted nothing to do moved to pending status
			}
		}

		
		return "redirect:" + basePath + command.getReturnPage();
		
	}

	@RequestMapping(value = "/manageUsersEdit.html", method = { RequestMethod.GET, RequestMethod.POST }, params = {
			"Reset" })
	public String refresh(@ModelAttribute("command") @Valid UserForm command, BindingResult result, Model model)
			throws Exception {
		
		

		User currentUser = getCurrentUser();
		String userName = command.getUserName();
		String fromPage = command.getFromPage();
		String returnPage = command.getReturnPage();
		User user = securityService.getUser(userName);
		
		// Build userForm based on aquired user to pass
		// to form to edit
		UserForm userForm = getUser(userName);
		securityConversionService.convert(user, userForm);		
		completeUserForm(userForm);		
		userForm.setReturnPage(returnPage);	
		userForm.setFromPage(fromPage);
		
		
		model.addAttribute("userGroups", getUserGroups());
		model.addAttribute("registrationStatuses", getRegistrationStatuses());
		model.addAttribute("registrationDeniedReasons", getRegistrationDeniedReasons(true));
		model.addAttribute("jobDescriptions", getJobDescriptions());
		model.addAttribute("facilities", getFacilities(true));
		model.addAttribute("vendors", getVendors(true));
		model.addAttribute("visns", getVisns(true));
		model.addAttribute("inactiveReasons", getInactiveReasons(true));
		
		//model.addAttribute("rolesMap", getAvailableRoles());

		
		setRoles(model, userForm);
		
		model.addAttribute("canEdit", canEdit(currentUser, user));
		model.addAttribute("isCurrentUserAppAdmin", isApplicationAdmin(currentUser));
		
		// Set up values for checkbox
		Map<String, String> availRoles = getAvailableRoles();
		if(availRoles != null){

			setModelsRolesMap(model, availRoles);

			setUserFormsSelectedRoles(userForm);
		}
		
		if(fromPage.equalsIgnoreCase("userSearch"))
		{
			userForm.setReturnPage("manageUsersSearch.html");			
		}
		else {
			
			TermType registrationStatus = new TermType();
	
			if(fromPage.equalsIgnoreCase("approveReg")) {
					model.addAttribute("pageTitle", "Approve Registration");
					userForm.setAssignedRegistrationStatusId("2");
					registrationStatus.setValue("2");
					registrationStatus.setLabel("Approved");
					userForm.setRegistrationStatus(registrationStatus);
					userForm.setRegistrationDeniedReason(null);
					model.addAttribute("showRegistrationDenied", false);
					model.addAttribute("isDeny", false);				
			}
			else {
				model.addAttribute("showRegistrationDenied", true);
				model.addAttribute("pageTitle", "Deny Registration");
				model.addAttribute("isDeny", true);
				model.addAttribute("disableRegDeniedDropdown", false);
				userForm.setAssignedRegistrationStatusId("3");
				registrationStatus.setValue("3");
				registrationStatus.setLabel("Denied");
				userForm.setRegistrationStatus(registrationStatus);				
			}
			userForm.setReturnPage("manageRegistrations.html");			
		}
		
		TermType regDeniedReason = userForm.getRegistrationDeniedReason();
		
		if(regDeniedReason != null) {
			model.addAttribute("showRegistrationDenied", true);
			model.addAttribute("disableRegDeniedDropdown", true);
			userForm.setRegistrationDeniedReasonId(regDeniedReason.getValue());
		}	
		
		model.addAttribute("command", userForm);			
		
		return "manageUsersEdit";
	}
	
	private void setModelsRolesMap(Model model, Map<String, String> availRoles) {
		
		String[] temp = new String[availRoles.size()];
		int i = 0;
		Iterator iter = availRoles.keySet().iterator();
		while(iter.hasNext())
		{
			temp[i] = (String)iter.next();
			i++;
		}
		model.addAttribute("rolesMap", temp);
		
	}

	private void setUserFormsSelectedRoles(UserForm userForm) {
		
		List<String> assignedRoles = userForm.getAssignedRoles();
		String[] selectedRoles = new String[assignedRoles.size()];
		int i = 0;
		for(String role : assignedRoles) {
			selectedRoles[i] = role;
			i++;
		}
		userForm.setSelectedRoles(selectedRoles);
		
	}
	/**
	 * Builds a validated user form. This form has had edits in it to update a
	 * user.
	 * 
	 * @param userForm
	 * @return
	 * @throws Exception
	 */
	private void buildValidatedUserForm(UserForm userForm) throws Exception {

		// If the user selected a VISN, Create a TermType
		// and set it on the form.
		String assignedVisnId = userForm.getAssignedVisnId();
		if (assignedVisnId != null) {

			if (assignedVisnId.equalsIgnoreCase(NEGATIVE_ONE)) {
				userForm.setVisn(null);
			} else {
				TermType newVisn = new TermType();
				newVisn.setValue(assignedVisnId);
				newVisn.setLabel(assignedVisnId);
				userForm.setVisn(newVisn);
			}
		}

		// If the user selected a Facility, Create a TermType
		// and set it on the form.
		String assignedFacilityId = userForm.getAssignedFacilityId();
		if (assignedFacilityId != null) {

			if (assignedFacilityId.equalsIgnoreCase(NEGATIVE_ONE)) {
				userForm.setFacility(null);
			} else {
				TermType newFacility = new TermType();
				newFacility.setValue(assignedFacilityId);
				userForm.setFacility(newFacility);
			}
		}

		// If the user selected a VENDOR, Create a TermType
		// and set it on the form.
		String assignedVendorId = userForm.getAssignedVendorId();
		if (assignedVendorId != null) {

			if (assignedVendorId.equalsIgnoreCase(NEGATIVE_ONE)) {
				userForm.setVendor(null);
			} else {
				TermType newVendor = new TermType();
				newVendor.setLabel(assignedVendorId);
				userForm.setVendor(newVendor);
			}
		}
		
		String[] assignedRoles = userForm.getSelectedRoles();
		if(assignedRoles != null) {
			List<TermType> updatedRoleAssignments = new ArrayList<TermType>();
			for(String name : assignedRoles) {
				TermType role = new TermType(name, name);
				updatedRoleAssignments.add(role);
			}
			
			userForm.setRoles(updatedRoleAssignments);
		}
		else {
			userForm.setRoles(new ArrayList<TermType>());			
		}
		
		// Set locked status.
		Boolean isLocked = userForm.getLocked();
		if (isLocked != null && isLocked) {
			userForm.setAccountLockDate(new Date());
		}
		
		// Set inactive status
		Boolean isInactive = userForm.getInactive();
		if(isInactive != null && isInactive) {
			
			if(userForm.getInactivationDateAsString() != null)
			{
				Date inactiveDate = DateUtils.parseDate(userForm.getInactivationDateAsString(), DateUtils.MMDDYYYY);
				userForm.setInactiveDate(inactiveDate);
			}
			else {
				
			}
			
			String inactiveReasonId = userForm.getInactiveReasonId();
			if(inactiveReasonId != null) {
				TermType inactiveReason = new TermType();
				inactiveReason.setValue(inactiveReasonId);
				userForm.setInactiveReason(inactiveReason);
			}
		}
		
		String registrationDeniedReasonid = userForm.getRegistrationDeniedReasonId();
		if(registrationDeniedReasonid != null) {
			TermType registrationDeniedReason = new TermType();
			registrationDeniedReason.setValue(registrationDeniedReasonid);
			userForm.setRegistrationDeniedReason(registrationDeniedReason);
		}
	}

	/**
	 * Build User Form when the edit screen initially loads.
	 * 
	 * @param userForm
	 * @return
	 * @throws Exception
	 */
	private void completeUserForm(UserForm userForm) throws Exception {

		// Set userGroup id.
		TermType usersGroup = userForm.getUserGroup();
		if (usersGroup != null) {
			userForm.setAssignedUserGroup(usersGroup.getValue());
		}

		// Set registration status id.
		TermType regStatus = userForm.getRegistrationStatus();
		if (regStatus != null) {
			userForm.setAssignedRegistrationStatusId(regStatus.getValue());
		}

		// Set Job Description Id
		TermType regReason = userForm.getRegistrationReason();
		if (regReason != null) {
			userForm.setJobDescriptionId(regReason.getValue());
		}

		TermType visn = userForm.getVisn();
		if (visn != null) {
			userForm.setAssignedVisnId(visn.getValue().trim());
		}

		TermType facility = userForm.getFacility();
		if (facility != null) {
			userForm.setAssignedFacilityId(facility.getValue().trim());
		}

		TermType vendor = userForm.getVendor();
		if (vendor != null) {
			userForm.setAssignedVendorId(vendor.getLabel());
		}
		
		
		List<TermType> userRoles = userForm.getRoles();
		
		if(userRoles != null) {
			
			if(userForm.getAssignedRoles() == null) {
				userForm.setAssignedRoles(new ArrayList<String>());
			}
			for(TermType role : userRoles) {
				String name = role.getValue();
				userForm.getAssignedRoles().add(name);
			}
		}
		
		
		Date accountLockedDate = userForm.getAccountLockDate();
		if(accountLockedDate != null) {
			userForm.setLocked(true);
		}
		else {
			userForm.setLocked(false);
		}
			
		Date accountInactiveDate = userForm.getInactiveDate();
		if(accountInactiveDate != null) {
			userForm.setInactive(true);
			SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");
			userForm.setInactivationDateAsString( dateFormat.format(accountInactiveDate)  );
			
			TermType inactiveReason = userForm.getInactiveReason();
			if(inactiveReason != null) {			
				userForm.setInactiveReasonId(inactiveReason.getValue());
			}
		}
		else {
			userForm.setInactive(false);
		}
	}
	
	
	/**
	 * 
	 * Set up the rules that are needed to determine which page to return
	 * to when the cancel button is pressed on the Manage User / Approve-Deny registration 
	 * page.
	 * 
	 * @param userForm
	 * @param model
	 */
	private void setFromPageRules(UserForm userForm, Model model) {
		
		String fromPage = userForm.getReturnPage();
		
		if(fromPage.equalsIgnoreCase("userSearch"))
		{
			userForm.setReturnPage("manageUsersSearch.html");			
		}
		else {
			
			TermType registrationStatus = new TermType();
	
			if(fromPage.equalsIgnoreCase("approveReg")) {
				model.addAttribute("pageTitle", "Approve Registration");
				userForm.setAssignedRegistrationStatusId("2");
				registrationStatus.setValue("2");
				registrationStatus.setLabel("Approved");
				userForm.setRegistrationStatus(registrationStatus);
			}
			else {
				model.addAttribute("pageTitle", "Deny Registration");
				userForm.setAssignedRegistrationStatusId("3");
				registrationStatus.setValue("3");
				registrationStatus.setLabel("Denied");
				userForm.setRegistrationStatus(registrationStatus);
			}
			userForm.setReturnPage("manageRegistrations.html");			
		}
	}
	
	/**
	 * Brute force method to enforce roles based rules of view widgets
	 * this needs to be changed when time allows.
	 * @param model
	 * @param userForm
	 */
	private void setRoles(ModelMap model, UserForm userForm) {
		
		model.addAttribute("isNationalAdmin", userForm.getAssignedRoles().contains(CCHTPermissions.NATIONAL_ADMIN));	
		model.addAttribute("isApplicationAdmin",  userForm.getAssignedRoles().contains(CCHTPermissions.APPLICATION_ADMIN));
		model.addAttribute("isCareCoordinator",  userForm.getAssignedRoles().contains(CCHTPermissions.CARE_COORDINATOR));
		model.addAttribute("isFacilityAdmin",  userForm.getAssignedRoles().contains(CCHTPermissions.FACILITY_ADMIN));
		model.addAttribute("isOTS",  userForm.getAssignedRoles().contains(CCHTPermissions.OTS_CONTRACT_MANAGER));
		model.addAttribute("isPSA",  userForm.getAssignedRoles().contains("Program Support Assistant"));
		model.addAttribute("isQIR",  userForm.getAssignedRoles().contains("QIR Originator"));
		model.addAttribute("isReportsOnly", userForm.getAssignedRoles().contains("Reports Only"));	
		model.addAttribute("management",  userForm.getAssignedRoles().contains(CCHTPermissions.MANAGEMENT));
		model.addAttribute("isVendor", userForm.getAssignedRoles().contains(CCHTPermissions.VENDOR));
		model.addAttribute("isVisnAdmin", userForm.getAssignedRoles().contains(CCHTPermissions.VISN_ADMIN));
		
	}
	
	/**
	 * Brute force method to enforce roles based rules of view widgets
	 * this needs to be changed when time allows.
	 * @param model
	 * @param userForm
	 */
	private void setRoles(Model model, UserForm userForm) {
		
		model.addAttribute("isNationalAdmin", userForm.getAssignedRoles().contains(CCHTPermissions.NATIONAL_ADMIN));	
		model.addAttribute("isApplicationAdmin",  userForm.getAssignedRoles().contains(CCHTPermissions.APPLICATION_ADMIN));
		model.addAttribute("isCareCoordinator",  userForm.getAssignedRoles().contains(CCHTPermissions.CARE_COORDINATOR));
		model.addAttribute("isFacilityAdmin",  userForm.getAssignedRoles().contains(CCHTPermissions.FACILITY_ADMIN));
		model.addAttribute("isOTS",  userForm.getAssignedRoles().contains(CCHTPermissions.OTS_CONTRACT_MANAGER));
		model.addAttribute("isPSA",  userForm.getAssignedRoles().contains("Program Support Assistant"));
		model.addAttribute("isQIR",  userForm.getAssignedRoles().contains("QIR Originator"));
		model.addAttribute("isReportsOnly", userForm.getAssignedRoles().contains("Reports Only"));	
		model.addAttribute("management",  userForm.getAssignedRoles().contains(CCHTPermissions.MANAGEMENT));
		model.addAttribute("isVendor", userForm.getAssignedRoles().contains(CCHTPermissions.VENDOR));
		model.addAttribute("isVisnAdmin", userForm.getAssignedRoles().contains(CCHTPermissions.VISN_ADMIN));
	}
	
	private boolean canEdit(User currentUser, User editUser) {
		
			
		boolean isNationalAdmin = currentUser.isPermissionGranted(CCHTPermissions.NATIONAL_ADMIN);		
		if(isNationalAdmin) {
			return true;
		}
		
		boolean isVisnAdmin = currentUser.isPermissionGranted(CCHTPermissions.VISN_ADMIN);
		boolean isFacilityAdmin = currentUser.isPermissionGranted(CCHTPermissions.FACILITY_ADMIN);
		
		
		// Get current logged in users information
		SimpleVisn currentUserVisn = currentUser.getVisn();
		SimpleFacility currentUserFacility = currentUser.getFacility();		
		Long currentUserVisnId = new Long(-1);
		Long currentUserFacilityId = new Long(-1);
		
		if(currentUserVisn != null) {
			currentUserVisnId = currentUserVisn.getId();
		}
		
		if(currentUserFacility!= null) {
			currentUserFacilityId = currentUserFacility.getId();
		}
		
		// Get the user to be edited information
		SimpleVisn editUserVisn = editUser.getVisn();
		SimpleFacility editUserFacility = editUser.getFacility();
		Long editUserVisnId = new Long(-1);
		Long editUserFacilityId = new Long(-1);
		
		if(editUserVisn != null) {
			editUserVisnId = editUserVisn.getId();
		}
		
		if(editUserFacility!= null) {
			editUserFacilityId = editUserFacility.getId();
		}
		
		
		
		// Determine if the logged in user can edit the desired user.
		if(isVisnAdmin) {
			return currentUserVisnId.equals(editUserVisnId);
		}
		
		if(isFacilityAdmin && !isVisnAdmin) {
			return currentUserFacilityId.equals(editUserFacilityId);
		}
		

		return true;
		
	}
}
