/**
 * 
 */
package gov.va.med.ccht.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import gov.va.med.ccht.model.qir.QIR;
import gov.va.med.ccht.model.qir.QIRAttachment;
import gov.va.med.ccht.model.qir.QIRSearchParameters;
import gov.va.med.ccht.model.qir.QIRSearchResult;
import gov.va.med.ccht.model.qir.QIRSimpleAttachment;
import gov.va.med.ccht.model.terminology.DocumentType;
import gov.va.med.ccht.ui.common.AbstractController;
import gov.va.med.ccht.ui.common.ControllerException;
import gov.va.med.ccht.ui.common.ControllerResult;
import gov.va.med.ccht.ui.model.AttachmentForm;
import gov.va.med.ccht.ui.model.QIRForm;
import gov.va.med.fw.model.EntityKey;
import gov.va.med.fw.model.EntityKeyFactory;
import gov.va.med.fw.service.ServiceException;
import gov.va.med.fw.service.ServiceOptimisticLockException;
import gov.va.med.fw.util.StringUtils;

/**
 * @author DNS   katikm
 *
 */
@Controller
public class QIRController extends AbstractController {

	@Autowired
	private QIRConversionService qirConversionService;

	public List<QIRSearchResult> searchQIR(QIRSearchParameters qirSearchParameters) throws Exception {
		List<Object[]> data = qirService.searchQIR(qirSearchParameters);

		List<QIRSearchResult> results = new ArrayList<QIRSearchResult>();
		for (Object[] objs : data) {

			QIRSearchResult record = new QIRSearchResult();
			record.id = String.valueOf(objs[0]);
			record.qirType = (String) objs[1];
			record.qirStatusType = (String) objs[2];
			record.visn = (String) objs[3];
			record.facilityCode = (String) objs[4];
			record.facility = (String) objs[5];
			record.deviceType = (String) objs[6];
			record.vendor = (String) objs[7];
			record.complaint = (String) objs[8];
			record.headline = (String) objs[9];
			record.submittedDate = (Date) objs[10];
			record.submittedBy = (String) objs[11];
			record.submittedByName = (String) objs[12];
			record.attachmentCount = ((Integer) objs[13]).toString();
			if ((Integer) objs[13] > 0) {
				record.hasAttachments = "true";
			}

			results.add(record);
		}
		return results;
	}

	public QIRForm getQIR(String id) throws Exception {
		EntityKey<QIR> entityKey = EntityKeyFactory.createEntityKey(new Long(id), QIR.class);
		QIR qir = qirService.getQIR(entityKey);
		QIRForm qirForm = new QIRForm();
		qirConversionService.convert(qir, qirForm);
		return qirForm;
	}

	public QIRForm getQIRById(String id) throws Exception {
		EntityKey<QIR> entityKey = EntityKeyFactory.createEntityKey(new Long(id), QIR.class);
		QIR qir = qirService.getQIR(entityKey);
		QIRForm qirForm = new QIRForm();
		qirConversionService.convert(qir, qirForm);
		return qirForm;
	}

	public List<QIRForm> getQIRs(List<QIRSearchResult> qirs) throws Exception {
		List<QIRForm> qirList = new ArrayList<QIRForm>();

		for (QIRSearchResult qir : qirs) {
			String id = qir.id;
			QIRForm newQirForm = getQIRById(id);

			String label = newQirForm.getQirStatusType().getLabel();

			boolean isNew = label.equalsIgnoreCase("New");
			boolean withDrawn = label.equalsIgnoreCase("Withdrawn");

			if (newQirForm.getVendorResponseDueDate() == null && !(isNew || withDrawn)) {
				Date vendorResponseDueDate = qirService.getDueDateFromHistory(id);
				newQirForm.setVendorResponseDueDate(vendorResponseDueDate);
			}

			qirList.add(newQirForm);
		}
		return qirList;
	}

	public void addQIR(QIRForm qirForm) throws Exception {
		QIR qir = new QIR();
		qirConversionService.convert(qirForm, qir);
		qirService.updateQIR(qir);
		addAttachments(qir, qirForm);
	}

	public ControllerResult updateQIR(QIRForm qirForm) throws Exception {
		return this.updateQIR(qirForm, false);
	}

	public ControllerResult updateQIR(QIRForm qirForm, boolean updatedDueDate) throws Exception {
		try {
			EntityKey<QIR> entityKey = EntityKeyFactory.createEntityKey(new Long(qirForm.getId()),
					new Integer(qirForm.getVersionNumber()), QIR.class);
			QIR qir = qirService.getQIR(entityKey);
			if (qir != null) {
				// copy QIR data excluding attachments
				qirConversionService.convert(qirForm, qir);
				qirService.updateQIR(qir);
				addAttachments(qir, qirForm);
				// removeAttachments(qir, qirForm);
				return new ControllerResult(ControllerResult.SUCCESS);
			} else {
				throw new ControllerException("QIR Not found");
			}
		} catch (ServiceOptimisticLockException ople) {
			return new ControllerResult(ControllerResult.DATA_CHANGED);
		}
	}

	private void addAttachments(QIR qir, QIRForm qirForm) throws ServiceException {
		// Add attachments
		List<AttachmentForm> attachments = qirForm.getAttachments();
		if (attachments == null)
			attachments = new ArrayList<AttachmentForm>();
		for (AttachmentForm attachment : attachments) {
			if (StringUtils.isEmpty(attachment.getId())) {
				QIRAttachment newAttachment = new QIRAttachment();
				newAttachment.setDate(attachment.getDate());
				newAttachment.setDocumentName(attachment.getName());
				newAttachment.setDocumentType(qirConversionService.getLookup(DocumentType.class, attachment.getType()));
				newAttachment.setLength(attachment.getLength());
				newAttachment.setData(attachment.getData());
				newAttachment.setQirId(qir.getId());
				qirService.updateQIRAttachment(newAttachment);
			}
		}
	}

	public void generateVendorResponseDueNotifications() throws Exception {
		qirService.generateVendorResponseDueNotifications();
	}
}
