package gov.va.med.ccht.controller;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import gov.va.med.ccht.model.inventory.Facility;
import gov.va.med.ccht.model.terminology.InactiveReason;
import gov.va.med.ccht.model.terminology.RegistrationDeniedReason;
import gov.va.med.ccht.model.terminology.RegistrationReason;
import gov.va.med.ccht.model.terminology.RegistrationStatus;
import gov.va.med.ccht.model.terminology.UserGroup;
import gov.va.med.ccht.service.common.TerminologyCache;

@Controller
public class TerminologyCacheController {

	@Autowired
	private TerminologyCache terminologyCache;

	public Map<String, String> getUserGroups() throws Exception {
		Map<String, String> userGroupMap = new HashMap<String, String>();
		List<UserGroup> userGroupList = (List<UserGroup>) terminologyCache.getTerminologyService()
				.findAll("gov.va.med.ccht.model.terminology.UserGroup");

		for (UserGroup userGroup : userGroupList) {
			userGroupMap.put(userGroup.getCode(), userGroup.getName());
		}
		return userGroupMap;
	}

	public Map<String, String> getJobDescriptions() throws Exception {
		Map<String, String> jobDescriptionMap = new HashMap<String, String>();
		List<RegistrationReason> jobDescriptionList = (List<RegistrationReason>) terminologyCache
				.getTerminologyService().findAll("gov.va.med.ccht.model.terminology.RegistrationReason");

		for (RegistrationReason jobDescription : jobDescriptionList) {
			jobDescriptionMap.put(jobDescription.getCode(), jobDescription.getName());
		}
		return jobDescriptionMap;
	}

	public List<InactiveReason> getInactivationReasons() throws Exception {
		Map<String, InactiveReason> inactiveReasonsMap = new HashMap<String, InactiveReason>();
		List<InactiveReason> inactiveReasonsList = (List<InactiveReason>) terminologyCache.getTerminologyService()
				.findAll("gov.va.med.ccht.model.terminology.InactiveReason");

		return inactiveReasonsList;
//		for (InactiveReason reason : inactiveReasonsList) {
//			inactiveReasonsMap.put(reason.getIdentifier().toString(), reason);
//		}
//		return inactiveReasonsMap;
	}

	public Map<String, RegistrationStatus> getRegistrationStatusLookUpMap() throws Exception {
		Map<String, RegistrationStatus> registrationStatusMap = new HashMap<String, RegistrationStatus>();
		List<RegistrationStatus> registrationStatusList = (List<RegistrationStatus>) terminologyCache
				.getTerminologyService().findAll("gov.va.med.ccht.model.terminology.RegistrationStatus");

		for (RegistrationStatus status : registrationStatusList) {
			registrationStatusMap.put(status.getCode(), status);
		}
		return registrationStatusMap;
	}

	public Map<Long, String> getRegistrationStatus(Map<Long, RegistrationStatus> arg) throws Exception {
		Map<Long, String> registrationStatusMap = new HashMap<Long, String>();

		Map<Long, RegistrationStatus> registrationStatusList = arg;

		for (Map.Entry<Long, RegistrationStatus> entry : arg.entrySet()) {
			RegistrationStatus status = entry.getValue();
			registrationStatusMap.put(status.getIdentifier(), status.getName());
		}
		return registrationStatusMap;
	}

	public Map<String, RegistrationDeniedReason> getRegistrationDeniedLookUpMap() throws Exception {
		Map<String, RegistrationDeniedReason> registrationDeniedReasonMap = new HashMap<String, RegistrationDeniedReason>();
		List<RegistrationDeniedReason> registrationStatusList = (List<RegistrationDeniedReason>) terminologyCache
				.getTerminologyService().findAll("gov.va.med.ccht.model.terminology.RegistrationDeniedReason");

		for (RegistrationDeniedReason status : registrationStatusList) {
			//registrationDeniedReasonMap.put(status.getIdentifier().toString(), status);
			registrationDeniedReasonMap.put(status.getCode(), status);
		}
		return registrationDeniedReasonMap;
	}

	public Map<Long, String> getRegistrationDeniedReasons(Map<Long, RegistrationDeniedReason> arg) throws Exception {
		Map<Long, String> registrationDeniedReasons = new HashMap<Long, String>();
		registrationDeniedReasons.put(new Long(-1), "Select");
		Map<Long, RegistrationDeniedReason> registrationStatusList = arg;

		for (Map.Entry<Long, RegistrationDeniedReason> entry : arg.entrySet()) {
			RegistrationDeniedReason status = entry.getValue();
			registrationDeniedReasons.put(status.getIdentifier(), status.getName());
		}
		return registrationDeniedReasons;
	}

	public Map<Long, String> getFacilities() throws Exception {
		Map<Long, String> facilities = new LinkedHashMap<Long, String>();
		@SuppressWarnings({ "unchecked" })
		SortedSet<Facility> facilityList = new TreeSet<>((List<Facility>) terminologyCache.getTerminologyService()
				.findAll("import gov.va.med.ccht.model.inventory.Facility"));

		for (Facility status : facilityList) {
			facilities.put(status.getId(), status.getName());
		}
		return facilities;
	}
}
