package gov.va.med.ccht.controller;

import static gov.va.med.ccht.model.qir.QIRStatusType.AGREED;
import static gov.va.med.ccht.model.qir.QIRStatusType.APPROVED;
import static gov.va.med.ccht.model.qir.QIRStatusType.NEW;
import static gov.va.med.ccht.model.qir.QIRStatusType.REPLIED;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpSession;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import gov.va.med.ccht.model.CCHTPermissions;
import gov.va.med.ccht.model.User;
import gov.va.med.ccht.model.inventory.SimpleFacility;
import gov.va.med.ccht.model.inventory.SimpleVisn;
import gov.va.med.ccht.model.inventory.Vendor;
import gov.va.med.ccht.model.qir.QIRSearchParameters;
import gov.va.med.ccht.model.qir.QIRSearchResult;
import gov.va.med.fw.security.UserPrincipal;
import gov.va.med.fw.ui.model.TermType;

@Controller
public class ViewMyQIRSController extends CommonCchtController {

	public static final String SESSION_ATTR_LAST_QIR_SEARCH_RESULT_ID_SET = "lastQIRSearchResultIDs";

	@RequestMapping(value = "/viewMyQirs.html", method = RequestMethod.GET)
	public String showViewMyQirs(Model model, HttpSession session) throws Exception {
		QIRSearchParameters qirSearchParameters = new QIRSearchParameters();
		UserPrincipal currentUser = getCurrentUser();
		User user = (User) getCurrentUser();
		String userId = currentUser.getUserCredentials().getUserID();

		if (currentUser.isPermissionGranted(CCHTPermissions.APPROVE_QIR)) {
			// national admin
			qirSearchParameters.setQirStatuses(Arrays.asList(NEW, APPROVED, REPLIED, AGREED));
		} else if (currentUser.isPermissionGranted(CCHTPermissions.REPLY_QIR)) {
			// vendor
			Vendor vendor = user.getVendor();
			if (vendor != null) {
				qirSearchParameters.setVendor(new TermType(vendor.getName(), vendor.getCode()));
				qirSearchParameters.setQirStatuses(Arrays.asList(APPROVED, REPLIED, AGREED));
			}
		} else if (currentUser.isPermissionGranted(CCHTPermissions.VISN_ADMIN)) {
			SimpleVisn visn = user.getVisn();
			if (visn != null) {
				TermType theVisn = new TermType();
				theVisn.setLabel(visn.getDescription());
				theVisn.setValue(visn.getName());
				qirSearchParameters.setVisn(theVisn);
			}
			qirSearchParameters.setSubmittedBy(userId);
			qirSearchParameters.setQirStatuses(Arrays.asList(NEW, APPROVED, REPLIED, AGREED));
		} else if (currentUser.isPermissionGranted(CCHTPermissions.FACILITY_ADMIN)) {
			SimpleFacility facility = user.getFacility();
			if (facility != null) {
				TermType theFacility = new TermType();
				theFacility.setLabel(facility.getName());
				theFacility.setValue(facility.getCode());
				qirSearchParameters.setFacility(theFacility);
			}
			qirSearchParameters.setSubmittedBy(userId);
			qirSearchParameters.setQirStatuses(Arrays.asList(NEW, APPROVED, REPLIED, AGREED));
		} else {
			qirSearchParameters.setQirStatuses(Arrays.asList(NEW, APPROVED, REPLIED, AGREED));
			qirSearchParameters.setSubmittedBy(userId);
		}
		qirSearchParameters.setMaxRecords(100);

		List<Object[]> myQirs = qirService.searchQIR(qirSearchParameters);

		Set<Long> uniqueIds = new HashSet<>();

		List<QIRSearchResult> results = new ArrayList<QIRSearchResult>();
		for (Object[] objs : myQirs) {
			SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");

			QIRSearchResult record = new QIRSearchResult();
			record.id = String.valueOf(objs[0]);
			uniqueIds.add(Long.valueOf(record.id));

			record.qirType = (String) objs[1];
			record.qirStatusType = (String) objs[2];
			record.visn = (String) objs[3];
			record.facilityCode = (String) objs[4];
			record.facility = (String) objs[5];
			record.deviceType = (String) objs[6];
			record.vendor = (String) objs[7];
			record.complaint = (String) objs[8];
			record.headline = (String) objs[9];
			record.submittedDate = (Date) objs[10];
			record.submittedDateAsString = dateFormat.format((Date) objs[10]);
			record.submittedBy = (String) objs[11];
			record.version = Short.valueOf((Short) objs[12]).toString();
			record.submittedByName = (String) objs[13];
			record.attachmentCount = ((Integer) objs[14]).toString();

			if ((Integer) objs[14] > 0) {
				record.hasAttachments = "true";
			}

			results.add(record);
		}

		session.setAttribute(SESSION_ATTR_LAST_QIR_SEARCH_RESULT_ID_SET, uniqueIds);

		model.addAttribute("qirList", results);

		return "viewMyQirs";
	}

}
