package gov.va.med.ccht.controller.validators;

import org.springframework.stereotype.Component;
import org.springframework.validation.BindingResult;
import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import gov.va.med.ccht.ui.model.QIRForm;
import gov.va.med.fw.util.StringUtils;

@Component
public class QIRFormValidator implements Validator {

	@Override
	public boolean supports(Class<?> arg0) {

		return QIRForm.class.isAssignableFrom(arg0);
	}

	@Override
	public void validate(Object target, Errors errors) {

		QIRForm form = (QIRForm) target;
		boolean notDirty = true;
		// Validate headline
		if (form.getHeadline() == null || form.getHeadline().length() == 0) {
			if (notDirty) {
				errors.reject("errorBlockTitle", "Error Message(s):");
				notDirty = false;
			}

			errors.rejectValue("headline", "NotEmpty.command.headline", "  - Headline is required.");
		}

		// Validate facility
		if (form.getSelectedFacilityId() == null || form.getSelectedFacilityId().length() == 0
				|| form.getSelectedFacilityId().equalsIgnoreCase("-1")) {
			if (notDirty) {
				errors.reject("errorBlockTitle", "Error Message(s):");
				notDirty = false;
			}
			errors.rejectValue("selectedFacilityId", "Min.command.selectedFacilityId", " - Facility is required.");
		}

		if (form.getSelectedDeviceId() < 1 && form.getSelectedVendorId() < 1) {
			if (notDirty) {
				errors.reject("errorBlockTitle", "Error Message(s):");
				notDirty = false;
			}

			errors.rejectValue("selectedVendorId", "Min.command.selectedVendorId", " - Vendor is required.");
		}

		if (form.getComplaint() == null || form.getComplaint().length() == 0) {
			if (notDirty) {
				errors.reject("errorBlockTitle", "Error Message(s):");
				notDirty = false;
			}
			errors.rejectValue("complaint", "NotEmpty.command.complaint", " - Cause of Complaint is required.");
		}

		if (form.getRemarks() == null || form.getRemarks().length() == 0) {
			if (notDirty) {
				errors.reject("errorBlockTitle", "Error Message(s):");
				notDirty = false;
			}
			errors.rejectValue("remarks", "NotEmpty.command.remarks", " - Recommendations and Remarks is required.");
		}

	}

	// @Override
	public void validateEditQir(QIRForm form, Errors errors) {
		boolean notDirty = true;

		// Validate is dmp issue selection

		if (form.getIsDmpIssue() != null && !form.getIsDmpIssue().equalsIgnoreCase("N")) {

			if (form.getIsDmpIssue().length() == 0 || form.getIsDmpIssue().equalsIgnoreCase("S")) {
				if (notDirty) {
					errors.reject("errorBlockTitle", "Error Message(s):");
					notDirty = false;
				}
				errors.rejectValue("isDmpIssue", "NotEmpty.command.isDmpIssue", "Please select DMP Issue.");

			}

			if (form.getIsDmpIssue().equalsIgnoreCase("Y") && form.getSelectedDmpQirTypeId().equalsIgnoreCase("-1")) {
				if (notDirty) {
					errors.reject("errorBlockTitle", "Error Message(s):");
					notDirty = false;
				}
				errors.rejectValue("dmpQirType", "NotEmpty.command.dmpQirType", "Please select DMP Issue Type.");

			}
		}

		// Validate complaints
		if (form.getComplaint() == null || form.getComplaint().length() == 0) {
			if (notDirty) {
				errors.reject("errorBlockTitle", "Error Message(s):");
				notDirty = false;
			}
			errors.rejectValue("complaint", "NotEmpty.command.complaint", " - Cause of Complaint is required.");
		}
	}

	public void validateApproveQir(QIRForm form, Errors errors) {
		boolean notDirty = true;
		String dmpIssue = form.getIsDmpIssue();
		if (!"Y".equals(dmpIssue) && !"N".equals(dmpIssue)) {
			if (notDirty) {
				errors.reject("errorBlockTitle", "Error Message(s):");
				notDirty = false;
			}
			errors.rejectValue("isDmpIssue", "", "Please select DMP Issue.");
		}

		if ("Y".equals(dmpIssue) && "-1".equals(form.getSelectedDmpQirTypeId())) {
			if (notDirty) {
				errors.reject("errorBlockTitle", "Error Message(s):");
				notDirty = false;
			}
			errors.rejectValue("selectedDmpQirTypeId", "", "Please select DMP Issue Type.");
		}
	}

	public void validateReplyQir(QIRForm form, Errors errors) {
		boolean notDirty = true;
		if (StringUtils.isBlank(form.getVendorActions())) {
			if (notDirty) {
				errors.reject("errorBlockTitle", "Error Message(s):");
				notDirty = false;
			}
			errors.rejectValue("vendorActions", "", "Vendor Actions is required.");
		}
	}

	public void validateAgreeQir(QIRForm command, BindingResult result) {
		// TODO Auto-generated method stub

	}

}
