package gov.va.med.ccht.ui.model;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import gov.va.med.ccht.model.AppUserGroup;
import gov.va.med.ccht.model.User;
import gov.va.med.ccht.model.UserRole;
import gov.va.med.ccht.model.inventory.SimpleFacility;
import gov.va.med.ccht.model.inventory.SimpleVisn;
import gov.va.med.ccht.model.inventory.Vendor;
import gov.va.med.ccht.model.terminology.InactiveReason;
import gov.va.med.ccht.model.terminology.RegistrationDeniedReason;
import gov.va.med.ccht.model.terminology.RegistrationReason;
import gov.va.med.ccht.model.terminology.RegistrationStatus;
import gov.va.med.fw.security.SimpleRole;
import gov.va.med.fw.ui.model.TermType;

public class UserRegistrationForm {

	//
	private String fullName;
	private String familyName;
	private String givenName;
	private String middleName;
	private String title;
	private String department;
	private String description;
	private String office;

	private User user;

	// User Name
	private String userName;

	// User ID

	// Email
	private String email;

	// Phone
	private String phone;

	// User Group
	private Map<Long, String> userGroups;
	private Long assignedUserGroup;

	// Registration Status
	private Map<Long, RegistrationStatus> registrationStatusLookUpMap;
	private Map<Long, String> registrationStatusMap;
	private Long assignedRegistrationStatusId;

	// Job Description
	private Map<Long, String> jobDescriptionMap;
	private Long assignedJobDescriptionId;

	// Assigned VISN
	private Map<Long, SimpleVisn> visnLookUp;
	private Map<Long, String> visns;
	private Long assignedVisnId;

	// Assigned Facility
	private Map<Long, SimpleFacility> facilityLookUp;
	private Map<Long, String> facilities;
	private Long facilitiesId;

	// Vendor
	private Map<Long, Vendor> vendorLookUp;
	private Map<Long, String> vendors;
	private Long selectedVendorId;

	// User Roles
	private Map<Long, SimpleRole> allRolesLookUpMap;
	private Map<Long, String> allRolesMap;
	private List<Long> allRoles;
	private List<Long> assignedRoles;

	// Locked
	private boolean locked;

	// Inactivate
	private boolean inactive;

	// Inactivate Date
	private String inactivationDateAsString;

	// Inactivate Reason
	private Map<Long, String> inactiveReasons;
	private Map<Long, InactiveReason> inactiveReasonsLookUpMap;
	private Long inactiveReasonId;

	// Registration Denied Reason
	private Map<Long, RegistrationDeniedReason> registrationDeniedReasonsLookUp;
	private Map<Long, String> registrationDeniedReasons;
	private Long registrationDeniedReasonId;

	public UserRegistrationForm(User user) {
		this.user = user;
		this.fullName = user.getFullName();
		this.givenName = user.getGivenName();
		this.familyName = user.getFamilyName();
		this.middleName = user.getMiddleName();
		this.title = user.getTitle();
		this.email = user.getEmail();
		this.phone = user.getTelephoneNumber();
		this.userName = user.getUsername();

		Set<AppUserGroup> temp = user.getUserGroups();

		Vendor assignedVendor = user.getVendor();
		if (assignedVendor != null) {
			this.selectedVendorId = assignedVendor.getId();
		}

		SimpleVisn assignedVisn = user.getVisn();
		if (assignedVisn != null) {
			this.assignedVisnId = assignedVisn.getId();
		}

		SimpleFacility facility = user.getFacility();
		if (facility != null) {
			this.facilitiesId = facility.getId();
		}

		InactiveReason inactiveReason = user.getInactiveReason();
		if (inactiveReason != null) {
			this.inactiveReasonId = inactiveReason.getIdentifier();
		}

		RegistrationReason reason = user.getRegistrationReason();
		if (reason != null) {
			this.assignedJobDescriptionId = reason.getIdentifier();
		}

		RegistrationStatus registrationStatus = user.getRegistrationStatus();
		if (registrationStatus != null) {
			this.assignedRegistrationStatusId = registrationStatus.getIdentifier();
		}
	}

	public Long getAssignedJobDescriptionId() {
		return assignedJobDescriptionId;
	}

	public Long getAssignedRegistrationStatusId() {
		return assignedRegistrationStatusId;
	}

	public Long getAssignedUserGroup() {
		return assignedUserGroup;
	}

	public Long getAssignedVisnId() {
		return assignedVisnId;
	}

	public String getEmail() {
		return email;
	}

	public Map<Long, String> getFacilities() {
		return facilities;
	}

	public Long getFacilitiesId() {
		return facilitiesId;
	}

	public Map<Long, SimpleFacility> getFacilityLookUp() {
		return facilityLookUp;
	}

	public String getFullName() {
		return fullName;
	}

	public String getInactivationDateAsString() {
		return inactivationDateAsString;
	}

	public Long getInactiveReasonId() {
		return inactiveReasonId;
	}

	public Map<Long, String> getInactiveReasons() {
		return inactiveReasons;
	}

	public Map<Long, String> getJobDescriptionMap() {
		return jobDescriptionMap;
	}

	public String getPhone() {
		return phone;
	}

	public Map<Long, String> getRegistrationStatusMap() {
		return registrationStatusMap;
	}

	public Long getSelectedVendorId() {
		return selectedVendorId;
	}

	public User getUser() {
		return user;
	}

	public Map<Long, String> getUserGroups() {
		return userGroups;
	}

	public String getUserName() {
		return userName;
	}

	public Map<Long, Vendor> getVendorLookUp() {
		return vendorLookUp;
	}

	public Map<Long, String> getVendors() {
		return vendors;
	}

	public Map<Long, SimpleVisn> getVisnLookUp() {
		return visnLookUp;
	}

	public Map<Long, String> getVisns() {
		return visns;
	}

	public boolean isInactive() {
		return inactive;
	}

	public boolean isLocked() {
		return locked;
	}

	public void setAssignedJobDescriptionId(Long assignedJobDescriptionId) {
		this.assignedJobDescriptionId = assignedJobDescriptionId;
	}

	public void setAssignedRegistrationStatusId(Long assignedRegistrationStatusId) {
		this.assignedRegistrationStatusId = assignedRegistrationStatusId;
	}

	public void setAssignedUserGroup(Long assignedUserGroup) {
		this.assignedUserGroup = assignedUserGroup;
	}

	public void setAssignedVisnId(Long assignedVisnId) {
		this.assignedVisnId = assignedVisnId;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public void setFacilities(Map<Long, String> facilities) {
		this.facilities = facilities;
	}

	public void setFacilitiesId(Long facilitiesId) {
		this.facilitiesId = facilitiesId;
	}

	public void setFacilityLookUp(Map<Long, SimpleFacility> facilityLookUp) {
		this.facilityLookUp = facilityLookUp;
	}

	public void setFullName(String fullName) {
		this.fullName = fullName;
	}

	public void setInactivationDateAsString(String inactivationDateAsString) {
		this.inactivationDateAsString = inactivationDateAsString;
	}

	public void setInactive(boolean inactive) {
		this.inactive = inactive;
	}

	public void setInactiveReasonId(Long inactiveReasonId) {
		this.inactiveReasonId = inactiveReasonId;
	}

	public void setInactiveReasons(Map<Long, String> inactiveReasons) {
		this.inactiveReasons = inactiveReasons;
	}

	public void setJobDescriptionMap(Map<Long, String> jobDescriptionMap) {
		this.jobDescriptionMap = jobDescriptionMap;
	}

	public void setLocked(boolean locked) {
		this.locked = locked;
	}

	public void setPhone(String phone) {
		this.phone = phone;
	}

	public void setRegistrationStatusMap(Map<Long, String> registrationStatusMap) {
		this.registrationStatusMap = registrationStatusMap;
	}

	public void setSelectedVendorId(Long selectedVendorId) {
		this.selectedVendorId = selectedVendorId;
	}

	public void setUser(User user) {
		this.user = user;
	}

	public void setUserGroups(Map<Long, String> userGroups) {
		this.userGroups = userGroups;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	public void setVendorLookUp(Map<Long, Vendor> vendorLookUp) {
		this.vendorLookUp = vendorLookUp;
	}

	public void setVendors(Map<Long, String> vendors) {
		this.vendors = vendors;
	}

	public void setVisnLookUp(Map<Long, SimpleVisn> visnLookUp) {
		this.visnLookUp = visnLookUp;
	}

	public void setVisns(Map<Long, String> visns) {
		this.visns = visns;
	}

	public Map<Long, SimpleRole> getAllRolesLookUpMap() {
		return allRolesLookUpMap;
	}

	public void setAllRolesLookUpMap(Map<Long, SimpleRole> allRoles) {
		this.allRolesLookUpMap = allRoles;
	}

	public Map<Long, String> getAllRolesMap() {
		return allRolesMap;
	}

	public void setAllRolesMap(Map<Long, String> allRolesMap) {
		this.allRolesMap = allRolesMap;
	}

	public List<Long> getAssignedRoles() {
		return assignedRoles;
	}

	public void setAssignedRoles(List<Long> assignedRoles) {
		this.assignedRoles = assignedRoles;
	}

	public List<Long> getAllRoles() {
		return allRoles;
	}

	public void setAllRoles(List<Long> allRoles) {
		this.allRoles = allRoles;
	}

	public String getFamilyName() {
		return familyName;
	}

	public void setFamilyName(String familyName) {
		this.familyName = familyName;
	}

	public String getGivenName() {
		return givenName;
	}

	public void setGivenName(String givenName) {
		this.givenName = givenName;
	}

	public String getMiddleName() {
		return middleName;
	}

	public void setMiddleName(String middleName) {
		this.middleName = middleName;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getDepartment() {
		return department;
	}

	public void setDepartment(String department) {
		this.department = department;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getOffice() {
		return office;
	}

	public void setOffice(String office) {
		this.office = office;
	}

	public Date getAccountExpireDate() {
		return user.getAccountExpireDate();
	}

	public RegistrationReason getRegistrationReason() {
		return user.getRegistrationReason();
	}

	public List<SimpleRole> getAssignedUserRoles() {

		List<SimpleRole> retVal = new LinkedList<SimpleRole>();

		if (assignedRoles != null) {
			for (Long id : assignedRoles) {
				SimpleRole userRole = allRolesLookUpMap.get(id);
				retVal.add(userRole);
			}
		}

		return retVal;
	}

	// public TermType getSelectedRegistrationStatus()
	// {
	// RegistrationStatus rs = new RegistrationStatus();
	// TermType retVal = new TermType();
	// rs = registrationStatusLookUpMap.get(assignedRegistrationStatusId);
	//
	// retVal.setValue(rs.getIdentifier().toString());
	// retVal.setLabel(rs.getName());
	//
	// return retVal;
	// }

	public RegistrationStatus getSelectedRegistrationStatus() {

		if (assignedRegistrationStatusId != null) {
			return registrationStatusLookUpMap.get(assignedRegistrationStatusId);
		} else {
			return null;
		}

	}

	public RegistrationDeniedReason getRegistrationDeniedReason() {
		RegistrationDeniedReason retVal = null;

		if (registrationDeniedReasonId != null) {
			return registrationDeniedReasonsLookUp.get(registrationDeniedReasonId);
		} else {
			return null;
		}
	}

	public InactiveReason getInactiveReason() {

		if (inactiveReasonId != null) {
			return inactiveReasonsLookUpMap.get(inactiveReasonId);
		} else {
			return null;
		}
	}

	public SimpleVisn getVisn() {

		if (assignedVisnId != null) {
			return visnLookUp.get(assignedVisnId);
		} else {
			return null;
		}
	}

	public SimpleFacility getFacility() {

		if (facilitiesId != null) {
			return facilityLookUp.get(facilitiesId);
		} else {
			return null;
		}
	}

	public Vendor getVendor() {
		if (selectedVendorId != null) {
			return vendorLookUp.get(selectedVendorId);
		} else {
			return null;
		}
	}

	public Date getInactiveDate() {
		return null;
	}

	public Map<Long, RegistrationStatus> getRegistrationStatusLookUpMap() {
		return registrationStatusLookUpMap;
	}

	public void setRegistrationStatusLookUpMap(Map<Long, RegistrationStatus> registrationStatusLookUpMap) {
		this.registrationStatusLookUpMap = registrationStatusLookUpMap;
	}

	public Map<Long, InactiveReason> getInactiveReasonsLookUpMap() {
		return inactiveReasonsLookUpMap;
	}

	public void setInactiveReasonsLookUpMap(Map<Long, InactiveReason> inactiveReasonsLookUpMap) {
		this.inactiveReasonsLookUpMap = inactiveReasonsLookUpMap;
	}

	public Map<Long, RegistrationDeniedReason> getRegistrationDeniedReasonsLookUp() {
		return registrationDeniedReasonsLookUp;
	}

	public void setRegistrationDeniedReasonsLookUp(
			Map<Long, RegistrationDeniedReason> registrationDeniedReasonsLookUp) {
		this.registrationDeniedReasonsLookUp = registrationDeniedReasonsLookUp;
	}

	public Map<Long, String> getRegistrationDeniedReasons() {
		return registrationDeniedReasons;
	}

	public void setRegistrationDeniedReasons(Map<Long, String> registrationDeniedReasons) {
		this.registrationDeniedReasons = registrationDeniedReasons;
	}

	public Long getRegistrationDeniedReasonId() {
		return registrationDeniedReasonId;
	}

	public void setRegistrationDeniedReasonId(Long registrationDeniedReasonId) {
		this.registrationDeniedReasonId = registrationDeniedReasonId;
	}

}
