package gov.va.med.fw.validators;


import java.util.ArrayList;
import java.util.List;

import gov.va.med.fw.model.ldap.LdapPerson;

public class DaoDataValidator {
	
	
	List<Character> basicWhitelist;
	String tempString;
	
	final int START_OF_ASCII = 32;
	final int END_OF_ASCII = 126;
	final int ASCII_AMP = 38;
	final int ASCII_LESS_THAN = 60;
	final int ASCII_GREATER_THAN = 62;
	
	public DaoDataValidator(){
		basicWhitelist = new ArrayList<Character>();
		
		// using decimal ascii values add values to whitelist.
		for(int i = START_OF_ASCII; i <= END_OF_ASCII; i++){
			if(i != ASCII_AMP || i != ASCII_LESS_THAN || i != ASCII_GREATER_THAN){
				basicWhitelist.add((char) i);
			}
		}	
	}

	// needs a more efficient rewrite
	public boolean validate(String toValidate) {

		List<Character> toValidateChars = new ArrayList<Character>();
		
		for (char c : toValidate.toCharArray()) {
			toValidateChars.add(c);
		}
		//make sure toValidate does not contain any chars not in the white list.
		return basicWhitelist.containsAll(toValidateChars); 
	}
	
	public boolean validateLdapPerson(LdapPerson ldapPerson){
		
		if(!validate(ldapPerson.getDn())){
			return false;
		}
		if(!validate(ldapPerson.getShortDn())){
			return false;
		}
		if(!validate(ldapPerson.getSamAccountName())){
			return false;
		}
		if(!validate(ldapPerson.getUserPrincipalName())){
			return false;
		}
		if(!validate(ldapPerson.getFullName())){
			return false;
		}
		if(!validate(ldapPerson.getDisplayName())){
			return false;
		}
		if(!validate(ldapPerson.getLastName())){
			return false;
		}
		if(!validate(ldapPerson.getFirstName())){
			return false;
		}
		if(!validate(ldapPerson.getMiddleName())){
			return false;
		}
		if(!validate(ldapPerson.getBadPasswordCount())){
			return false;
		}
		if(!validate(ldapPerson.getTitle())){
			return false;
		}
		if(!validate(ldapPerson.getDepartment())){
			return false;
		}
		if(!validate(ldapPerson.getDescription())){
			return false;
		}
		if(!validate(ldapPerson.getOffice())){
			return false;
		}
		if(!validate(ldapPerson.getTelephoneNumber())){
			return false;
		}
		if(!validate(ldapPerson.getEmail())){
			return false;
		}
		
		// these for loops are lists
		for(Object element : ldapPerson.getGroups())	
		{
			tempString = (String) element;
			if(!validate(tempString)){
				return false;
			}			
		}

		for(Object element : ldapPerson.getCissGroups())
		{
			tempString = (String) element;
			if(!validate(tempString)){
				return false;
			}			
		}
		for(Object element : ldapPerson.getNewGroups())
		{
			tempString = (String) element;
			if(!validate(tempString)){
				return false;
			}			
		}
		
		for(String element : ldapPerson.getDeletedGroups())
		{
			tempString = (String) element;
			if(!validate(tempString)){
				return false;
			}			
		}
		// if we do not return by this point ldapPerson is valid
		return true;
		
	}
}
